/*
***************************************************************************
**                                                                       **
**	Restart manager  -  Extension for Pale Moon                          **
**                                                                       **
**	Options window assistive code                                        **
**                                                                       **
**	Copyright(C) 2018 - 2020 by George Malamas                           **
**                                                                       **
**	Free software for all people                                         **
**                                                                       **
**	Last edit: July 10, 2020                                             **
**                                                                       **
**	Release number: 7                                                    **
**                                                                       **
***************************************************************************
*/
/*
This code adapts our options dialog to each app version, set the state of some 
elements at the moment of dialog loading and during user input controls the state 
of some dependent check boxes.
*/
// "use strict";

var aChkbx = window.document.getElementById("askCheckbox");
var appVersion = parseFloat(Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo).version);
if(appVersion < 20 && appVersion >= 3) { // Must be done at the beginning of loading!
 	aChkbx.setAttribute("preference", "ask2Pref");
};
var rChkbx = window.document.getElementById("restoreCheckbox");
var wChkbx = window.document.getElementById("warnOnCloseCheckbox");
var iconSelect = window.document.getElementById("iconSel");
var shortCutSet = window.document.getElementById("scSetUp-radios");
var vbxChkbxes = window.document.getElementById("4appVer20plus");
var warn2Chkbx = window.document.getElementById("warnOnClosePrvChkbox");
var invChkbx = window.document.getElementById("invCaches");
var hChkbx = window.document.getElementById("highlight");
var btnStylebox = window.document.getElementById("btnStyle-box");
var smallBtnSelBox = window.document.getElementById("smallBtnSel");
var smallBtnSelSep = window.document.getElementById("smallBtnSel-sep");
var restartCap = window.document.getElementById("restart-cap");
var exitCap = window.document.getElementById("exit-cap");
var sRestoreCap = window.document.getElementById("sRestore-cap");
var shortcutVisState = window.document.getElementById("shortcutVis-state");
var shortcutState = window.document.getElementById("shortcut-state");
var restartSCImg = window.document.getElementById("restmanNoShortcut");
var txtField = window.document.getElementById("shortcut-char");
var txtFieldCurrentValue ="R";
var txtBoxPref = "rShortcutCharPref";
var promptLabel = window.document.getElementById("scSetUp-label3");
var codeMonBox = window.document.getElementById("codeMon_hbox");
var scSetUpLabel1 = window.document.getElementById("scSetUp-label1");
var restmanRadioBtn3 = window.document.getElementById("restman-radioBtn3");
var mnrVbox = window.document.getElementById("mnr-vbox");
var clrMnrChkbox = window.document.getElementById("clrMnrChkbox");
var toProfList = window.document.getElementById("toWhichProf");
var restmanVersion = window.document.getElementById("restmanVersion");
var versionBox = window.document.getElementById("restmanVersionBox-bg");
var currentKMod = null;
var allShortcutsOfInterest = [];
var browserWin2 = null;
var winOpenerType2 = null;

function toggleRWState() { // Ask on restart.
	if(aChkbx.checked) {
		rChkbx.disabled = false;
		if(!rChkbx.checked) {
			rChkbx.click();
		};
	} else {
		if(rChkbx.checked) {
			rChkbx.click();
		};
		if(wChkbx.checked) {
			wChkbx.click();
		};
		rChkbx.disabled = true;
		wChkbx.disabled = true;
	};
};

function toggleWState() { // appVer >= 20.
	if(!rChkbx.checked) {
		wChkbx.disabled = false;
		if(!wChkbx.checked) {
			wChkbx.click();
		};
	} else {
		if(wChkbx.checked) {
			wChkbx.click();
		};
		wChkbx.disabled = true;
	};
};

function toggleShortcutSetUpState(aParam) {
	if(_RestMan.appInfos.appVer >= 3) {
		var shortcutSetUp = window.document.getElementsByClassName("shortcutSetUp");
		if(!shortcutState.checked) {
			Array.forEach(shortcutSetUp, function(aState) {
				if(aState.disabled) {
					aState.disabled = false;
				};
			});
			if(aParam == "notOnLoad") {
				_RestMan.applyUISettings("add+check"); // "checkAvailability()" on enabling.
			};
			if(window.document.getElementById("scTypeSel").value == 1) {
				var appExitScPref = window.document.getElementById("appExitScPref");
				if(_RestMan.appInfos.targetOS == "WINNT") {
					appExitScPref.valueFromPreferences = 1;
				} else if(_RestMan.appInfos.targetOS == "Linux") {
					appExitScPref.valueFromPreferences = 2;
				};
			};
		} else if(shortcutState.checked) {
			Array.forEach(shortcutSetUp, function(aState) {
				if(!aState.disabled) {
					aState.disabled = true;
				};
			});
			if(aParam == "notOnLoad") {
				_RestMan.applyUISettings();
			};
			if(window.document.getElementById("scTypeSel").value == 1) {
				window.document.getElementById("appExitScPref").valueFromPreferences = 0;
			};
		};
	} else {
		if(!shortcutState.checked) {
			window.document.getElementById("scSetUp-label1").disabled = false;
			shortCutSet.disabled = false;
			window.document.getElementById("scSetUp-label2").disabled = false;
			promptLabel.disabled = false;
			shortcutVisState.disabled = false;
			txtField.disabled = false;
			if(aParam == "notOnLoad") {
				_RestMan.applyUISettings("add+check"); // "checkAvailability()" on enabling.
			};
			if(window.document.getElementById("scTypeSel").value == 1) {
				var appExitScPref = window.document.getElementById("appExitScPref");
				if(_RestMan.appInfos.targetOS == "WINNT") {
					appExitScPref.valueFromPreferences = 1;
				} else if(_RestMan.appInfos.targetOS == "Linux") {
					appExitScPref.valueFromPreferences = 2;
				};
			};
		} else if(shortcutState.checked) {
			window.document.getElementById("scSetUp-label1").disabled = true;
			shortCutSet.disabled = true;
			window.document.getElementById("scSetUp-label2").disabled = true;
			promptLabel.disabled = true;
			txtField.disabled = true;
			shortcutVisState.disabled = true;
			if(aParam == "notOnLoad") {
				_RestMan.applyUISettings();
			};
			if(window.document.getElementById("scTypeSel").value == 1) {
				window.document.getElementById("appExitScPref").valueFromPreferences = 0;
			};
		};
	};
};

function toggleToProfListState(event) {
	toProfList.disabled = !event.target.checked;
};

function toggleHhighlightState() {
	if(invChkbx.checked) {
		hChkbx.disabled = false;
		if(!hChkbx.checked) {
			hChkbx.click();
		};
	} else {
		if(hChkbx.checked) {
			hChkbx.click();
		};
		hChkbx.disabled = true;
	};
};

function toggleScType(aValue) {// Shortcut type ("Restore Previous Session", only on appVer >= 4).
	if(aValue == 0) { // Restart
		(restartCap.hidden = false, exitCap.hidden = true, sRestoreCap.hidden = true, // Possible values 0/1/2.
		shortcutVisState.style.visibility = "visible",
		restartSCImg.hidden = false,
		txtBoxPref = "rShortcutCharPref",
		txtField.setAttribute("preference", txtBoxPref),
		txtField.value = _RestMan.locals.rShortcutChar,
		txtFieldCurrentValue = txtField.value,
		shortCutSet.setAttribute("preference", "rShortcutKeyPref"),
		shortCutSet.value = _RestMan.locals.rShortcutKeyMod == "alt" ? "alt-option" : "shift",
		shortcutState.setAttribute("preference", "rShortcutPref"),
		shortcutState.checked = !_RestMan.locals.restartShortcut,
		shortcutVisState.checked = !_RestMan.locals.restShortcutVis,
		currentKMod = shortCutSet.value);
	} else if(aValue == 1) { // Exit
		(restartCap.hidden = true, exitCap.hidden = false, sRestoreCap.hidden = true,
		shortcutVisState.style.visibility = "hidden",
		restartSCImg.hidden = true,
		txtBoxPref = "appExitScCharPref",
		txtField.setAttribute("preference", txtBoxPref),
		txtField.value = _RestMan.locals.appExitScChar,
		txtFieldCurrentValue = txtField.value,
		shortCutSet.setAttribute("preference", "appExitScKeyPref"),
		shortCutSet.value = _RestMan.locals.appExitScKeyMod == "alt" ? "alt-option" : "shift",
		shortcutState.removeAttribute("preference"),
		shortcutState.checked = (_RestMan.locals.appExitShortcut == 0 ||
								(_RestMan.locals.appExitShortcut == 1 && _RestMan.appInfos.targetOS !== "WINNT")),
		currentKMod = shortCutSet.value);
	} else if(aValue == 2) { // Restore Previous Session
		(restartCap.hidden = true, exitCap.hidden = true, sRestoreCap.hidden = false,
		shortcutVisState.style.visibility = "hidden",
		restartSCImg.hidden = true,
		txtBoxPref = "sesRestScCharPref",
		txtField.setAttribute("preference", txtBoxPref),
		txtField.value = _RestMan.locals.sesRestScChar,
		txtFieldCurrentValue = txtField.value,
		shortCutSet.setAttribute("preference", "sesRestScKeyPref"),
		shortCutSet.value = _RestMan.locals.sesRestScKeyMod == "shift" ? "shift" : "alt-option",
		shortcutState.setAttribute("preference", "sesRestScPref"),
		shortcutState.checked = !_RestMan.locals.sesRestShortcut,
		currentKMod = shortCutSet.value);
	};
	toggleShortcutSetUpState();
};

function selectScType(aButton) { // Previous/Next buttons.
	var scSelector = window.document.getElementById("scTypeSel");
	var currentValue = scSelector.value;
	var bypass = (_RestMan.appInfos.targetOS !== "WINNT" && _RestMan.appInfos.targetOS !== "Linux") ? true : false;
	switch(aButton) {
		case "next":
			var maxValue = _RestMan.appInfos.appVer >= 4 ? 2 : 1;
			if(currentValue < maxValue && !bypass) {
				scSelector.childNodes[currentValue].nextSibling.click();
			} else if(currentValue == maxValue) {
				scSelector.firstChild.click();
			} else if(currentValue == 0 && bypass) {
				scSelector.lastChild.click();
			};
			break;
		case "prev":
			if(currentValue > 0 && !bypass) {
				scSelector.childNodes[currentValue].previousSibling.click();
			} else if(currentValue == 0) {
				scSelector.lastChild.click();
			} else if(currentValue == 2 && bypass) {
				scSelector.firstChild.click();
			};
			break;
	};
};

function checkGetShortcuts() { // We are interested only for keyboard shortcuts of the same type as ours, with two
	// key modifiers and key letter, as follows: Ctrl + [Shift or Alt] + key letter.
	// They are stored in "allShortcutsOfInterest" as long as our pref window is open, to check them against user's
	// input, on enabling/configuring of our shortcuts. (appVer >= 4 for "Restore Previous Session" shortcut.)
	var keysets = browserWin2.document.getElementsByTagName("keyset");
	for(var ksn = 0; ksn < keysets.length; ksn++) {
		if(keysets[ksn].hasChildNodes()) {
			var keys = keysets[ksn].getElementsByTagName("key");
			for(var kn = 0; kn < keys.length; kn++) {
				if(keys[kn].hasAttribute("modifiers") && keys[kn].hasAttribute("key")) {
					var modifiers = keys[kn].getAttribute("modifiers").replace(/ |,/g,"").toLowerCase();
					if((modifiers == "accelshift" || modifiers == "accelalt" || modifiers == "controlshift"
						|| modifiers == "controlalt") && (keys[kn].getAttribute("key").toString().length == 1)) {
						allShortcutsOfInterest.push(keys[kn].cloneNode(false));
					};
				};
			};
		};
	};
	allShortcutsOfInterest.forEach(function(akey) {
		akey.setAttribute("modifiers", akey.getAttribute("modifiers").replace(/ |,/g,"").toLowerCase());
		akey.setAttribute("key", akey.getAttribute("key").toLowerCase());
		if(!akey.hasAttribute("label") || akey.getAttribute("label").replace(/\s+/g,"").length == 0) {
			var mItems = browserWin2.document.getElementsByAttribute("key", akey.id);
			for(var i = 0; i < mItems.length; i++) {
				if(mItems[i].hasAttribute("label") && mItems[i].getAttribute("label").replace(/\s+/g,"").length > 0) {
					akey.setAttribute("label", mItems[i].getAttribute("label"));
				};
			};
		};
	});
	currentKMod = shortCutSet.value;
};

function checkInput() { // We want to allow only letter character of any language. It's not
// easily possible ("\w" matches only latin characters). So, we exclude the numbers
// and some commonly used keyboard symbols.
	if(txtField.value == "" ||
		(/[\d!\u00a1@#$\u00a7%^&*()\\\/|\{\}\[\]\-_+=,.?]/).test(txtField.value) ||
		(/[\u00bf;:'\u0384"~`\u2022<>\u00ab\u00bb\u00a6\u00ac]/).test(txtField.value) ||
		(/[\u20a4\u20a3\u00a5\u00a4\u00a3\u00a2\u20ac\u00b7\t ]/).test(txtField.value)) {
		var blinks = 0;
		function blinkLabelRestoreValue(aState) {
			if(blinks < 5 && aState == "set") {
				promptLabel.style.color = "red";
				blinks++;
				window.setTimeout(function() {
					blinkLabelRestoreValue("reset");
				}, 250);
			};
			if(aState == "reset") {
				promptLabel.style.removeProperty("color");
				if(blinks == 5) {
					txtField.value = txtFieldCurrentValue;
					txtField.select();
					return;
				};
				window.setTimeout(function() {
					blinkLabelRestoreValue("set");
				}, 250);
			};
		};
		blinkLabelRestoreValue("set");
		return false;
	};
	if(txtField.value.toLocaleUpperCase() == txtFieldCurrentValue) {
		txtField.value = txtFieldCurrentValue; // Makes it upper case.
		window.setTimeout(function() {
			txtField.select();
		}, 1500);
		return false;
	};
	window.setTimeout(function() {
		txtField.select();
	}, 2500);
	return true;
};

function checkAvailability(aParam) { // On enabling/changing of our shortcuts.
	if(currentKMod == aParam && txtFieldCurrentValue == txtField.value.toLocaleUpperCase()) {
		return;
	};
	if(aParam && aParam !== "add+check") {
		currentKMod = aParam;
	};
	if(aParam == "add+check") {
		allShortcutsOfInterest = [];
		checkGetShortcuts();
	};
	var key2check = browserWin2.document.getElementById("keyRestmanRestart_KeyCombo");
	var shortcutType = window.document.getElementById("scTypeSel").value;
	if(shortcutType == 0) {
		key2check = browserWin2.document.getElementById("keyRestmanRestart_KeyCombo"); // Restart
	} else if(shortcutType == 1) {
		key2check = browserWin2.document.getElementById("keyRestmanExit_KeyCombo"); // Exit
	} else if(_RestMan.appInfos.appVer >= 4 && shortcutType == 2) {
		key2check = browserWin2.document.getElementById("keyRestmanRPS_KeyCombo"); // Restore Previous Session
	};
	var keys2exclude = []; // Our disabled shortcuts, if any.
	if(!_RestMan.rmPrefBranch.getBoolPref("restart_shortcut")) {
		keys2exclude.push("keyRestmanRestart_KeyCombo");
	};
	if(_RestMan.appInfos.appVer >= 4 && _RestMan.rmPrefBranch.getIntPref("exit_shortcut") == 0) {
		keys2exclude.push("keyRestmanExit_KeyCombo");
	};
	if(!_RestMan.rmPrefBranch.getBoolPref("sRestore_shortcut")) {
		keys2exclude.push("keyRestmanRPS_KeyCombo");
	};
	var nkMod = key2check.getAttribute("modifiers").replace(/accel,/,"").toLowerCase();
	var nk = key2check.getAttribute("key").toLowerCase();
	var duplicates = [];
	function chekKeys2exclude(aKey) { // Alternative to "Array.includes()", which is unsupported by older app versions.
		for(var k = 0; k < keys2exclude.length; k++) {
			if(aKey == keys2exclude[k]) {
				return true;
			};
		};
		return false;
	};
	for(var kn = 0; kn < allShortcutsOfInterest.length; kn++) {
		if(allShortcutsOfInterest[kn].id == key2check.id) {
			allShortcutsOfInterest[kn].setAttribute("modifiers", ("accel" + nkMod));
			allShortcutsOfInterest[kn].setAttribute("key", nk);
		} else if((allShortcutsOfInterest[kn].getAttribute("modifiers").replace(/accel|control/g,"") == nkMod) &&
			(allShortcutsOfInterest[kn].getAttribute("key") == nk) &&
			!chekKeys2exclude(allShortcutsOfInterest[kn].id)) {
			var dup = allShortcutsOfInterest[kn].cloneNode(false);
			duplicates.push(dup);
		};
	};
	key2check = null;
	if(duplicates.length > 0) {
		var strings = window.document.getElementById("rm_strset");
		var strings_fbk = window.document.getElementById("rm_strset_fbk"); // Fallback for missing strings of incomplete locales, to get them from "en-US".
		var msgTitle = "";
		try {msgTitle = strings.getString("shortcutWarning.title")} catch(ex) {msgTitle = strings_fbk.getString("shortcutWarning.title")};
		var msgDesc = "";
		try {msgDesc = strings.getString("shortcutWarning.msgdesc") + "\n"} catch(ex) {msgDesc = strings_fbk.getString("shortcutWarning.msgdesc") + "\n"};
		var anon = "";
		try {anon = strings.getString("shortcutWarning.anon")} catch(ex) {anon = strings_fbk.getString("shortcutWarning.anon")};
		var msgPrompt = "";
		try {msgPrompt = "\n" + strings.getString("shortcutWarning.msgprompt")} catch(ex) {msgPrompt = "\n" + strings_fbk.getString("shortcutWarning.msgprompt")};
		var msg = [];
		for(var nd = 0; nd < duplicates.length; nd++) {
			var msgLines = null;
			if(duplicates[nd].hasAttribute("label")) {
				msgLines = duplicates[nd].getAttribute("label");
			} else {
				msgLines = anon; // Anonymous.
			};
			if(duplicates[nd].hasAttribute("id")) {
				msgLines = "[ " + "\""+ msgLines +"\"" + " - key id: " + "\""+ duplicates[nd].id +"\"" + " ]";
			} else if(duplicates[nd].hasAttribute("command")) {
				msgLines = "[ " + "\""+ msgLines +"\"" + " - key id: " + "\" \"" + " - command: " + "\""+
							duplicates[nd].getAttribute("command") +"\"" + " ]";
			} else if(duplicates[nd].hasAttribute("oncommand")) {
				msgLines = "[ " + "\""+ msgLines +"\"" + " - key id: " + "\" \"" + " - oncommand: " + "\""+
							duplicates[nd].getAttribute("oncommand") +"\"" + " ]";
			};
			msg.push(msgLines);
		};
		window.setTimeout(function() {
			_RestMan.promptSrvc.alert(null, msgTitle, msgDesc + msg.join("\n") + "\n" + msgPrompt);
			duplicates = [];
			txtField.focus();
		}, 250);
	};
};

function setOnLoadState() {
	if(!browserWin2) {
		return;
	};
	txtFieldCurrentValue = txtField.value;
	var acceptButton = window.document.documentElement.getButton("accept");
	// Some app versions ignore the attribute "buttons" in "prefwindow" type dialogs,
	// for some type of buttons, e.g. "accept", despite "browser.preferences.instantApply = false".
	// We need the "accept" button always, even in Linux/Mac.
	if(acceptButton.hidden) {
		acceptButton.removeAttribute("hidden");
	};
	if(acceptButton.disabled) { // Mac OS.
		acceptButton.removeAttribute("disabled");
	};
	if(scSetUpLabel1.getAttribute("chromedir") == "rtl") {
		scSetUpLabel1.value = "+ Cmd\\Ctrl";
		restmanRadioBtn3.label = "Option\\Alt";
		restmanRadioBtn3.parentNode.insertBefore(restmanRadioBtn3, restmanRadioBtn3.nextSibling);
		if(_RestMan.appInfos.appVer >= 3) { // AppVer 3+ put in front of the text the last "/", in RTL locales.
			restmanVersion.setAttribute("tooltiptext", " /https://addons.palemoon.org/addon/restart-manager ");
		};
	};
	if(_RestMan.locals.codMonChkBoxHid) {
		codeMonBox.hidden = true;
	};
	if(invChkbx.checked) {
		hChkbx.disabled = false;
	} else {
		hChkbx.disabled = true;
	};
	if(_RestMan.appInfos.appVer >= 20) {
		if(aChkbx.checked) {
			rChkbx.disabled = false;
			if(rChkbx.disabled || rChkbx.checked) {
				wChkbx.disabled = true;
			};
		} else {
			rChkbx.disabled = true;
			wChkbx.disabled = true;
		};
		var widthValue = window.getComputedStyle(wChkbx).getPropertyValue("width");
		warn2Chkbx.style.setProperty("width", widthValue);
		warn2Chkbx.removeAttribute("hidden");
		if(_RestMan.locals.codeMonitoring) {
			window.setTimeout(function() {
				alert("Computed width of third checkbox (Warn me...)\n+ its label:"
					+ "\n\n" + widthValue);
			}, 600);
		};
	} else {
		aChkbx.removeAttribute("oncommand");
		vbxChkbxes.parentNode.removeChild(vbxChkbxes);
	};
	var shortcutTypeSel = window.document.getElementById("scTypeSel");
	if(_RestMan.appInfos.appVer < 4) {
		btnStylebox.parentNode.removeChild(btnStylebox);
		smallBtnSelSep.parentNode.removeChild(smallBtnSelSep);
		smallBtnSelBox.parentNode.removeChild(smallBtnSelBox);
		shortcutTypeSel.removeChild(shortcutTypeSel.lastChild);
		var srCap = window.document.getElementById("sRestore-cap");
		srCap.parentNode.removeChild(srCap);
		if(_RestMan.appInfos.appVer < 3) {
			shortcutTypeSel.childNodes[shortcutTypeSel.value].setAttribute("selected", true);
		};
	} else {
		if(!_RestMan.restmanSrvc.getSet("allowSpecials", "getV")) { // get value.
			invChkbx.disabled = true;
		};
	};
	if(_RestMan.appInfos.targetOS !== "WINNT" && _RestMan.appInfos.targetOS !== "Linux") {
		shortcutTypeSel.childNodes[1].hidden = true;
	};
	if(_RestMan.appInfos.appVer >= 3) {
		var shortcutSetUp = window.document.getElementsByClassName("shortcutSetUp");
		if(shortcutState.checked) {
			Array.forEach(shortcutSetUp, function(aState) {
				aState.disabled = true;
			});
		};
		window.document.addEventListener("focus", function setRelativeWidth() {
			this.removeEventListener("focus", setRelativeWidth, false);
			var contentWidth = window.document.documentElement.clientWidth;
			versionBox.style.width = (contentWidth * .62) + "px";
		}, false);
		if(_RestMan.appInfos.appVer >= 3.5) {
			var sel = window.document.querySelectorAll("groupbox,.gbCap,.b-border-box,#mnr-vbox,#toWhichProf,#kbscCaptionBox,#scSelectBox");
			Array.forEach(sel, function(s) {
				s.setAttribute("os", _RestMan.appInfos.targetOS);
			});
			toProfList.disabled = !clrMnrChkbox.checked;
		};
	} else {
		if(shortcutState.checked) {
			window.document.getElementById("scSetUp-label1").disabled = true;
			shortCutSet.disabled = true;
			window.document.getElementById("scSetUp-label2").disabled = true;
			window.document.getElementById("scSetUp-label3").disabled = true;
			txtField.disabled = true;
			shortcutVisState.disabled = true;
		};
		var gbs = window.document.documentElement.getElementsByTagName("groupbox");
		Array.forEach(gbs, function(gb) {
			gb.setAttribute("os", _RestMan.appInfos.targetOS);
		});
		var caps = window.document.documentElement.getElementsByTagName("caption");
		Array.forEach(caps, function(cap) {
			if(cap.hasAttribute("class")) {
				cap.setAttribute("os", _RestMan.appInfos.targetOS);
			};
		});
		["kbscCaptionBox","scSelectBox"].forEach(function(id) {
			window.document.getElementById(id).setAttribute("os", _RestMan.appInfos.targetOS);
		});
	};
	if(_RestMan.appInfos.appVer < 3.5) {
		iconSelect.setAttribute("style", "-moz-margin-start: 0.25em;");
		mnrVbox.parentNode.removeChild(mnrVbox);
	};
	if(_RestMan.appInfos.appVer > 3 && _RestMan.appInfos.targetOS !== "WINNT") {
		iconSelect.setAttribute("style", "-moz-margin-start: -0.035em;");
	};
	if(_RestMan.appInfos.targetOS == "Linux" || _RestMan.appInfos.targetOS == "Darwin") {
		window.document.getElementById("prevCtrl").setAttribute("LinMac", "");
		window.document.getElementById("nextCtrl").setAttribute("LinMac", "");
	};
	if(_RestMan.appInfos.appVer < 3.6) { // Prevent default style to colorize red the labels on focus/active. AppVer
	// 3.5+ uses "-moz-nativehyperlinktext" as default display color for "text-link" class, which is unsupported
	// by appVer 2. With this method we apply always (inline) the default color for "text-link" labels in each
	// appVer/platform, avoiding extra style sheets.
		var rmv = window.document.getElementById("restmanVersion");
		var rmvColor = window.getComputedStyle(rmv, null).getPropertyValue("color");
		rmv.style.color = rmvColor;
		window.document.getElementById("supportEmail").style.color = rmvColor;
	};
	var value = window.document.getElementById("scTypeSel").value;
	toggleScType(value);
	window.setTimeout(function() {
		checkGetShortcuts();
	}, 750);
	if(_RestMan.appInfos.appVer >= 4) {
		Array.forEach(window.document.getElementsByClassName("ttLbl"), function(elem) {
			elem.removeAttribute("hidden");
		});
	} else {
		var infoImg2 = window.document.getElementById("infoImg2");
		infoImg2.parentNode.removeChild(infoImg2); // "node.remove()", appVer 18+.
	};
	var devsCap = window.document.getElementById("devsCap");
	devsCap.label = devsCap.label.replace(/:|：/,"");
};

function openHostPage(hostPageUrl) {
	function pageLoader(win) {
		if(win.gBrowser.mCurrentTab.linkedBrowser.currentURI.spec == "about:blank") {
			win.gBrowser.loadURI(hostPageUrl);
		} else {
			win.gBrowser.selectedTab = win.gBrowser.addTab(hostPageUrl);
		};
	};
	if(winOpenerType2 == "navigator:browser") {
		pageLoader(window.opener);
	} else if(winOpenerType2 == "Addons:Manager" || winOpenerType2 == "Extension:Manager") {
		if((_RestMan.appInfos.appVer >= 15 || _RestMan.appInfos.appVer < 3) && ("openURL" in window.opener)) {
			window.opener.openURL(hostPageUrl);
		} else { // "openURL()" (contentAreaUtils.js) is buggy in excluded appVer above.
			pageLoader(browserWin2);
		};
	} else { // No suitable "window.opener".
		pageLoader(browserWin2);
	};
};

function sendMail(address) {
	if(_RestMan.appInfos.appVer > 2) {
		if(winOpenerType2 && ("openURL" in window.opener)) {
			window.opener.openURL(address);
		} else {
			browserWin2.openURL(address);
		};
	} else {
		browserWin2.loadURI(address, null, null, null);
	};
};
