/*
***************************************************************************
**                                                                       **
**	Restart manager  -  Extension for Pale Moon                          **
**                                                                       **
**	Desktop update notification                                          **
**                                                                       **
**	Copyright(C) 2020 by George Malamas                                  **
**                                                                       **
**	Free software for all people                                         **
**                                                                       **
**	Last edit: July 1, 2020                                              **
**                                                                       **
**	Release number: 3                                                    **
**                                                                       **
***************************************************************************
*/
// This is our desktop notification we show once after updates.
// It is implemented on top of standard browser's built-in desktop
// notification api, to apply some style and functional tweaks.

const {classes: Cc, interfaces: Ci} = Components;

//var cs = Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService); // cs.logStringMessage();
// To test the notification:
// 1.  - Set a smaller value in our pref "version" in about:config, or reset it.
// 2a. - Restart the browser or in app v2 open a new browser window.
// Or:
// 2b. - Run "_RestMan.restmanSrvc.getSet("versionChecked", "setF");" in Scratchpad or Browser Console and
//       then open a new browser window (appVer 20+).

function updateNotification() {
	window.setTimeout(window.blur, 0);
	var appVer = window.opener._RestMan.appInfos.appVer;
	var appId = window.opener._RestMan.appInfos.appId;
	var targetOS = window.opener._RestMan.appInfos.targetOS;
	var prefSrvc = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).QueryInterface(Ci.nsIPrefBranch);
	var obsSrvc = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
	var alertsSrvc = Cc["@mozilla.org/alerts-service;1"].getService(Ci.nsIAlertsService);
	var nTitle = window.document.getElementById("softUpdTitle").value;
	var nBody1 = window.opener._RestMan.locals.restmanName + " " + window.opener._RestMan.locals.restmanVer;
	var nBody2 = null;
	if(window.opener.document.getElementById("releaseNotes")) { // appVer 2-3.6, 28.6+
		nBody2 = window.opener.document.getElementById("releaseNotes").label;
	} else {
		nBody2 = window.document.getElementById("relNotesLbl").value.replace(/:|：/,"");
	};
	var nImg = "chrome://restartmanager/skin/restman32_blue.png";
	var nIcon = "chrome://branding/content/icon16.png";
	var hostPageUrl = "https://addons.palemoon.org/addon/restart-manager/";
	var nLink = window.document.createElement("label");
	nLink.setAttribute("value", nBody2);
	nLink.setAttribute("class", "text-link");
	nLink.setAttribute("tooltiptext", hostPageUrl);
	if(appVer > 2) {
		nLink.setAttribute("onclick", "openHostPage();");
	};
	if(appVer >= 27) {
		nLink.style.marginInlineStart = 0;
	} else {
		nLink.style.MozMarginStart = 0;
	};
	var nSpacer = window.document.createElement("spacer")
	nSpacer.setAttribute("flex", 1);
	var nCntDisp = window.document.createElement("label");
	nCntDisp.setAttribute("id", "nCntDispId");
	nCntDisp.setAttribute("style", "color: graytext; opacity: .8;");
	nCntDisp.setAttribute("value", 40);
	var nLinkBox = window.document.createElement("hbox");
	nLinkBox.appendChild(nLink);
	nLinkBox.appendChild(nSpacer);
	if((appVer >= 28 && appId == "{8de7fcbb-c55c-4fbe-bfc5-fc555c87dbc4}") || // Pale Moon's v25+ id.
		appVer < 22) {
		nLinkBox.appendChild(nCntDisp);
		if(appVer < 19) {
			nCntDisp.style.MozMarginEnd = 0;
		};
	};
	var nSepar = window.document.createElement("separator")
	nSepar.setAttribute("class", "thin");
	var nComp = null;
	function openHostPage() {
		if(!nComp) {
			nComp = window.Components;
		};
		var wm = nComp.classes["@mozilla.org/appshell/window-mediator;1"].getService(nComp.interfaces.nsIWindowMediator);
		var brWin = wm.getMostRecentWindow("navigator:browser");
		function pageLoader() {
			if(brWin.gBrowser.mCurrentTab.linkedBrowser.currentURI.spec == "about:blank") {
				brWin.gBrowser.loadURI(hostPageUrl);
			} else {
				brWin.gBrowser.selectedTab = brWin.gBrowser.addTab(hostPageUrl);
			};
		};
		if(brWin) {
			brWin.focus();
			pageLoader();
		} else {
			brWin = window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no");
			brWin.addEventListener("load", function onLoad() {
				brWin.removeEventListener("load", onLoad, false);
				brWin.setTimeout(function() {
					pageLoader();
				}, 1000);
			}, false);
		};
	};
	var winOpenObs = {
		observe: function(aSubject, aTopic, aData) {
			if((aSubject instanceof ChromeWindow) && (aTopic == "domwindowopened")) {
				aSubject.addEventListener("load", function cust(event) { // Try to catch and customize our
				// notification with unsupported by the alerts service features..
					this.removeEventListener("load", cust, false);
					var nWin = event.currentTarget;

					// Be sure this is the right target, our notification.
					if((nWin.document.documentElement.getAttribute("windowtype") == "alert:alert") &&
						((nWin.document.getElementById("alertTextLabel").textContent == nBody1) || // AppVer 19+.
						(nWin.document.getElementById("alertTextLabel").value == nBody1))) { // AppVer < 19.
						nWin.addEventListener("unload", function closeOpener() {
							nWin.removeEventListener("unload", closeOpener, false);
							window.close();
						}, false);
						var nBox = nWin.document.getElementById("alertBox");
						var nTxtBox = nWin.document.getElementById("alertTextBox");
						var elem1 = nWin.document.getElementById("alertTextLabel");
						elem1.style.cursor = "default"; // Required in appVer 19 - 27.
						nTxtBox.insertBefore(nLinkBox, elem1.nextSibling);
						nTxtBox.insertBefore(nSepar, elem1.nextSibling);
						if(appVer >= 28 && appId == "{8de7fcbb-c55c-4fbe-bfc5-fc555c87dbc4}") { // Pale Moon's v25+ id.
							var nTitleIcon = nWin.document.getElementById("alertIcon");
							var nWinImg = nWin.document.getElementById("alertImage");
							nBox.setAttribute("hasIcon", true);
							nTitleIcon.setAttribute("src", nIcon);
							nWin.document.getElementById("alertTitleBox").style.backgroundColor = "rgba(255, 170, 0, .25)";
							nTxtBox.parentNode.style.backgroundColor = "rgba(135, 206, 235, .25)";
							nWinImg.style.width = "60px";
							nWinImg.style.height = "60px";
							nWinImg.style.margin = "17px";
							nWinImg.style.marginTop = "10px";
							nWinImg.style.borderRadius = "10%";
							nWinImg.style.backgroundColor = "rgba(255, 255, 255, .35)";
						} else if((appVer == 28 && appId == "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}") || // Firerox.
							(appVer < 28 && appVer > 21)) { // Pale Moon 24 - 27, Firefox 22 - 27.
							var nXBox = nWin.document.querySelector(".alertCloseBox");
							nXBox.firstChild.orient = "vertical";
							nXBox.setAttribute("align", "end");
							nTxtBox.style.width = "234px";
							var nCntDispBox = nWin.document.createElement("hbox");
							var nSpacer2 = window.document.createElement("spacer")
							nSpacer2.setAttribute("flex", 1);
							var nSpacer3 = window.document.createElement("spacer")
							nSpacer3.setAttribute("flex", 1);
							nCntDispBox.appendChild(nSpacer3);
							nCntDispBox.appendChild(nCntDisp);
							nXBox.appendChild(nSpacer2);
							nXBox.appendChild(nCntDispBox);
							nCntDispBox.style.marginBottom = "11px";
							nCntDisp.setAttribute("value", 99999);
							nCntDispBox.style.width = nCntDispBox.clientWidth + "px";
							nCntDisp.setAttribute("value", 40);
							nBox.style.backgroundColor = "rgba(135, 206, 235, .6)";
						} else if(appVer < 22) {
							nBox.style.backgroundColor = "rgb(210, 237, 248)";
						};
						nComp = nWin.Components;
						nWin.openHostPage = openHostPage;
						if(appVer >= 19) {
							nBox.style.animationDelay = "1s";
							nBox.addEventListener("animationstart", function removeProp() {
								this.removeEventListener("animationstart", removeProp, false);
								nWin.setTimeout(function() {
									nBox.style.removeProperty("animation-delay");
								}, 3500);
							}, false);
							var currentVal = prefSrvc.getBoolPref("alerts.disableSlidingEffect");
							prefSrvc.setBoolPref("alerts.disableSlidingEffect", false);
							nWin.setTimeout(function() {
								nBox.style.setProperty("animation-duration", "44s", "important");
								prefSrvc.setBoolPref("alerts.disableSlidingEffect", currentVal);
							}, 750);
						} else {
							var currentVal = prefSrvc.getIntPref("alerts.totalOpenTime");
							prefSrvc.setIntPref("alerts.totalOpenTime", 40000);
							nWin.setTimeout(function() { // Restore the predefined value.
								prefSrvc.setIntPref("alerts.totalOpenTime", currentVal);
							}, 500);
						};
						if(appVer < 3.5) { // In these appVer, notifications cannot be closed by the user.
							nWin.onclick = nWin.closeAlert; // alert.js
							if(appVer < 3) {
								nLinkBox.onclick = nWin.openHostPage;
							};
						};
						nWin.addEventListener("click", function(click_event) {
						// Close notification/open link, only with left mouse button.
							if(click_event.button !== 0) {
								click_event.stopPropagation();
							};
						}, true);
						var nBoxBounds = null;
						(function nCounter(cnt, iter, av) {
							!cnt ? cnt = 40 : cnt = cnt; // seconds.
							!iter ? iter = 0 : iter = iter; // iterations.
							var nCnt = nWin.setInterval(function() {
								if(++iter == 10) {
									if(av >= 19) {
										// Take care of our rule been bypassed by a more prioritized one (e.g. in userChrome.css).
										var andu = Math.floor(parseFloat(nWin.getComputedStyle(nBox, null).animationDuration));
										if(Number.isInteger(andu)) {
											cnt = Math.ceil((andu - Math.sqrt(andu * .25)) - 1);
										};
										av = undefined; // Define "cnt" only once, after 1 sec (10 timeouts of
									};					// "setInterval") of first invocation of "nCounter".
									iter = 0;
									if(cnt == 0) {
										nWin.clearInterval(nCnt);
									};
									return nCntDisp.setAttribute("value", cnt--);
								};
							}, 100);
							function pause() {
								this.removeEventListener("mouseover", pause, false);
								if((targetOS == "Linux") && this.style.animationPlayState) {
									this.style.removeProperty("animation-play-state");
								};
								var eventType = (targetOS == "Linux") ? "mouseout" : "mouseleave";
								this.addEventListener(eventType, resume, false);
								nWin.clearInterval(nCnt);
							};
							function resume(ev) {
								// Bug in Linux: The "mouseleave" event does not fire. Most likely this is the reason
								// for this type of desktop notifications (alert.xul) getting stuck in pause mode. The
								// animation effect remains in "hover" state, preventing the notification to close at
								// the end of predefined animation duration time.
								if(((ev.type == "mouseleave") && (ev.currentTarget == this)) ||
								  ((ev.type == "mouseout") && // Linux.
								  (((ev.clientX -1) < nBoxBounds.left) || ((ev.clientX + 1) > nBoxBounds.right) ||
								  ((ev.clientY -1) < nBoxBounds.top) || ((ev.clientY + 1) > nBoxBounds.bottom)))) {
									if(targetOS == "Linux") {
										this.style.animationPlayState = "running";
									};
									this.removeEventListener(ev.type, resume, false);
									if(cnt > 0) {
										return window.setTimeout(nCounter, 0, cnt, iter);
									};
								};
							};
							if(appVer >= 27) { // Support of css animation pause by mouse
							// hover, we have to pause our counter as well..
								nBox.addEventListener("mouseover", pause, false);
							};
						})(undefined, undefined, appVer);
						if(targetOS == "Linux") {
							nBoxBounds = nBox.getBoundingClientRect();
						};
						winOpenObs.remObs(aTopic);
					};
				}, false);
			};
		},
		addObs: function() {
			obsSrvc.addObserver(this, "domwindowopened", false);
		},
		remObs: function(aTopic) {
			obsSrvc.removeObserver(this, aTopic);
		}
	};
	window.setTimeout(function() {
		winOpenObs.addObs();
		function openDlg() {
			window.openDialog("chrome://global/content/alerts/alert.xul","_blank",
				"chrome,dialog,titlebar=no,popup",nImg,nTitle,nBody1,false,"",null);
		};
		if(targetOS == "WINNT") {
			try { // The alerts service does not work in all appVer under all platforms.
				alertsSrvc.showAlertNotification(nImg,nTitle,nBody1,false,"",null);
			} catch(ex) {
				openDlg();
			};
		} else {
			openDlg();
		};
	}, 5000);
};
