/*
***************************************************************************
**                                                                       **
**	Restart manager  -  Extension for Pale Moon                          **
**                                                                       **
**	Main code for main window and options dialog                         **
**                                                                       **
**	Copyright(C) 2018 - 2020 by George Malamas                           **
**                                                                       **
**	Free software for all people                                         **
**                                                                       **
**	Last edit: July 12, 2020                                             **
**                                                                       **
**	Release number: 9                                                    **
**                                                                       **
***************************************************************************
*/
// To do: Make the code compatible with strict mode.
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions_and_function_scope/Strict_mode
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions_and_function_scope/Strict_mode/Transitioning_to_strict_mode
// Any help or advises will be very appreciated :-)
// "use strict";

const {classes: cc, interfaces: ci, results: cr, utils: cu} = Components;
// Workaround for "redeclaration of const Cc,Ci,Cu" error in some
// app versions, while some other versions require these
// declarations, even for the overlay part of our code..

// this.EXPORTED_SYMBOLS = ["_RestMan"]; // (In combination with importing this file in our js component)
// It works fine, but causes almost quadruple of memory commitment..

var _RestMan = {
	/*<Shared stuff>**********************************/
	// Most of the services we need:
	appInfo:             cc["@mozilla.org/xre/app-info;1"].getService(ci.nsIXULAppInfo),
	appStartup:          cc["@mozilla.org/toolkit/app-startup;1"].getService(ci.nsIAppStartup),
	dirSrvc:             cc["@mozilla.org/file/directory_service;1"].getService(ci.nsIProperties),
	obsSrvc:             cc["@mozilla.org/observer-service;1"].getService(ci.nsIObserverService),
	prefSrvc:            cc["@mozilla.org/preferences-service;1"].getService(ci.nsIPrefService),
	promptSrvc:          cc["@mozilla.org/embedcomp/prompt-service;1"].getService(ci.nsIPromptService),
	restmanSrvc:         null, // Will be defined later, if appVer >= 3. Otherwise, we'll be broken in appVer 1.5 & 2..
	strings:             cc["@mozilla.org/intl/stringbundle;1"].getService(ci.nsIStringBundleService),
	timer:               cc["@mozilla.org/timer;1"].createInstance(ci.nsITimer),
	winMed:              cc["@mozilla.org/appshell/window-mediator;1"].getService(ci.nsIWindowMediator),
	logger:              undefined,
	setLogger:           function() {
							if(("console" in window) && (_RestMan.prefSrvc.getBranch("devtools.browserconsole.").getChildList("").length > 10)) {
								_RestMan.logger = window.console.log.bind(window.console);
							} else {
								_RestMan.logger = function(msg) {
									return cc["@mozilla.org/consoleservice;1"].getService(ci.nsIConsoleService).logStringMessage(msg);
								};
							};
							delete _RestMan.setLogger;
						 },
	// App info:
	appInfos: {
		appId:           "{8de7fcbb-c55c-4fbe-bfc5-fc555c87dbc4}", // Pale Moon's v25+ id.
		appName:         "Pale Moon", // All app versions.
		appVer:          null, // All app versions.
		appLang:         "en-US", // All app versions.
		targetOS:        "WINNT" // appVer 4+.
	},
	// Our prefBranch:
	rmPrefBranch:        cc["@mozilla.org/preferences-service;1"].getService(ci.nsIPrefService).getBranch("extensions.restartmanager."),
	// Browser's prefBranch:
	brPrefBranch:        cc["@mozilla.org/preferences-service;1"].getService(ci.nsIPrefService).getBranch("browser."), // Pale Moon
	// Most of our prefs:
	locals: {
		menuItemIcon:    true, // All app versions.
		iconStyle:       0, // All app versions.
		rmButtonStyle:   1, // appVer 4+.
		rmSmallButton:   false, // Use small button allways, appVer 4+.
		askOnRestart:    true, // All app versions.
		sessionRestore:  false, // Restore previous session after restart, appVer >= 20.
		warnOnCloseMT:   false, // Multiple tabs, all app versions.
		warnOnCloseMPT:  true, // Multiple private tabs, appVer 20+ with "Private Tab" extension.
		detectOpenDlgs:  true, // Detection of opened dialogs on restart, exit or close of the last browser window.
		codeMonitoring:  false, // Can be activated in "about:config"("codeMonEnabled"), to enable code flow monitoring in selected points, implemented by a sequence of alert messages.
		codMonChkBoxHid: true, // Checkbox in options dialog(hidden by default, "codeMonCheckbox_hidden"), which controls the above pref ("codeMonEnabled").
		restShortcutVis: true, // Restart shortcut indication in restart menu items.
		restartShortcut: true, // Restart shortcut active, all app versions.
		rShortcutKeyMod: "alt", // All app versions.
		rShortcutChar:   "R", // All app versions.
		appExitShortcut: 1, // Exit shortcut: 0 = disabled, 1 = active in Windows, 2 = enabled by user in linux, all app versions.
		appExitScKeyMod: "alt", // All app versions.
		appExitScChar:   "X", // All app versions.
		sesRestShortcut: true, // Restore Previous Session shortcut active, appVer >= 4.
		sesRestScKeyMod: "shift", // appVer >= 4.
		sesRestScChar:   "L", // appVer >= 4.
		invalCaches:     false, // Invalidate Caches On Restart, all app versions.
		invCachHghlght:  false, // Highlight Invalidate Caches, all app versions.
		alwaysWrnOnExit: false, // Show warning on exit/close last browser window in cases pm does not, appVer >= 20.
		restmanVer:      "2.00",
		restmanName:     "Restart manager",
		// App version specific strings & string sources:
		confirmLabel:    "Confirm", // All app versions.
		neverAskLabel:   "Do not ask next time", // All app versions.
		restartLabel:    "Restart", // All app versions.
		restartAccKey:   "R",       // All app versions.
		restartTitle:    "Restart Pale Moon", // appVer >= 4.
		restartMessage:  "Restart Palemoon now?", // appVer 1.5 & 2. Old Pale Moon versions are not fully "cleared" of firefox.
		optionsTitle:    "Restart manager - Options", // appVer < 27, appVer 27+ -> "Restart manager - Preferences".
		optionsLabel:    "Options", // appVer < 27, appVer 27+ -> "Preferences".
		optionsAccessk:  "O",       // All app versions.
		appearanceLabel: "Appearance", //appVer >= 6.
		clrMnrOnRestart: true, // Clear "MOZ_NO_REMOTE" on restart, appVer >= 3.5.
		clrMnrOnProfile: 0 // 0: any profile, 1: same profile, 2: other profile - appVer >= 3.5.
	},
	// Helper objects
	temps: {
		sharedResLoaded: false, // Shared resources, all app versions.
		openedDialog:    null, // Detection of opened dialogs, all app versions.
		dnsPrefObserver: null, // "Do restart no restore" pref observer, appVer >= 20.
		setPrefObserver: false, // Apply ui settings pref observer, all app versions.
		restoreUserSet:  0, // User settings we change temporarily on a specific case, appVer >= 20.
		rpsObs:          null, // Reference to Session restore observer, to be able remove it from various places, appVer >= 4.
		smStartupType:   null, // Startup type of "Session manager" extension, appVer >= 20.
		lastDetectedDlg: null, // Warn for detected dialog only once per last detection.
		rejectRequest:   false, // Give time the user as long blinker is active, to better notice the warning.
		menuPopupLis:    null, // Reference to event listener, required to be able remove/re-add it in some case in appVer < 4.
		profilesIniData: null, // Profiles data in "profiles.ini" file, appVer >= 3.5.
		profilesInUse:   0 //appVer >= 3.5.
	},
	/*********************************</Shared stuff>*/

	// Our main localized strings are included, or have been composed from included strings in each app version.
	// Due to this localization method, some of our strings may not be so appropriate, or may be poor,
	// or there may be problems in some languages, especially in eastern languages.
	// For this reason also, due to lack of strings, we use some images instead of labels in our options dialog.

	onLoadResources: function(onLoad) {
	// Defining resources needed on application startup, new browser window opening, our preferences window
	// opening, entering customization mode. It is called always on application startup, new browser window
	// opening and then separately as/when necessary.
		_RestMan.appInfos.appId = _RestMan.appInfo.QueryInterface(ci.nsIXULRuntime).ID;
		_RestMan.appInfos.appVer = window.parseFloat(_RestMan.appInfo.version); // parseInt
		_RestMan.appInfos.targetOS = _RestMan.appInfo.QueryInterface(ci.nsIXULRuntime).OS;
		if(_RestMan.appInfos.appVer >= 4) {
			_RestMan.appInfos.appName = _RestMan.appInfo.name;
			_RestMan.locals.sesRestShortcut = _RestMan.rmPrefBranch.getBoolPref("sRestore_shortcut");
			if(_RestMan.rmPrefBranch.getCharPref("sRestShortcutKey_mod") == "shift") {
				_RestMan.locals.sesRestScKeyMod = "shift";
			} else if(_RestMan.rmPrefBranch.getCharPref("sRestShortcutKey_mod") == "alt-option") {
				_RestMan.locals.sesRestScKeyMod = "alt";
			};
			var sesRestScCharPref = _RestMan.rmPrefBranch.getComplexValue("sesRestShortcut_char", ci.nsISupportsString).data;
			if(!sesRestScCharPref || (/\s/).test(sesRestScCharPref)) {
				_RestMan.rmPrefBranch.setCharPref("sesRestShortcut_char", _RestMan.locals.sesRestScChar);
			} else {
				_RestMan.locals.sesRestScChar = sesRestScCharPref;
			};

			// Get our current version & our name:
			if(onLoad && ("AddonManager" in window)) { // Avoid unnecessary import of "AddonManager.jsm" and
			// also run "versionChecker()", when "onLoadResources()" is called in the scope of our pref window.
				AddonManager.getAddonByID("restartmanager@chania.gr", function(addon) {
					_RestMan.locals.restmanVer = addon.version;
					_RestMan.locals.restmanName = addon.name;
					_RestMan.versionChecker(); // Must be called by the asynchronous callback.
				});
			};
		} else {
			var brandBundle = _RestMan.strings.createBundle("chrome://branding/locale/brand.properties");
			var appName = brandBundle.GetStringFromName("brandShortName");
			if(appName !== null && appName !== undefined) {
				_RestMan.appInfos.appName = appName;
			} else {
				_RestMan.appInfos.appName = _RestMan.appInfo.name;
			}; // In some old versions of Pale Moon, "appInfo.name" reports "Firefox"..

			// Get our current version & our name:
			if(onLoad && (window.document.documentElement.id == "main-window")) {
				var extman = cc["@mozilla.org/extensions/manager;1"].getService(ci.nsIExtensionManager);
				_RestMan.locals.restmanVer = extman.getItemForID("restartmanager@chania.gr").version;
				_RestMan.locals.restmanName = extman.getItemForID("restartmanager@chania.gr").name;
				window.setTimeout(_RestMan.versionChecker, 500);
			};
		};
		_RestMan.appInfos.appLang = _RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getCharPref("general.useragent.locale");
		if((_RestMan.appInfos.appVer >= 3) && (_RestMan.restmanSrvc == null)) {
			_RestMan.restmanSrvc = cc["@restart_manager/chania.gr;1"].getService().wrappedJSObject; // Our js component.
		};
		_RestMan.locals.codeMonitoring = _RestMan.rmPrefBranch.getBoolPref("codeMonEnabled");
		_RestMan.locals.menuItemIcon = _RestMan.rmPrefBranch.getBoolPref("menuItemIcon");
		_RestMan.locals.iconStyle = _RestMan.rmPrefBranch.getIntPref("iconStyle");
		_RestMan.locals.rmButtonStyle = _RestMan.rmPrefBranch.getIntPref("buttonStyle");
		_RestMan.locals.rmSmallButton = _RestMan.rmPrefBranch.getBoolPref("allways_smallBtn"); // Used in appVer 4+.

		// We get from here the label and access key for our button's context menu.
		var appOptionsContext = window.document.getElementById("menu_preferences"); // Menu bar "Tools --> Options/Preferences".
		if(appOptionsContext) { // Does not exist nor needed when this function is called in the scope of our pref window.
			_RestMan.locals.optionsLabel = appOptionsContext.label;
			_RestMan.locals.optionsAccessk = appOptionsContext.getAttribute("accesskey");
		}; //
		_RestMan.locals.detectOpenDlgs = _RestMan.rmPrefBranch.getBoolPref("openDlg_detection");
		_RestMan.locals.restShortcutVis = _RestMan.rmPrefBranch.getBoolPref("restart_shortcutVis");
		_RestMan.locals.restartShortcut = _RestMan.rmPrefBranch.getBoolPref("restart_shortcut");
		_RestMan.locals.appExitShortcut = _RestMan.rmPrefBranch.getIntPref("exit_shortcut");
		if(_RestMan.rmPrefBranch.getCharPref("rShortcutKey_modifier") == "alt-option") {
			_RestMan.locals.rShortcutKeyMod = "alt";
		} else if(_RestMan.rmPrefBranch.getCharPref("rShortcutKey_modifier") == "shift") {
			_RestMan.locals.rShortcutKeyMod = "shift";
		};
		var rShortcutCharPref = _RestMan.rmPrefBranch.getComplexValue("rShortcut_char", ci.nsISupportsString).data;
		if(!rShortcutCharPref || (/\s/).test(rShortcutCharPref)) {
			_RestMan.rmPrefBranch.setCharPref("rShortcut_char", _RestMan.locals.rShortcutChar);
		} else {
			_RestMan.locals.rShortcutChar = rShortcutCharPref;
		};
		_RestMan.locals.invCachHghlght = _RestMan.rmPrefBranch.getBoolPref("invalCaches_highlight");
		if(_RestMan.rmPrefBranch.getCharPref("exitScKey_modifier") == "alt-option") {
			_RestMan.locals.appExitScKeyMod = "alt";
		} else if(_RestMan.rmPrefBranch.getCharPref("exitScKey_modifier") == "shift") {
			_RestMan.locals.appExitScKeyMod = "shift";
		};
		var appExitScCharPref = _RestMan.rmPrefBranch.getComplexValue("exitShortcut_char", ci.nsISupportsString).data;
		if(!appExitScCharPref || (/\s/).test(appExitScCharPref)) {
			_RestMan.rmPrefBranch.setCharPref("exitShortcut_char", _RestMan.locals.appExitScChar);
		} else {
			_RestMan.locals.appExitScChar = appExitScCharPref;
		};
		if(_RestMan.appInfos.appVer >= 20) {
			var browserBundle = _RestMan.strings.createBundle("chrome://browser/locale/browser.properties");
			_RestMan.locals.restartLabel = browserBundle.GetStringFromName("safeModeRestartButton"); // Restart
			var prefsBundle = _RestMan.strings.createBundle("chrome://browser/locale/preferences/preferences.properties");
			_RestMan.locals.restartTitle = prefsBundle.formatStringFromName("shouldRestartTitle", [_RestMan.appInfos.appName], 1); // Restart brandShortName
			_RestMan.temps.restoreUserSet = _RestMan.rmPrefBranch.getIntPref("restore_UserSettings");
			_RestMan.locals.alwaysWrnOnExit = _RestMan.rmPrefBranch.getBoolPref("always_warnOnExit");
		} else if(_RestMan.appInfos.appVer < 20 && _RestMan.appInfos.appVer >= 3) {
			var quitBundle = _RestMan.strings.createBundle("chrome://browser/locale/quitDialog.properties");
			_RestMan.locals.restartLabel = quitBundle.GetStringFromName("restartTitle").replace(/&/,"").replace(/\(.\)/,""); // Restart
			if(_RestMan.appInfos.appVer >= 4) {
				_RestMan.locals.restartTitle = quitBundle.formatStringFromName("restartDialogTitle", [_RestMan.appInfos.appName], 1); // Restart brandShortName
			};
		} else if(_RestMan.appInfos.appVer < 3 && _RestMan.appInfos.appVer > 1.5) {
			var extBundle = _RestMan.strings.createBundle("chrome://mozapps/locale/extensions/extensions.properties");
			var capital = extBundle.GetStringFromName("restartButton").replace(/ +%S|%S +/,"").charAt(0).toLocaleUpperCase(); // R
			_RestMan.locals.restartLabel = extBundle.GetStringFromName("restartButton").replace(/ +%S|%S +/,"").replace(/^./,capital); // Restart
		} else if(_RestMan.appInfos.appVer < 2) {
			// The most suitable source to get the "Restart" string is the profileSelection.properties file. However, in this file the property
			// "restartTitle=" has a wrong string in many locales, including en-US("restartTitle=Close %S" instead of "restartTitle=Restart %S").
			var updateBundle = _RestMan.strings.createBundle("chrome://mozapps/locale/update/updates.properties");
			var capital = updateBundle.GetStringFromName("restartButton").replace(/ +%S|%S +/,"").charAt(0).toLocaleUpperCase(); // R
			_RestMan.locals.restartLabel = updateBundle.GetStringFromName("restartButton").replace(/ +%S|%S +/,"").replace(/^./,capital); // Restart Now
		};
		var aWin = window;
		if(window.document.documentElement.getAttribute("windowtype") == "restman:options") { // Our pref window.
			aWin = _RestMan.winMed.getMostRecentWindow("navigator:browser");
		};
		var exitElem = aWin.document.getElementById("appmenu-quit") || // App menu --> "Exit"
					   aWin.document.getElementById("menu_FileQuitItem"); // "File" menu --> "Exit"
		if(exitElem) {
			var exitLabel = exitElem.label;
			var exitAccKey = exitElem.getAttribute("accesskey");
			var ak = new RegExp(exitAccKey, "i");
			if(ak.test(exitLabel)) {
				_RestMan.locals.restartAccKey = _RestMan.locals.restartLabel.match(/^./);
			} else { // Probably it's an eastern locale (or it is a localization mistake!).
				_RestMan.locals.restartAccKey = "R";
			};
		};
		if(_RestMan.appInfos.appVer >= 3.5) {
			_RestMan.locals.clrMnrOnRestart = _RestMan.rmPrefBranch.getBoolPref("clearMnr_onRestart");
			_RestMan.locals.clrMnrOnProfile = _RestMan.rmPrefBranch.getIntPref("clearMnr_onProfile");
		};
	},

	_RestManOnLoad: function RestMan_RestManOnLoad(reAddLis) {
		if(reAddLis) { // appVer < 4, on exiting customization mode.
			addMenuPopUpLis("fileMenu", reAddLis);
			return;
		};
		if(window.document.getElementById("commonDialog")) {
			var appVer = window.parseFloat(_RestMan.appInfo.version);
			if(appVer < 20 && appVer >= 3) {
				var restmanSrvc = cc["@restart_manager/chania.gr;1"].getService().wrappedJSObject;
				if(restmanSrvc.getSet("restartFlag", "getV")) { // get value.
				// We patch "on the fly" the message of restart dialog in appVer 3-19, to avoid
				// repeating here the code of the dialog, just to modify slightly its message..
				// Details for this can be found in "restartDlg3to19()" function.
					var browserBundle = _RestMan.strings.createBundle("chrome://browser/locale/browser.properties");
					var msgAddition = null;
					if(appVer >= 4) {
						msgAddition = browserBundle.GetStringFromName("addonInstallRestartButton"); // Restart Now
					} else if(appVer < 4) {
						var brandBundle = _RestMan.strings.createBundle("chrome://branding/locale/brand.properties");
						var updatesBundle = _RestMan.strings.createBundle("chrome://mozapps/locale/update/updates.properties");
						var appName = brandBundle.GetStringFromName("brandShortName");
						if(!appName) {
							appName = _RestMan.appInfo.name;
						};
						if(appVer < 4 && appVer > 3) {
							msgAddition = updatesBundle.formatStringFromName("restartNowButton", [appName], 1); // Restart brandShortName
						} else if(appVer < 3.5) {
							msgAddition = updatesBundle.formatStringFromName("restartButton", [appName], 1); // Restart brandShortName Now
						};
					};
					var questionMark = browserBundle.GetStringFromName("droponhomemsg").match(/.$/); // ?
					var dlgMessage = window.document.getElementById("info.body");
					var textToAdd = window.document.createTextNode("\n" + msgAddition + questionMark); // Restart (brandShortName) (Now)?
					if(dlgMessage) {
						dlgMessage.appendChild(textToAdd);
					};
					restmanSrvc.getSet("restartFlag", "setF"); // set false.
				};
			};
			return;
		};

		// In compliance with new Pale Moon's logic about menu items:
		function addMenuPopUpLis(aTarget, aParam) {
		// We display our app/file menu item with or without "…" at the end, depending on the action of
		// the menu item, which depends on current settings and contitions: If it wil cause an immediate
		// restart, we don't show "…", while if it wil cause the appearance of a dialog, we display it
		// with "…" at the end. As a bonus, we apply the same on the "Exit" menu item in both app/file
		// menus, in appVer 20+.
			var targetMenu = null;
			if(aTarget == "appMenu") {
				targetMenu = window.document.getElementById("appmenu-popup");
			} else if(aTarget == "fileMenu") {
				targetMenu = window.document.getElementById("menu_FilePopup");
			};
			if(targetMenu) {
				function onMenuPopUpShowing(event) {
					if(event.target !== targetMenu) { // Prevent actions on showing of submenus within app menu popup.
						return;
					};
					if(!_RestMan.temps.sharedResLoaded) {
						_RestMan.sharedResources();
					};
					var targetRestartMenuItem = null;
					var targetExitMenuItem = null;
					if(targetMenu.id == "appmenu-popup") { // "targetRestartMenuItem" must be defined here. In case of remove/disable/change of
					// our shortcut, "targetRestartMenuItem" is destroyed/reconstructed. Otherwise "onMenuPopUpShowing()" will lose its target.
						targetRestartMenuItem = window.document.getElementById("restman-appMenuItem");
						targetExitMenuItem = window.document.getElementById("appmenu-quit");
					} else if(targetMenu.id == "menu_FilePopup") {
						targetRestartMenuItem = window.document.getElementById("restman-fileMenuItem");
						targetExitMenuItem = window.document.getElementById("menu_FileQuitItem");
					}; 
					var countsPuS = _RestMan.winTabCounter("getCountsPuS");
					// [nonPrvWinCount, allTabCount, allPinnedTabs, nPrvNPndTabCnt].
					if(_RestMan.appInfos.appVer >= 20) {
						if(countsPuS[0] >= 1) {
							if(((_RestMan.locals.askOnRestart || (!_RestMan.locals.askOnRestart && _RestMan.locals.warnOnCloseMT)) && countsPuS[3] > 0
								&& (countsPuS[1] - countsPuS[2]) > 1) || (_RestMan.locals.warnOnCloseMPT && countsPuS[3] == 0 &&
								(countsPuS[1] - countsPuS[2]) > 1)) { // The action will show one of our dialogs on restart.
								targetRestartMenuItem.label = targetRestartMenuItem.label.replace(/…/g,"") + "…";
							} else { // The action will cause an immediate restart.
								targetRestartMenuItem.label = targetRestartMenuItem.label.replace(/…/g,"");
							};
							if((_RestMan.brPrefBranch.getBoolPref("warnOnQuit") && ((_RestMan.brPrefBranch.getBoolPref("showQuitWarning")
								 && _RestMan.brPrefBranch.getIntPref("startup.page") !== 3) ||
								_RestMan.rmPrefBranch.getBoolPref("always_warnOnExit"))) &&
								(((countsPuS[1] - countsPuS[2]) > 1 && (countsPuS[3] > 0 || (countsPuS[3] == 0 &&
								_RestMan.locals.warnOnCloseMPT))))) { // The action will show the exit dialog or our exit warning dialog.
								targetExitMenuItem.label = targetExitMenuItem.label.replace(/…/g,"") + "…";
							} else { // The action will cause an immediate exit.
								targetExitMenuItem.label = targetExitMenuItem.label.replace(/…/g,"");
							};
						} else { // At least 1 private window.
							if(_RestMan.locals.warnOnCloseMPT && countsPuS[1] > 1) { // The action will show our warning dialog, for restart or exit.
								targetRestartMenuItem.label = targetRestartMenuItem.label.replace(/…/g,"") + "…";
								targetExitMenuItem.label = targetExitMenuItem.label.replace(/…/g,"") + "…";
							} else { // The action will cause an immediate restart or exit.
								targetRestartMenuItem.label = targetRestartMenuItem.label.replace(/…/g,"");
								targetExitMenuItem.label = targetExitMenuItem.label.replace(/…/g,"");
							};
						};
					} else if(_RestMan.appInfos.appVer < 20 && _RestMan.appInfos.appVer > 3) {
						if(_RestMan.locals.askOnRestart && countsPuS[1] > 1 && 
							!cc["@mozilla.org/privatebrowsing;1"].getService(ci.nsIPrivateBrowsingService).
							privateBrowsingEnabled) { // The action will show the built-in restart dialog.
							targetRestartMenuItem.label = targetRestartMenuItem.label.replace(/…/g,"") + "…";
						} else { // The action will cause an immediate restart.
							targetRestartMenuItem.label = targetRestartMenuItem.label.replace(/…/g,"");
						};
					} else if(_RestMan.appInfos.appVer < 3.5) {
						if(_RestMan.locals.askOnRestart && countsPuS[1] > 1) { // The action will show our restart warning dialog.
							targetRestartMenuItem.label = targetRestartMenuItem.label.replace(/…/g,"") + "…";
						} else { // The action will cause an immediate restart.
							targetRestartMenuItem.label = targetRestartMenuItem.label.replace(/…/g,"");
						};
					};
					if(event.type === "popupshown") {
						targetMenu.removeEventListener("popupshown", _RestMan.temps.menuPopupLis, false);
						_RestMan.temps.menuPopupLis = null;
						_RestMan.temps.menuPopupLis = onMenuPopUpShowing;
						targetMenu.addEventListener("popupshowing", _RestMan.temps.menuPopupLis, false);
					};
				};
				if(aParam === true) { // appVer < 4, on exiting customization mode.
				// In these appVer, the "onMenuPopUpShowing" functionality above is broken after customizing, causing
				// undefined "targetRestartMenuItem.label" errors on "popupshowing" event.
					targetMenu.removeEventListener("popupshowing", _RestMan.temps.menuPopupLis, false);
					_RestMan.temps.menuPopupLis = null;
					addMenuPopUpLis("fileMenu", "popupshown");
					return;
				};
				var aEventType = "popupshowing";
				if(aParam === "popupshown") {
					aEventType = aParam;
				};
				_RestMan.temps.menuPopupLis = null;
				_RestMan.temps.menuPopupLis = onMenuPopUpShowing;
				targetMenu.addEventListener(aEventType, _RestMan.temps.menuPopupLis, false);
				if(_RestMan.appInfos.appVer >= 4) {
					_RestMan.temps.menuPopupLis = null;
				};
			};
		};

		_RestMan.setLogger();

		_RestMan.onLoadResources(true);

		// Setting up of our restart menu items.
		var appMenuRestartItem = window.document.getElementById("restman-appMenuItem");
		if(appMenuRestartItem) {
			appMenuRestartItem.label = _RestMan.locals.restartLabel;
			appMenuRestartItem.setAttribute("accesskey", _RestMan.locals.restartAccKey);
			if(_RestMan.locals.menuItemIcon) {
				appMenuRestartItem.setAttribute("menuicon", true);
			};
			appMenuRestartItem.setAttribute("iconStyle", _RestMan.locals.iconStyle);
			if(_RestMan.locals.invCachHghlght) {
				appMenuRestartItem.setAttribute("highlightMenuItem", true);
			};
			if(_RestMan.appInfos.appVer >= 22) {
				appMenuRestartItem.setAttribute("appVer22plus", "");
			};
			addMenuPopUpLis("appMenu");
		};
		var fileMenuRestartItem = window.document.getElementById("restman-fileMenuItem");
		if(fileMenuRestartItem) {
			fileMenuRestartItem.label = _RestMan.locals.restartLabel;
			fileMenuRestartItem.setAttribute("accesskey", _RestMan.locals.restartAccKey);
			if(_RestMan.locals.menuItemIcon) {
				fileMenuRestartItem.setAttribute("menuicon", true);
			};
			fileMenuRestartItem.setAttribute("iconStyle", _RestMan.locals.iconStyle);
			if(_RestMan.locals.invCachHghlght) {
				fileMenuRestartItem.setAttribute("highlightMenuItem", true);
			};
			if(_RestMan.appInfos.appVer < 3) {
				fileMenuRestartItem.setAttribute("appVer1v5-2", "");
			};
			if(_RestMan.appInfos.appVer >= 22) {
				fileMenuRestartItem.setAttribute("appVer22plus", "");
			};
			addMenuPopUpLis("fileMenu");
		};

		// Setting up of our restart shortcut.
		var restmanKeyset = window.document.getElementById("restmanKeyset_keys");
		var rmRestartKey = window.document.getElementById("keyRestmanRestart_KeyCombo");
		if(restmanKeyset && rmRestartKey && !_RestMan.locals.restartShortcut) {
			restmanKeyset.removeChild(rmRestartKey);
			if(appMenuRestartItem && appMenuRestartItem.hasAttribute("key")) {
				appMenuRestartItem.removeAttribute("key");
			};
			if(fileMenuRestartItem && fileMenuRestartItem.hasAttribute("key")) {
				fileMenuRestartItem.removeAttribute("key");
			};
		} else if(restmanKeyset && rmRestartKey &&_RestMan.locals.restartShortcut) { // Set shortcut attributes.
			rmRestartKey.setAttribute("modifiers", "accel," + _RestMan.locals.rShortcutKeyMod);
			rmRestartKey.setAttribute("key", _RestMan.locals.rShortcutChar);
			if(!_RestMan.locals.restShortcutVis) {
				if(appMenuRestartItem && appMenuRestartItem.hasAttribute("key")) {
					appMenuRestartItem.removeAttribute("key");
				};
				if(fileMenuRestartItem && fileMenuRestartItem.hasAttribute("key")) {
					fileMenuRestartItem.removeAttribute("key");
				};
			};
		};

		// Setting up of our "Exit" shortcut addition.
		var rmExitKey = window.document.getElementById("keyRestmanExit_KeyCombo");
		var appMenuExitItem = window.document.getElementById("appmenu-quit");
		var fileMenuExitItem = window.document.getElementById("menu_FileQuitItem");
		if(rmExitKey && (_RestMan.locals.appExitShortcut == 0 || (_RestMan.locals.appExitShortcut == 1 &&
			_RestMan.appInfos.targetOS !== "WINNT"))) {
			restmanKeyset.removeChild(rmExitKey);
			if(_RestMan.appInfos.targetOS == "WINNT") {
				if(appMenuExitItem && appMenuExitItem.hasAttribute("key")) {
					appMenuExitItem.removeAttribute("key");
				};
				if(fileMenuExitItem && fileMenuExitItem.hasAttribute("key")) {
					fileMenuExitItem.removeAttribute("key");
				};
			} else if(_RestMan.appInfos.targetOS == "Linux") { // Restore built-in shortcut.
				if(appMenuExitItem) {
					appMenuExitItem.setAttribute("key", "key_quitApplication");
				};
				if(fileMenuExitItem) {
					fileMenuExitItem.setAttribute("key", "key_quitApplication");
				};
			};
		} else if(rmExitKey && ((_RestMan.locals.appExitShortcut == 1 && _RestMan.appInfos.targetOS == "WINNT") ||
			(_RestMan.locals.appExitShortcut == 2 && _RestMan.appInfos.targetOS == "Linux"))) { // Set shortcut attributes.
			rmExitKey.setAttribute("modifiers", "accel," + _RestMan.locals.appExitScKeyMod);
			rmExitKey.setAttribute("key", _RestMan.locals.appExitScChar);
			if(_RestMan.appInfos.appVer < 3.5) {
				rmExitKey.removeAttribute("command");
				rmExitKey.setAttribute("oncommand", "goQuitApplication();");
			};
			if(_RestMan.locals.appExitShortcut == 2) { // Remove built-in key in Linux.
				var builtInKey = window.document.getElementById("key_quitApplication");
				if(builtInKey) {
					builtInKey.parentNode.removeChild(builtInKey);
				};
			};
		};

		// Setting up of our "Restore Previous Session" shortcut addition.
		var rmSessRestKey = window.document.getElementById("keyRestmanRPS_KeyCombo");
		var menuHistoryRPSItem = window.document.getElementById("historyRestoreLastSession");
		if(_RestMan.appInfos.appVer >= 4) {
			var appmenuHistoryRPSItem = window.document.getElementById("appmenu_restoreLastSession");
			var historyBtnRPSItem = window.document.getElementById("HMB_historyRestoreLastSession");
			if(rmSessRestKey && !_RestMan.locals.sesRestShortcut) {
				restmanKeyset.removeChild(rmSessRestKey);
				if(appmenuHistoryRPSItem && appmenuHistoryRPSItem.hasAttribute("key")) {
					appmenuHistoryRPSItem.removeAttribute("key");
				};
				if(menuHistoryRPSItem && menuHistoryRPSItem.hasAttribute("key")) {
					menuHistoryRPSItem.removeAttribute("key");
				};
				if(historyBtnRPSItem && historyBtnRPSItem.hasAttribute("key")) {
					historyBtnRPSItem.removeAttribute("key");
				};
			} else if(rmSessRestKey && _RestMan.locals.sesRestShortcut) { // Set shortcut attributes.
				rmSessRestKey.setAttribute("modifiers", "accel," + _RestMan.locals.sesRestScKeyMod);
				rmSessRestKey.setAttribute("key", _RestMan.locals.sesRestScChar);
			};
		};
		if(_RestMan.appInfos.appVer < 4) {
			if(rmSessRestKey) {
				restmanKeyset.removeChild(rmSessRestKey);
			};
			if(menuHistoryRPSItem) {
				menuHistoryRPSItem.parentNode.removeChild(menuHistoryRPSItem);
			};
		};

		if(_RestMan.appInfos.appVer >= 27) {
		// Removing of Pale Moon's native restart menu items, introduced in v27.5.0.
			var appmenuRestart = window.document.getElementById("appmenu_restart");
			var fileMenuRestart = window.document.getElementById("menu_restart");
			if(appmenuRestart) { 
				appmenuRestart.parentNode.removeChild(appmenuRestart);
			};
			if(fileMenuRestart) {
				fileMenuRestart.parentNode.removeChild(fileMenuRestart);
			};
		};

		// Setting up of our restart toolbar button.
		var restartButton = window.document.getElementById("restman-button");
		if(restartButton) {
			restartButton.label = _RestMan.locals.restartLabel;
			restartButton.setAttribute("tooltiptext", _RestMan.locals.restartLabel);
			restartButton.setAttribute("iconStyle", _RestMan.locals.iconStyle);
			if(_RestMan.locals.invCachHghlght) {
				restartButton.setAttribute("highlightBtn", true);
			};
			if(_RestMan.appInfos.appVer >= 4) {
				if(_RestMan.locals.rmButtonStyle == 0)  {
					restartButton.setAttribute("class", "toolbarbutton-1");
				};
				if(_RestMan.locals.rmSmallButton)  {
					restartButton.setAttribute("smallButton", true);
				};
				if(_RestMan.appInfos.appVer >= 22) {
					restartButton.setAttribute("appVer22plus", "");
				};
				if(_RestMan.appInfos.appVer >= 14) {
					restartButton.setAttribute("appVer14plus", "");
				};
				if(_RestMan.appInfos.appVer < 14) {
					restartButton.setAttribute("appVer4-13", "");
				};
				// We set the width of our button's label to be always as when the label is bold, to avoid the button to be
				// expanded/reduced when the label becomes bold/normal, in text only mode. This way, the extra space of label
				// when it is normal "absorbs" the extra width when it is bold.
				var hostToolbar = window.document.getElementById(restartButton.parentNode.parentNode.id);
				function setLblWidth(aParam) {
					if(!_RestMan.locals.invCachHghlght) {
						restartButton.setAttribute("highlightBtn", true); // Makes the label "bold"
					};
					var btnLabel = restartButton.ownerDocument.getAnonymousNodes(restartButton)[1];
					// var boldWidth = Math.round(parseInt(window.getComputedStyle(btnLabel).getPropertyValue("width")));
					var boldWidth = btnLabel.clientWidth + 1; // Simpler than above & accurate, in all supported appVer.
					if(btnLabel.getAttribute("class") == "toolbarbutton-text") { // Just for peace of mind..
						btnLabel.style.width = boldWidth + "px";
					};
					if(!_RestMan.locals.invCachHghlght) {
						restartButton.removeAttribute("highlightBtn");
					};
					if(aParam == "restTVS") { // Restore toolbar's visibility status.
						hostToolbar.setAttribute("collapsed", true);
					} else if(aParam == "restVTVS0") { // Restore Vertical Toolbar's visibility status.
						_RestMan.prefSrvc.getBranch("extensions.verticaltoolbar.").setIntPref("display", 0);
						hostToolbar.parentNode.setAttribute("collapsed", true);
					} else if(aParam == "restVTVS2-autohide") { // Restore Vertical Toolbar's visibility status.
						_RestMan.prefSrvc.getBranch("extensions.verticaltoolbar.").setIntPref("display", 2);
						hostToolbar.parentNode.setAttribute("autohide", true);
					} else if(aParam == "restVTVS2-noAutohide") { // Restore Vertical Toolbar's visibility status.
						_RestMan.prefSrvc.getBranch("extensions.verticaltoolbar.").setIntPref("display", 2);
					};
				};
				var toolbarMode = hostToolbar.getAttribute("mode");
				if(toolbarMode == "full" || toolbarMode == "text") {
					if(hostToolbar.id !== "vertical-toolbar") {
						if(hostToolbar.hasAttribute("collapsed") && hostToolbar.getAttribute("collapsed") == "true") {
							hostToolbar.removeAttribute("collapsed");
							setLblWidth("restTVS");
						} else {
							setLblWidth();
						};
					} else if(hostToolbar.id == "vertical-toolbar") { // Vertical Toolbar.
					// A bit of hassle for Vertical Toolbar, but this extension is one of my favorites :-)
						var vertTlbrPrefs = _RestMan.prefSrvc.getBranch("extensions.verticaltoolbar.");
						if(vertTlbrPrefs.getIntPref("display") == 1) { // Always show.
							setLblWidth();
						} else if(vertTlbrPrefs.getIntPref("display") == 0) { // Never show.
							vertTlbrPrefs.setIntPref("display", 1);
							hostToolbar.parentNode.removeAttribute("collapsed");
							setLblWidth("restVTVS0");
						} else if(vertTlbrPrefs.getIntPref("display") == 2) { // Automatically hide.
							vertTlbrPrefs.setIntPref("display", 1);
							if(hostToolbar.parentNode.hasAttribute("autohide")) { // Button "Always Show" is not checked.
								hostToolbar.parentNode.removeAttribute("autohide");
								setLblWidth("restVTVS2-autohide");
							} else if(!hostToolbar.parentNode.hasAttribute("autohide")) { // Button "Always Show" is checked.
								setLblWidth("restVTVS2-noAutohide");
							};
						};
					};
				};
				if(_RestMan.appInfos.targetOS == "Linux") {
					restartButton.setAttribute("Linux", "");
				};
			} else {
				restartButton.setAttribute("class", "toolbarbutton-1");
			};
			if((window.getComputedStyle(restartButton, null).MozBoxOrient == "horizontal") ||
			   (restartButton.parentNode.parentNode.getAttribute("mode") == "text")) {
				restartButton.parentNode.removeAttribute("align"); // Compatibility mainly with "Vertical Toolbar".
			};
		};
		var rmBtn_OptionsContext = window.document.getElementById("restman-launchOptionsItem");
		if(rmBtn_OptionsContext) {
			rmBtn_OptionsContext.label = _RestMan.locals.optionsLabel;
			rmBtn_OptionsContext.setAttribute("accesskey", _RestMan.locals.optionsAccessk);
		};

		// Setting up of our current profile name indicator toolbar item.
		var profileName = window.document.getElementById("restman-profileName");
		if(profileName) {
			var getInfo = undefined;
			if(("CustomEvent" in window) && (typeof window.CustomEvent == "function")){ // "CustomEvent()" constructor, appVer ~11+.
				getInfo = new CustomEvent("getInfo");
			} else {
				getInfo = window.document.createEvent("Event");
				getInfo.initEvent("getInfo", true, true);
			};
			// "restmanProfMan()" works with events, because it is called mainly by menus/menuitems. To avoid extra
			// work and complexity in it, we call it here using a custom event, just to get the job done..
			profileName.childNodes[1].value = _RestMan.restmanProfMan(getInfo)[0];
			var currentPrefs = {
				icon: _RestMan.rmPrefBranch.getBoolPref("profileName_icon"),
				bold: _RestMan.rmPrefBranch.getBoolPref("profileName_bold"),
				ital: _RestMan.rmPrefBranch.getBoolPref("profileName_italic"),
				color: _RestMan.rmPrefBranch.getCharPref("profileName_color")
			};
			if(!currentPrefs.icon) {
				profileName.setAttribute("icon", false);
			};
			if(currentPrefs.bold) {
				profileName.childNodes[1].style.fontWeight = "bold";
			};
			if(currentPrefs.ital) {
				profileName.childNodes[1].style.fontStyle = "italic";
			};
			if(currentPrefs.color) {
				profileName.childNodes[1].style.color = currentPrefs.color;
			};
			if(_RestMan.restmanSrvc.getSet("startedWithNoRemote", "getV")) { // get value.
				profileName.childNodes[1].setAttribute("mozNoRemote", true);
			};
		} else if(_RestMan.appInfos.appVer < 3.5) { // We don't support it in these oldies..
			var toolboxPalette = window.document.getElementById("navigator-toolbox").palette;
			for(var n = toolboxPalette.childNodes.length - 1; n >= 0; n--) { // Reverse searching because since
				// the item we search for is not built-in, most likely it will be near the end of the list.
				if(toolboxPalette.childNodes[n].id == "restman-profileName") {
					toolboxPalette.removeChild(toolboxPalette.childNodes[n]);
					break;
				};
			};
		};

		if(_RestMan.appInfos.appVer >= 20) {
			var browserWin = window.document.getElementById("main-window");
			// In a mixed session (normal & private), despite of pm defaults, we want to save the session of the
			// last closed non-private window, even on exiting or restarting pm, from within private browsing mode.
			// Details for this can be found in "sessionSaveHelper" function.
			// For this, we can't use the "unload" event, because, when it is dispatched is too late for the session
			// to be saved.
			// Also, we may have to perform some actions on closing of the last browser window. Details for this
			// can be found in "onQuitRequest" function.
			// We can't use listeners for "close" & "DOMWindowClose" events, because in case of warning (pm default)
			// or quit dialog, the listener captures the event after the dialog is closed, in both cases of dialog
			// accept/cancel.
			// There are also two important reasons we want to get the events before pm:
			// 1.- Pm has only one warning dialog on attempt to close multiple tabs, which is used in both non-private
			// and private or mixed browsing modes. If this dialog is disabled, there is no any warning in all modes.
			// Also, this dialog is not shown on closing the last browser window, if it is private, or on pm exiting from 
			// private browsing mode (appVer < 26).
			// We show our warning dialog in six different cass. All the details about it can be found in description
			// of "warningDialog()" below.
			// 2.- Pm does not recognize the case where a window is non-private but all of its tabs are private
			// (using "Private tab" extension), showing its quit dialog (if enabled) which offers "Save and exit" option.
			// We handle properly this case, showing our warning dialog instead.
			// So, on closing a window, we check first for privacy and act accordingly: We let pm do its job as usual or
			// we take the control.
			// On pm exiting, this check is performed of the observer in our js component.
			// The following approach seems to solve our problems reliably, because it works and does not affect the normal
			// operation of pm.
			if(browserWin) {
				if(browserWin.getAttribute("windowtype") == "navigator:browser") { // We do this check because some other
					// windows have also id="main-window" (scratchpad.xul, pageInfo.xul, filepicker.xul). Of course, our
					// overlay is applied only in browser.xul, but we feel a bit.. insecure.. :-)

					// When the window is closed with the close button:
					browserWin.setAttribute("onclose", "return _RestMan.onBrowserWinClose(event);");

					// When the window is closed with Ctrl+Shift+W:
					browserWin.parentNode.getElementById("cmd_closeWindow").setAttribute("oncommand", "_RestMan.onCmdBrowserWinClose(event);");
				//====================================================================================================================//
					// Getting involved in restart in "Safe Mode" for better control:
					var appmenuSafeModeItem = browserWin.parentNode.getElementById("appmenu_safeMode");
					var helpSafeModeItem = browserWin.parentNode.getElementById("helpSafeMode");
					if(appmenuSafeModeItem) {
						appmenuSafeModeItem.setAttribute("oncommand", "_RestMan._RestManOnRestartRequest(\"restart\", \"safeMode\");");
					};
					if(helpSafeModeItem) {
						helpSafeModeItem.setAttribute("oncommand", "_RestMan._RestManOnRestartRequest(\"restart\", \"safeMode\");");
					};
					if(_RestMan.appInfos.appVer >= 27) {
						window.gBrowser.addEventListener("pageshow", function onPageshow(event) {
							var uri = event.target.location.href;
							if(uri == "about:profiles" || uri == "chrome://global/content/aboutProfiles.xhtml" ||
							   uri == "about:support" || uri == "chrome://global/content/aboutSupport.xhtml") {
								var btnContainer = event.target.getElementById("action-box");
								var smRestartBtn = event.target.getElementById("restart-in-safe-mode-button");
								var restartBtn = event.target.getElementById("restart-button");
								if(btnContainer && smRestartBtn && restartBtn) {
									if(_RestMan.appInfos.appVer >= 28) {
										btnContainer.addEventListener("click", function(clickEvent) {
											clickEvent.stopPropagation();
										}, true);
									} else {
										Array.from(btnContainer.querySelectorAll("#safe-mode-box, #restart-box")).forEach(function(el) {
											el.addEventListener("click", function(clickEvent) {
												clickEvent.stopPropagation();
											}, true);
										});
									};

									// Simulation of mouse "click" event, because we can't use the real
									// "click" event since we have blocked it above.
									function restartFromPlacesSM() {
										_RestMan._RestManOnRestartRequest("restart", "safeMode");
									};
									smRestartBtn.addEventListener("mousedown", function(onMousedown) {
										if(onMousedown.button !== 0) {
											return;
										};
										smRestartBtn.addEventListener("mouseup", restartFromPlacesSM, false);
										smRestartBtn.addEventListener("mouseout", function onMouseout() {
											smRestartBtn.removeEventListener("mouseout", onMouseout, false);
											smRestartBtn.removeEventListener("mouseup", restartFromPlacesSM, false);
										}, false);
									}, false);
									function restartFromPlaces() {
										_RestMan._RestManOnRestartRequest("restart", "restart");
									};
									restartBtn.addEventListener("mousedown", function(onMousedown) {
										if(onMousedown.button !== 0) {
											return;
										};
										restartBtn.addEventListener("mouseup", restartFromPlaces, false);
										restartBtn.addEventListener("mouseout", function onMouseout() {
											restartBtn.removeEventListener("mouseout", onMouseout, false);
											restartBtn.removeEventListener("mouseup", restartFromPlaces, false);
										}, false);
									}, false);
								};
							};
						}, false);
					};
				};
			};
		};

		if(_RestMan.appInfos.appVer >= 3) {
		// In case there are no browser windows and the user decides to open again
		// a browser window, we also have to register again our observer:
			window.setTimeout(function() {
				if(_RestMan.restmanSrvc.getSet("appHasStarted", "getV") && // get value.
					!_RestMan.restmanSrvc.getSet("observerIsRegistered", "getV")) { // get value.
					_RestMan.restmanSrvc.observerRegister();
				};
			}, 1500);
		};

		if(_RestMan.appInfos.appVer >= 3.6) { // Workaround for bug 1235822.
			var focusMgr = cc["@mozilla.org/focus-manager;1"].getService(ci.nsIFocusManager);
			// "nsIFocusManager" is supported by appVer 3.6+ (Gecko 1.9.2).
			window.gBrowser.addEventListener("DOMWillOpenModalDialog", function(event) {
				if(event.isTrusted && event.target.document instanceof HTMLDocument && focusMgr.activeWindow) {
					window.focus(); // When a content-modal dialog, awaiting response, is shown in
					// a window which is hidden behind other window(s) or it's minimized, there is no any
					// sign for the user on why this damn thing ignores any exit/restart requests..!
				};  // Our approach works on both window-modal and content-modal dialogs, because although
					// window-modal dialogs get focused, their parent window does not, if it is minimized.
			}, false);
		};

		if(_RestMan.appInfos.appVer >= 4) {
			function setReset() {
				var ss = cc["@mozilla.org/browser/sessionstore;1"].getService(ci.nsISessionStore);
				var rpsKey = window.document.getElementById("keyRestmanRPS_KeyCombo");
				if(ss.canRestoreLastSession && rpsKey) { // If the last session can be restored and our
				// keyboard shortcut is enabled, enable its key and set the observer.
					if((_RestMan.appInfos.appVer >= 20 && !PrivateBrowsingUtils.isWindowPrivate(window)) ||
					   (_RestMan.appInfos.appVer < 20 && !cc["@mozilla.org/privatebrowsing;1"].
					   getService(ci.nsIPrivateBrowsingService).privateBrowsingEnabled)) {
						rpsKey.removeAttribute("disabled");
						if(_RestMan.temps.rpsObs == null) {
							_RestMan.prevSessionRestObs(true);
						};
					};
					window.focus();
				};

				window.setTimeout(function restore() {
					if(_RestMan.appInfos.appVer >= 20 && _RestMan.temps.restoreUserSet !== 0) { // Check if we have to restore
					// the settings we change on "restart no restore" in specific cases, first step.
						if(_RestMan.temps.restoreUserSet == 3 || _RestMan.temps.restoreUserSet == 4 || _RestMan.temps.restoreUserSet == 7) {
							_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup", 1);
							if(_RestMan.temps.restoreUserSet == 3) {
								_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 1);
								_RestMan.temps.restoreUserSet = 1;
							};
							if(_RestMan.temps.restoreUserSet == 4) {
								_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 2);
								_RestMan.temps.restoreUserSet = 2;
							};
							if(_RestMan.temps.restoreUserSet == 7) {
								_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 0);
								_RestMan.temps.restoreUserSet = 0;
							};
						} else if(_RestMan.temps.restoreUserSet == 5 || _RestMan.temps.restoreUserSet == 6 || _RestMan.temps.restoreUserSet == 8) {
							_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup", 2);
							if(_RestMan.temps.restoreUserSet == 5) {
								_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 1);
								_RestMan.temps.restoreUserSet = 1;
							};
							if(_RestMan.temps.restoreUserSet == 6) {
								_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 2);
								_RestMan.temps.restoreUserSet = 2;
							};
							if(_RestMan.temps.restoreUserSet == 8) {
								_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 0);
								_RestMan.temps.restoreUserSet = 0;
							};
						};
						if(_RestMan.temps.restoreUserSet == 1 || _RestMan.temps.restoreUserSet == 2) {
							if(!ss.canRestoreLastSession) { // This may happen if the session was restored in
							// "Safe Mode" after a "restart no restore in "Safe Mode", where we were disabled
							// and could not perform any pending actions, or after a crash.
								if(!_RestMan.restmanSrvc.getSet("userSettingsRestored", "getV")) { // get value.
									_RestMan.restmanSrvc.getSet("userSettingsRestored", "setT"); // set true.
									_RestMan.onQuitRequest("restUserSet");
								} else {
									_RestMan.temps.restoreUserSet = 0;
								};
							} else {
								if(!PrivateBrowsingUtils.isWindowPrivate(window) && (_RestMan.temps.rpsObs == null)) {
									_RestMan.prevSessionRestObs(true);
								};
							};
						};
					};
				}, 1500);
			};
			if(_RestMan.restmanSrvc.getSet("appHasStarted", "getV")) { // get value.
				// "appHasStarted" becomes "true" after "sessionstore-windows-restored" notification.
				setReset();
			} else {
				_RestMan.obsSrvc.addObserver(function appHasStarted(aSubject, aTopic, aData) {
					_RestMan.obsSrvc.removeObserver(appHasStarted, aTopic);
					setReset();
				}, "sessionstore-windows-restored", false);
			};

			// After a "forced restart" of a normal browsing session, we make the browser think it was crashed, to show the
			// "about:sessionrestore" page. Here we modify the appearance of this page, to be somewhat different from this one
			// after a real crash, reduce the width and increase the height of its list content area. We also restore the settings
			// we changed before restart. After a "forced restart" of a private browsing session, we only have to restore the 
			// corresponding settings.
			if(_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getPrefType("extensions.restartmanager.forced_to_restart") !== 0 && // PREF_INVALID.
				_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).prefHasUserValue("extensions.restartmanager.forced_to_restart")) {
				function psRestore() {
					var prevSettings = JSON.parse(_RestMan.rmPrefBranch.getCharPref("forced_to_restart"));
					if(typeof prevSettings[1] === "boolean") {
						_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_from_crash", prevSettings[1]);
					};
					if(typeof prevSettings[2] === "number") {
						_RestMan.brPrefBranch.setIntPref("sessionstore.max_resumed_crashes", prevSettings[2]);
					};
					if(_RestMan.appInfos.appVer >= 26 && typeof prevSettings[3] === "boolean") {
						_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setBoolPref("dom.disable_beforeunload", prevSettings[3]);
					};
					if(typeof prevSettings[4] === "boolean") {
						_RestMan.brPrefBranch.setBoolPref("warnOnQuit", prevSettings[4]);
					};
					if(_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getPrefType("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.use_browser_crash_prompt") == 128) { // PREF_BOOL.
					// For simplicity, we ckeck only for the presence of the pref of interest, regardless if "Session Manager" is installed/active or not.
						if(typeof prevSettings[5] === "boolean") {
							_RestMan.prefSrvc.getBranch("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.").setBoolPref("use_browser_crash_prompt", prevSettings[5]);
						};
					};
					_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).clearUserPref("extensions.restartmanager.forced_to_restart");
					_RestMan.prefSrvc.savePrefFile(null);
				};
				window.addEventListener("readystatechange", function pageSetup() {
					if((window.content.document.readyState == "interactive" || window.content.document.readyState == "complete") && 
						window.content.document.documentURI == "about:sessionrestore") { // Precedes "about:blank" b4 this.
						window.removeEventListener("readystatechange", pageSetup, true);
						var tabList = window.content.document.getElementById("tabList");
						var restoreBtn = window.content.document.getElementById("errorTryAgain");
						if(tabList && restoreBtn) {
							var spEvent = window.content.document.createEvent("KeyboardEvent");
							spEvent.initKeyEvent("keydown", true, true, null, false, false, false, false, 32, 0);
							restoreBtn.addEventListener("focus", function deselect() {
								restoreBtn.removeEventListener("focus", deselect, false);
								var treeRows = tabList.treeBoxObject.view.rowCount;
								for(var r = 0; r < treeRows; r++) {
									if(tabList.view.isContainer(r)) {
									// "tabList.view.getItemAtIndex(r).dispatchEvent(spEvent);", Does not
									// work for unknown reasons, giving "getItemAtIndex is not a function"..
										tabList.view.selection.select(r);
										tabList.dispatchEvent(spEvent); // Space bar.
									};
								};
								tabList.style.removeProperty("visibility");
								tabList.view.selection.select(0);
								if(tabList.view.selection.isSelected(0)) {
									tabList.focus();
								};
							}, false);
							tabList.style.visibility = "hidden";
							tabList.style.marginTop = "1em";
							tabList.style.minHeight = "24em";
						};
						var div1 = window.content.document.getElementById("errorTitle");
						var div2 = window.content.document.getElementById("errorShortDesc");
						var contentArea = window.content.document.getElementById("errorPageContainer");
						if(div1) {
							div1.parentNode.removeChild(div1);
						};
						if(div2) {
							div2.parentNode.removeChild(div2);
						};
						if(contentArea) {
							contentArea.style.paddingTop = "1em";
							contentArea.style.paddingBottom = "1em";
							contentArea.style.maxWidth = "60em"
						};
					};
				}, true);
				window.setTimeout(psRestore, 2500); // Restore the settings we changed before "forced restart".
			};
			window.setTimeout(function() { // A level of integrity protection of our code.
				window.Object.getOwnPropertyNames(window._RestMan).forEach(function(prop) {
					window.Object.defineProperty(window._RestMan, prop, {
						configurable: false, writable: false
					});
				});
				if(window.Object.getOwnPropertyDescriptor(window, "_RestMan").configurable) { // appVer 28+, why..?
					window.Object.defineProperty(window, "_RestMan", {
						configurable: false, writable: false
					});
				};
			}, 1000);
		};
		window.setTimeout(function() {
			if(_RestMan.appInfos.appVer < 28) {
				// Some cleanup firt for appVer < 20 (Must precede the "for()" loop below!).
				if(_RestMan.appInfos.appVer < 20) {
					var cllbl = window.document.getElementById("closeWinLabel");
					var prlbl = window.document.getElementById("privateBrowsingTitle");
					if(cllbl) { 
						cllbl.parentNode.removeChild(cllbl);
					};
					if(prlbl) {
						prlbl.parentNode.removeChild(prlbl);
					};
				};
				// The "about:profiles" page does not exist in v27 & older.
				var rmContextMenu = window.document.getElementById("restman-contextMenu");
				rmContextMenu.removeChild(rmContextMenu.childNodes[2]); // Our menu item to open "about:profiles".
				if(_RestMan.appInfos.appVer < 3.5) { // The profile service is not accessible and "nsIINIParserFactory" is
				// not available, so we don't support restart to or start other profiles in these oldies..
					for(var n = 1; n < rmContextMenu.childNodes.length; null) {
						rmContextMenu.removeChild(rmContextMenu.childNodes[0]);
					};
					rmContextMenu.removeAttribute("onpopupshowing");
					rmContextMenu.removeAttribute("onpopuphidden");
				};
			};
			if(_RestMan.appInfos.appVer >= 3.5) {
			// We use the window title of built-in profile manager as the caption label
			// of our two profile lists in context menu of our restart button.
				window.pList1 = window.document.getElementById("profMenuList1");
				window.pList2 = window.document.getElementById("profMenuList2");
				window.pList1.firstChild.firstChild.label =
					window.pList1.firstChild.firstChild.label.replace(/^.+moon[\s:–—-]*|^.*Firefox[\s:–—-]*/i, "").
					replace(/.+/, function(match) {return match.toLocaleLowerCase()}).
					replace(/^./, function(match) {return match.toLocaleUpperCase()});
				window.pList2.firstChild.firstChild.label =
					window.pList2.firstChild.firstChild.label.replace(/^.+moon[\s:–—-]*|^.*Firefox[\s:–—-]*/i, "").
					replace(/.+/, function(match) {return match.toLocaleLowerCase()}).
					replace(/^./, function(match) {return match.toLocaleUpperCase()});
				delete window.pList1;
				delete window.pList2;
			}; // Lookbehind assertions: Not supported by Firefox/Pale Moon..
		}, 2000);
	},

	versionChecker: function() {
		if(_RestMan.appInfos.appVer >= 3 && _RestMan.restmanSrvc.getSet("versionChecked", "getV")) { // get value.
			return;
		};
		function showUpdateNotification() { // We use the following method to load the script because we need some 
		// strings, available only in dtd files. This method also is fully compatible with all appVer we support, 2-29+.
			var proxyWin = "data:application/vnd.mozilla.xul+xml," +
				window.encodeURIComponent("<?xml version='1.0'?>\n\
					<?xml-stylesheet href='chrome://global/skin/' type='text/css'?>\n\
					<!DOCTYPE window [\n\
					<!ENTITY % updateDTD SYSTEM 'chrome://mozapps/locale/update/updates.dtd'>\n\
						%updateDTD;\n\
					<!ENTITY % brandDTD SYSTEM 'chrome://branding/locale/brand.dtd'>\n\
						%brandDTD;<!-- Required by 'updateWizard.title' in 'updates.dtd' for some not updated locales in some app versions -->\n\
					<!ENTITY % extensionsDTD SYSTEM 'chrome://mozapps/locale/extensions/extensions.dtd'>\n\
						%extensionsDTD;\n\
					<!ENTITY addon.releaseNotes.label ''><!-- Used by appVer < 28.6 & >= 4, extensions.dtd -->\n\
					]>\n\
					<window xmlns='http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul'\n\
					buttons=','\n\
					hidden='true'\n\
					style='display: none; visibility: collapse; background: transparent;'\n\
					onload='updateNotification();'>\n\
					<label id='softUpdTitle' value='&updateWizard.title;' hidden='true'/>\n\
					<label id='relNotesLbl' value='&addon.releaseNotes.label;' hidden='true'/>\n\
					<script type='application/x-javascript' src='chrome://restartmanager/content/restmanUpdateNotifier.js'/></window>");
			window.openDialog(proxyWin, "_blank","chrome,dialog,titlebar=no,popup,screenY=-60");
		};
		var previousVer = null;
		var currentVer = _RestMan.locals.restmanVer;
		var vc = null;
		if(("Services" in window) && ("vc" in window.Services)) {
			vc = window.Services.vc;
		} else {
			vc = cc["@mozilla.org/xpcom/version-comparator;1"].getService(ci.nsIVersionComparator);
		};
		if(_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).prefHasUserValue("extensions.restartmanager.version")) {
			previousVer = _RestMan.rmPrefBranch.getCharPref("version");
			var vComp = vc.compare(previousVer, currentVer);
			if(vComp !== 0) {
				if(vComp < 0) { // We've been upgraded.
					showUpdateNotification();
				};
				_RestMan.rmPrefBranch.setCharPref("version", currentVer);
			};
		} else {
			_RestMan.rmPrefBranch.setCharPref("version", currentVer);
			if(!_RestMan.rmPrefBranch.getBoolPref("firstRun")) {
				showUpdateNotification(); // We've been upgraded, because in older than v2.06 versions,
			};							  // the pref "version" does not exist.
		};
		if(_RestMan.rmPrefBranch.getBoolPref("firstRun")) {
			_RestMan.rmPrefBranch.setBoolPref("firstRun", false);
			if(_RestMan.appInfos.appVer < 20 && _RestMan.appInfos.appVer >= 3) { // We do this here, because if we put
			// "browser.warnOnRestart" in our defaults file, some versions of pm don't respect user
			// setting, reverting this pref to our default value in every startup.
				_RestMan.brPrefBranch.setBoolPref("warnOnRestart", true);
			};
		};
		if(_RestMan.appInfos.appVer >= 3) {
			_RestMan.restmanSrvc.getSet("versionChecked", "setT"); // set true.
		};
	},

	prevSessionRestObs: function(rpsObsAdd) { // Called if there are pending restoration of user
	// settings and/or to disable our restore shortcut key after "Restore Previous Session".
		var ss = cc["@mozilla.org/browser/sessionstore;1"].getService(ci.nsISessionStore);
		var prvSrv = null; // Used in appVer < 20 & >= 4.
		if(_RestMan.appInfos.appVer < 20) {
			prvSrv = cc["@mozilla.org/privatebrowsing;1"].getService(ci.nsIPrivateBrowsingService);
		};
		if(rpsObsAdd) {
			_RestMan.temps.rpsObs = {
				observe: function(aSubject, aTopic, aData) {
					if(aTopic == "sessionstore-browser-state-restored") {
						function onRPS(event) { // On "Restore Previous Session".
							if(event && (event.type == "SSWindowStateReady")) {
								window.clearTimeout(timeout);
							};
							window.removeEventListener("SSWindowStateReady", onRPS, false);
							var rpsKey = window.document.getElementById("keyRestmanRPS_KeyCombo");
							if(_RestMan.temps.restoreUserSet !== 0) { // After a session restoration,
								// user settings restoration no longer need to be pending - appVer 20+.
								if(!_RestMan.restmanSrvc.getSet("userSettingsRestored", "getV")) { // get value.
									_RestMan.restmanSrvc.getSet("userSettingsRestored", "setT"); // set true.
									_RestMan.onQuitRequest("restUserSet"); // Current window.
									var allBrWinEnum = _RestMan.winMed.getEnumerator("navigator:browser");
									while(allBrWinEnum.hasMoreElements()) {
										var brWin = allBrWinEnum.getNext();
										if(brWin == window) { // Exclude the current window.
											continue;
										};
										brWin._RestMan.temps.restoreUserSet = 0; // Restored windows, if any.
									};
								} else {
									_RestMan.temps.restoreUserSet = 0; // Rest windows, if any.
								};
								if(!rpsKey && _RestMan.temps.rpsObs !== null) {
									_RestMan.temps.rpsObs.remObs();
								};
							};
							if(!ss.canRestoreLastSession) { // We do this check to disable the shortcut only
							// if the session was restored by browser's built-in mechanism, for conformity of
							// the status of the shortcut with browser's built-in menu items for restore last session.
								if(rpsKey) {
									rpsKey.setAttribute("disabled", true);
								};
								if(_RestMan.appInfos.appVer < 27 && _RestMan.appInfos.appVer >= 20) { // Bug in appVer 20-26,
									// where the related menu items remain enabled after "Restore Previous Session".
									window.document.getElementById("Browser:RestoreLastSession").
										setAttribute("disabled", true); // Browser's default command.
								};
								if((_RestMan.appInfos.appVer >= 20 || (_RestMan.appInfos.appVer < 20 && !prvSrv.privateBrowsingEnabled)) && _RestMan.temps.rpsObs !== null) {
								    // In appVer 4-19, if there is a restorable session, the notification of interest is sent always on entering/exiting
									// private browsing mode. We remove our observer only in non-private browsing mode and only if there is not any more
									// a restorable session. Otherwise, the observer is required, to set the "rpsKey" accordingly on entering/exiting
									// private browsing mode.
									_RestMan.temps.rpsObs.remObs();
								};
								window.gBrowser.browsers.forEach(function(browser) { // Fix of a small bug in all appVer 4+:
								// If there are more than one windows with "about:home", when a "Restore Previous Session" is
								// performed in a window, the "Restore Previous Session" button remains visible in rest windows.
									var launcher = browser.contentDocument.getElementById("launcher") || // appVer 15+.
										browser.contentDocument.getElementById("sessionRestoreContainer"); // appVer 4-12.
									if(launcher) {
										if(launcher.hasAttribute("session")) { // appVer 15+.
											launcher.removeAttribute("session");
										} else {
											launcher.setAttribute("hidden", true); // appVer 4-12.
										};
									};
								});
							} else {
								if(rpsKey) { // appVer < 20, on exiting private browsing mode, if there is a restorable session.
									rpsKey.removeAttribute("disabled");
								};
							};
							if(_RestMan.locals.codeMonitoring) {
								var _rpsKey = false;
								var _rpsKeyState = undefined;
								if(rpsKey) {
									_rpsKey = true;
									_rpsKeyState = rpsKey.hasAttribute("disabled") ? "disabled" : "enabled";
								};
								_RestMan.promptSrvc.alert(null, "Session restoration observer:",
									"Topic: " + "\""+aTopic+"\"" + "\nRestore user settings:  " + _RestMan.temps.restoreUserSet +
									 "\n" + "\"rpsKey\"" + ":  " + _rpsKey + ",  State:  " + _rpsKeyState);
							};
						};
						var timeout = window.setTimeout(onRPS, 500);
						window.addEventListener("SSWindowStateReady", onRPS, false);
					};
				},
				addObs: function() {
					_RestMan.obsSrvc.addObserver(this, "sessionstore-browser-state-restored", false);
				},
				remObs: function() {
					_RestMan.obsSrvc.removeObserver(this, "sessionstore-browser-state-restored");
					_RestMan.temps.rpsObs = null;
				}
			};
			_RestMan.temps.rpsObs.addObs();
		};
	},

	browserPrefWin: function(aEvent) {
		if(!aEvent) {
			return;
		} else if(aEvent.type == "load") {
			_RestMan.setLogger();
		};
		var winOpener = window.opener;
		if(!window.opener || // When browser's preferences window is opend in a tab or via command line or other fashion.
		   !("_RestMan" in window.opener)) { // Mac OS or as above.
			winOpener = _RestMan.winMed.getMostRecentWindow("navigator:browser");
		};
		var brStartupType = _RestMan.brPrefBranch.getIntPref("startup.page");
		var restoreUserSet = _RestMan.rmPrefBranch.getIntPref("restore_UserSettings");
		switch(aEvent.type) {
			case "load":
				function setChkBox() {
					var warnCloseChkbx = window.document.getElementById("warnCloseMultiple");
					var warnPrvChkbx = window.document.getElementById("warnOnClosePrvChkbox"); // Appended here by our overlay.
					if((/\(./).test(warnPrvChkbx.label)) {
						warnPrvChkbx.setAttribute("accesskey", warnPrvChkbx.label.match(/\(./)[0].charAt(1));
					}; // Else, in case of Chinese locale e.g., will be applied the predefined in our overlay
					// access key ("P").
					warnCloseChkbx.parentNode.insertBefore(warnPrvChkbx, warnCloseChkbx.nextSibling);
				};
				var tabsPane = window.document.getElementById("paneTabs");
				if(tabsPane.loaded) {
					setChkBox();
				} else {
					tabsPane.addEventListener("paneload", function onPaneload() {
						tabsPane.removeEventListener("paneload", onPaneload, false);
						setChkBox();
					}, false);
				};
				if(restoreUserSet == 0 || brStartupType == -11 || brStartupType == -12) {
					// -11,-12, set by "Session Manager" extension only when/during browser's pref window is open.
					if(winOpener && (brStartupType == -11 || brStartupType == -12)) { // "Session Manager" extension is active.
						(brStartupType == -11) ? winOpener._RestMan.temps.smStartupType = 1 : winOpener._RestMan.temps.smStartupType = 2;
					};
					return; // We have nothing to do..
				} else if(restoreUserSet !== 0) { // We have set temporarily startup.page = 1, while it was 3.
					_RestMan.brPrefBranch.setIntPref("startup.page", 3); // Set 3 as long the pref window is open,
																		 // to not confuse the user.
				};
				break;
			case "unload":
				if(restoreUserSet == 0) {
					if(winOpener) {
						winOpener._RestMan.temps.smStartupType = null;
					};
					return; // We have nothing to do..
				};
				var smStartupType = 0;
				var restoreSmStartupType = false;
				if(_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).prefHasUserValue("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup")) {
				// "Session Manager" extension, installed/active or not. Cheap yet reliable way to check only what we are interested for..
					smStartupType = _RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup");
					restoreSmStartupType = true;
				};
				if(!winOpener) {
					if(brStartupType == 3) { // Restore our temporariry setting until next app quit or a session restoration.
						_RestMan.brPrefBranch.setIntPref("startup.page", 1);
						if(restoreSmStartupType) {
							_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup", smStartupType);
						};
					} else if((brStartupType == 1 || brStartupType == 0) && smStartupType == 0) {
						if(restoreUserSet == 1) {
							_RestMan.brPrefBranch.setBoolPref("warnOnQuit", true); // pm's default.
						};
						_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 0);
					};
					return;
				};
				winOpener.setTimeout(function() {
					if(brStartupType == 3) { // Restore our temporariry setting until next app quit or a session restoration.
						_RestMan.brPrefBranch.setIntPref("startup.page", 1);
						if(restoreSmStartupType) {
							_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup", smStartupType);
						} else {
							this._RestMan.temps.smStartupType = null;
						};
					} else if(brStartupType == 1 || brStartupType == 0) {
						if(this._RestMan.temps.smStartupType == null || smStartupType == 0) {
							if(restoreUserSet == 1) {
								_RestMan.brPrefBranch.setBoolPref("warnOnQuit", true); // pm's default.
							};
							_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 0);
							var allBrWinEnum = _RestMan.winMed.getEnumerator("navigator:browser");
							var delay = 350;
							while(allBrWinEnum.hasMoreElements()) {
								var inEachWin = allBrWinEnum.getNext();
								if(inEachWin._RestMan) {
									inEachWin.setTimeout(function() {
										this._RestMan.temps.restoreUserSet = 0;
										this._RestMan.temps.smStartupType = null;
										if(!_RestMan.rmPrefBranch.getBoolPref("sRestore_shortcut") && this._RestMan.temps.rpsObs !== null) {
											this._RestMan.temps.rpsObs.remObs();
										};
									}, delay);
									delay += 100;
								};
							};
						} else if(this._RestMan.temps.smStartupType !== smStartupType) {
							this._RestMan.temps.smStartupType = null;
						};
					} else {
						this._RestMan.temps.smStartupType = null;
					};
				}, 500);
				break;
		};
	},

	sharedResources: function(aParam) {
	// Defining resources used in dialogs, our preferences window, restart/exit/close window processes.
		if(aParam) {
			_RestMan.onLoadResources();
		};
		var cdBundle = _RestMan.strings.createBundle("chrome://global/locale/commonDialogs.properties");
		var restmanBundle = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
		// "restmanBundle_fbk": Fallback for missing strings of incomplete locales, to get them from "en-US".
		var restmanBundle_fbk = _RestMan.strings.createBundle("chrome://restartmanager_fbk/locale/restartmanager.properties");
		var browserBundle = _RestMan.strings.createBundle("chrome://browser/locale/browser.properties");
		var restmanName = "Restart manager";
		try {
			restmanName = restmanBundle.GetStringFromName("extensions.restartmanager@chania.gr.name"); // Restart manager
		} catch(ex) {
			restmanName = restmanBundle_fbk.GetStringFromName("extensions.restartmanager@chania.gr.name"); // Restart manager
		};
		var optionsOrPrefsStr = browserBundle.GetStringFromName("popupWarningButton"); // Options/Preferences
		_RestMan.locals.optionsTitle = restmanName + " - " + optionsOrPrefsStr; // Restart manager - Options/Preferences
		_RestMan.locals.confirmLabel = cdBundle.GetStringFromName("ConfirmCheck"); // Confirm
		_RestMan.locals.invalCaches = _RestMan.rmPrefBranch.getBoolPref("invalidate_caches");
		_RestMan.locals.codMonChkBoxHid = _RestMan.rmPrefBranch.getBoolPref("codeMonCheckbox_hidden");
		if(_RestMan.appInfos.appVer >= 6) {
			if(_RestMan.appInfos.appVer >= 28) {
				var printdialogBundle = _RestMan.strings.createBundle("chrome://global/locale/printdialog.properties");
				_RestMan.locals.appearanceLabel = printdialogBundle.GetStringFromName("appearanceTitleMac"); // Appearance:
			} else if(_RestMan.appInfos.appVer < 28) {
				var extensionsBundle = _RestMan.strings.createBundle("chrome://mozapps/locale/extensions/extensions.properties");
				_RestMan.locals.appearanceLabel = extensionsBundle.GetStringFromName("type.theme.name") + ":"; // Appearance:
			};
		};
		if(_RestMan.appInfos.appVer >= 20 || _RestMan.appInfos.appVer < 3) {
			_RestMan.locals.askOnRestart = _RestMan.rmPrefBranch.getBoolPref("askOnRestart");
		};
		if(_RestMan.appInfos.appVer >= 20) {
			_RestMan.locals.sessionRestore = _RestMan.rmPrefBranch.getBoolPref("restoreSession_onRestart");
			_RestMan.locals.warnOnCloseMT = _RestMan.rmPrefBranch.getBoolPref("warnOnClose_multipleTabs");
			_RestMan.locals.warnOnCloseMPT = _RestMan.rmPrefBranch.getBoolPref("warnOnClose_multiplePrvTabs");
		};
		if(_RestMan.appInfos.appVer >= 23) {
			var quitBundle = _RestMan.strings.createBundle("chrome://browser/locale/quitDialog.properties");
			_RestMan.locals.neverAskLabel = quitBundle.GetStringFromName("neverAsk2"); // &Do not ask next time
		} else if(_RestMan.appInfos.appVer < 23 && _RestMan.appInfos.appVer >= 20) {
			var quitBundle = _RestMan.strings.createBundle("chrome://browser/locale/quitDialog.properties");
			_RestMan.locals.neverAskLabel = quitBundle.GetStringFromName("neverAsk"); // Do not ask next time
		} else if(_RestMan.appInfos.appVer < 20 && _RestMan.appInfos.appVer >= 3) {
			var quitBundle = _RestMan.strings.createBundle("chrome://browser/locale/quitDialog.properties");
			_RestMan.locals.askOnRestart = _RestMan.brPrefBranch.getBoolPref("warnOnRestart");
			_RestMan.locals.neverAskLabel = quitBundle.GetStringFromName("neverAsk"); // Do not ask next time
		} else if(_RestMan.appInfos.appVer < 3) {
			var askBundle = _RestMan.strings.createBundle("chrome://global/locale/nsProgressDialog.properties");
			var updateBundle = _RestMan.strings.createBundle("chrome://mozapps/locale/update/updates.properties");
			var downloadsBundle = _RestMan.strings.createBundle("chrome://mozapps/locale/downloads/downloads.properties");
			var questionMark = downloadsBundle.GetStringFromName("quitCancelDownloadsAlertTitle").match(/.$/); // ?
			_RestMan.locals.restartMessage = updateBundle.formatStringFromName("restartButton", [_RestMan.appInfos.appName], 1) + questionMark; // Restart brandShortName now
			_RestMan.locals.neverAskLabel = askBundle.GetStringFromName("dontAskAgain");	// Don't ask me again
		};
		_RestMan.temps.sharedResLoaded = true;
	},

	winTabCounter: function RestMan_winTabCounter(aParam, aEvent, aCaller) {
		// Functional module called when/where it is needed, to count windows & tabs
		// and check for private browsing mode, to avoid code repetitions.
		// It should be implemented in a separate jsm file for better code managing but
		// pm 1.5 & 2 don't support js modules..
		var callerName = null; // This function is also called by our component, which works in strict mode.
		// We get the caller of this function as follows, because the "function.caller" will not be
		// allowed in strict mode soon.
		if(aCaller && aCaller == "rro") {
			callerName = "RestMan_RestartRequestObserver";
		} else {
			callerName = RestMan_winTabCounter.caller.name;
		};
		var privateTabExt = false;
		if(_RestMan.locals.codeMonitoring) {
			if(_RestMan.appInfos.appVer >= 20) {
				cu.import("resource://gre/modules/AddonManager.jsm", _RestMan);
				_RestMan.AddonManager.getAddonByID("privateTab@infocatcher", function(pt) {
					if(pt && !pt.appDisabled && !pt.userDisabled && pt.isActive) { // "Private Tab" extension is installed & enabled.
						privateTabExt = true;
					} else {
						privateTabExt = false;
					};
				});
			};
			var suppressCodeMon = {value: false};
			var param1 = aParam ? aParam : "none";
			var param2 = aEvent ? aEvent.type : "none";
			_RestMan.promptSrvc.alertCheck(null, "RestMan_winTabCounter caller:",
				callerName + "\n\n" + "Passed parameters: " + "\naParam: " + param1 + "\naEvent: " + param2,
				"Suppress codeMon for this window", suppressCodeMon);
			if(suppressCodeMon.value) {
				_RestMan.locals.codeMonitoring = false;
			};
		};

		// It is necessary to reset the values below on every call of this function,
		// in case this function has been called again previously in the same scope.
		var allBrWinCount = 0;      // All browser windows.
		var nonPrvWinCount = 0;     // All non-private browser windows.
		var privWinCount = 0;       // All private browser windows.
		var nPrWinPrTbCnt = 0;      // Non-private windows with all their tabs private.
		var nPrWnNprTbCnt = 0;      // Non-private windows with at least 1 non-private tab.
		var allTabCount = 0;        // All tabs.
		var nonPrvTabCount = 0;     // All non-private tabs.
		var nPrvNPndTabCnt = 0;     // All non-private non-pinned tabs.
		var visTabCount = 0;        // All visible tabs.
		var nonPrVisTabCnt = 0;     // All non-private visible tabs.
		var prvTabCount = 0;        // All private tabs.
		var prvVisTabCount = 0;     // All private visible tabs.
		var allPinnedTabs = 0;      // All pinned tabs.
		var prvPinnedTabs = 0;      // All private pinned tabs.
		var allWinPrivate = true;   //
		var allTabsPrivate = false; //
		var allTabCountCW = 0;      // Tabs in current window.
		var nonPrvTabCntCW = 0;     // Non-private tabs in current window.
		var visTabCountCW = 0;      // Visible tabs in current window.
		var prvTabCountCW = 0;      // Private tabs in current window.
		var nPrVisTabCntCW = 0;     // Non-private visible tabs in current window.
		var prvVisTabCntCW = 0;     // Private visible tabs in current window.
		var pinnedTabsCW = 0;       // Pinned tabs in current window.
		var prvPndTabsCW = 0;       // Private pinned tabs in current window.
		var isWinPrivate = true;    // Current window.
		var allTabsPrvCW = false;   // Current window.
		// Note: Visible tabs = tabs in the current tab group.

		/**<Counts for all browser windows.>*************************/
		var allBrowserWinEnum = _RestMan.winMed.getEnumerator("navigator:browser");
		while(allBrowserWinEnum.hasMoreElements()) {
			allBrWinCount++;
			var eachBrowserWin = allBrowserWinEnum.getNext();
			if(_RestMan.appInfos.appVer >= 20) {
			// The js module "PrivateBrowsingUtils.jsm" is already imported by browser.js.
				if(eachBrowserWin.PrivateBrowsingUtils.isWindowPrivate(eachBrowserWin)) {
					privWinCount++;
				};
				if(!eachBrowserWin.PrivateBrowsingUtils.isWindowPrivate(eachBrowserWin)) {
					allWinPrivate = false;
				};
			};
			var tabBrowser = eachBrowserWin.gBrowser; // window.gBrowser.tabs
			if(tabBrowser) {
				allTabCount += tabBrowser.browsers.length;
				if(_RestMan.appInfos.appVer >= 20) {
					visTabCount += tabBrowser.visibleTabs.length;
					allPinnedTabs += tabBrowser._numPinnedTabs;
				};
			};
			if(_RestMan.appInfos.appVer >= 20) {
				Array.forEach(tabBrowser.tabs, function(tabElement) {
					if(tabElement.hasAttribute("privateTab-isPrivate")) { // Compatibility with "Private Tab" extension.
						prvTabCount++;
					};
					if(tabElement.hasAttribute("privateTab-isPrivate") && // Compatibility with "Private Tab" extension.
						tabElement.hasAttribute("pinned")) {
						prvPinnedTabs++;
					};
					if(!tabElement.hasAttribute("privateTab-isPrivate") && // Compatibility with "Private Tab" extension.
						!tabElement.hasAttribute("pinned")) {
						nPrvNPndTabCnt++;
					};
				});
				Array.forEach(tabBrowser.visibleTabs, function(tabElement) {
					if(tabElement.hasAttribute("privateTab-isPrivate")) { // Compatibility with "Private Tab" extension.
						prvVisTabCount++;
					};
				});
				if(!eachBrowserWin.PrivateBrowsingUtils.isWindowPrivate(eachBrowserWin)) {
					// Compatibility with "Private Tab" extension. Detection of non-private
					// windows with all their tabs private.
					var prvTabCountInNonPrvWin = 0;
					var nonPrvTabCountInNonPrvWin = 0;
					Array.forEach(tabBrowser.tabs, function(tabElement) {
						if(tabElement.hasAttribute("privateTab-isPrivate")) {
							prvTabCountInNonPrvWin++;
						};
						if(!tabElement.hasAttribute("privateTab-isPrivate")) {
							nonPrvTabCountInNonPrvWin++;
						};
					});
					if(nonPrvTabCountInNonPrvWin == 0 && prvTabCountInNonPrvWin >= 1) {
						nPrWinPrTbCnt++; // Non-private windows with all their tabs private.
					};					 // Compatibility with "Private Tab" extension.
					if(nonPrvTabCountInNonPrvWin >= 1) {
						nPrWnNprTbCnt++; // Non-private windows with at least 1 non-private tab.
					};					 // Compatibility with "Private Tab" extension.
				};
			};
		};
		if(_RestMan.appInfos.appVer >= 20) {
			nonPrvWinCount = allBrWinCount - privWinCount;
			nonPrvTabCount = allTabCount - prvTabCount;
			nonPrVisTabCnt = visTabCount - prvVisTabCount;
			if(nonPrvTabCount == 0 && prvTabCount >= 1) {
				allTabsPrivate = true;
			};
		};
		/*************************</Counts for all browser windows.>**/

		/**<Counts for current browser window.>*************************/
		var currentWinTabBrowser = window.gBrowser; // window.gBrowser.tabs
		if(currentWinTabBrowser) { // Count tabs / get privacy status of current window.
			allTabCountCW = currentWinTabBrowser.browsers.length;
			if(_RestMan.appInfos.appVer >= 20) {
				if(!window.PrivateBrowsingUtils.isWindowPrivate(window)) {
					isWinPrivate = false;
				};
				visTabCountCW = currentWinTabBrowser.visibleTabs.length;
				pinnedTabsCW = currentWinTabBrowser._numPinnedTabs;
				Array.forEach(currentWinTabBrowser.tabs, function(tabElement) {
					if(tabElement.hasAttribute("privateTab-isPrivate")) { // Compatibility with "Private Tab" extension.
						prvTabCountCW++;
					};
					if(tabElement.hasAttribute("privateTab-isPrivate") && // Compatibility with "Private Tab" extension.
						tabElement.hasAttribute("pinned")) {
						prvPndTabsCW++;
					};
				});
				Array.forEach(currentWinTabBrowser.visibleTabs, function(tabElement) {
					if(tabElement.hasAttribute("privateTab-isPrivate")) { // Compatibility with "Private Tab" extension.
						prvVisTabCntCW++;
					};
				});

				nonPrvTabCntCW = allTabCountCW - prvTabCountCW;
				nPrVisTabCntCW = visTabCountCW - prvVisTabCntCW;
				if(nonPrvTabCntCW == 0 && prvTabCountCW >= 1) {
					allTabsPrvCW = true;
				};
			};
		};
		/*************************</Counts for current browser window.>**/

		if(_RestMan.locals.codeMonitoring) {
			var suppressCodeMon = {value: false};
			if(_RestMan.appInfos.appVer >= 20) {
				if(privateTabExt) { // "Private Tab" extension is installed & enabled.
					_RestMan.promptSrvc.alertCheck(null, "winTabCounter:",
						"-- \"Private Tab\" extension is active --" +
						"\n--------------------------------------------------------" +
						"\n| ------------ Current session: ------------- |" +
						"\n   All browser windows: ------------- " + allBrWinCount +
						"\n   All non-private windows: --------- " + nonPrvWinCount +
						"\n   All private windows: -------------- " + privWinCount +
						"\n   All nonPrvWin/All tabs prv: ------- " + nPrWinPrTbCnt +
						"\n   All nonPrvWin/1+ NonPrv tabs: -- " + nPrWnNprTbCnt +
						"\n   All windows are private: ---------- " + allWinPrivate +
						"\n   All tabs: --------------------------- " + allTabCount +
						"\n   All non-private tabs: -------------- " + nonPrvTabCount +
						"\n   All nonPrivate/nonPinned tabs: --- " + nPrvNPndTabCnt +
						"\n   All visible tabs: -------------------- " + visTabCount +
						"\n   All private tabs: ------------------- " + prvTabCount +
						"\n   All non-private visible tabs: ------ " + nonPrVisTabCnt +
						"\n   All private visible tabs: ----------- " + prvVisTabCount +
						"\n   All pinned tabs: ------------------- " + allPinnedTabs +
						"\n   All private pinned tabs: ----------- " + prvPinnedTabs +
						"\n   All tabs are private: --------------- " + allTabsPrivate +
						"\n| ------------ Current window: ------------ |" +
						"\n   Window is private: ---------------- " + isWinPrivate +
						"\n   Tabs: ------------------------------- " + allTabCountCW +
						"\n   Non-private tabs: ----------------- " + nonPrvTabCntCW +
						"\n   Visible tabs: ----------------------- " + visTabCountCW +
						"\n   Private tabs: ----------------------- " + prvTabCountCW +
						"\n   Non-private visible tabs: --------- " + nPrVisTabCntCW +
						"\n   Private visible tabs: --------------- " + prvVisTabCntCW +
						"\n   Pinned tabs: ----------------------- " + pinnedTabsCW +
						"\n   Private pinned tabs: -------------- " + prvPndTabsCW +
						"\n   All tabs are private: --------------- " + allTabsPrvCW +
						"\n--------------------------------------------------------" +
						"\nNote: Visible tabs = tabs in the current tab group.",
					"Suppress codeMon for this window", suppressCodeMon);
				} else {
					_RestMan.promptSrvc.alertCheck(null, "winTabCounter:",
						"| -------- Current session: ---------- |" +
						"\n   All browser windows: ------ " + allBrWinCount +
						"\n   Non-private windows: ----- " + nonPrvWinCount +
						"\n   Private windows: ----------- " + privWinCount +
						"\n   All windows are private: --- " + allWinPrivate +
						"\n   All tabs: -------------------- " + allTabCount +
						"\n   Visible tabs: ---------------- " + visTabCount +
						"\n   Pinned tabs: ---------------- " + allPinnedTabs +
						"\n| -------- Current window: --------- |" +
						"\n   Window is private: --------- " + isWinPrivate +
						"\n   All tabs: -------------------- " + allTabCountCW +
						"\n   Visible tabs: ---------------- " + visTabCountCW +
						"\n   Pinned tabs: ---------------- " + pinnedTabsCW +
						"\n--------------------------------------------------------" +
						"\nNote: Visible tabs = tabs in the current tab group.",
					"Suppress codeMon for this window", suppressCodeMon);
				};
			} else {
				_RestMan.promptSrvc.alertCheck(null, "winTabCounter:",
					"| ------ Current session: ------ |" +
					"\n   All browser windows: ---- " + allBrWinCount +
					"\n   All tabs: ------------------ " + allTabCount +
					"\n| ------ Current window: ------ |" +
					"\n   All tabs: ------------------ " + allTabCountCW,
				"Suppress codeMon for this window", suppressCodeMon);
			};
			if(suppressCodeMon.value) {
				_RestMan.locals.codeMonitoring = false;
			};
		};
		if(aParam) {
			switch(aParam.toString()) {
				case "getWinTabCount": // Current window, requester is "RestMan_onBrowserWinClose()",
									   // if the window is private or all of its tabs are private. AppVer 20+.
					return [allBrWinCount, allTabCountCW];
				case "getAllTabCountRRO": // Request by our Restart request observer, in our js component. AppVer 3-19.
					return allTabCount;
				case "getCountsPuS": // Request by "addMenuPopUpLis()" on File/App menu "popupshowing" event. All appVer.
					return [nonPrvWinCount, allTabCount, allPinnedTabs, nPrvNPndTabCnt];
				case "getCountsRst": // Request by "_RestManOnRestartRequest()". All appVer.
					return [allWinPrivate, allTabsPrivate, allTabCount, nonPrvTabCount, nPrvNPndTabCnt, allTabCountCW,
							allPinnedTabs, pinnedTabsCW, prvTabCountCW, nPrWnNprTbCnt, isWinPrivate, privWinCount];
				case "getPrvStatusRRO": // Request by our Restart request observer, in our js component. AppVer 20+.
					return [allWinPrivate, allTabsPrivate, nPrvNPndTabCnt, allTabCount, allPinnedTabs];
				case "getCountsQt": // Request by "onQuitRequest()". AppVer 20+.
					return [allTabCount, visTabCount, allPinnedTabs, nonPrvTabCount, allBrWinCount, privWinCount];
				case "getBrWinCount": // Request by "restartDlg3to19()" on restart, if the command is aborted. AppVer 3-19.
					return allBrWinCount;
			};
		};
		if(callerName == "RestMan_onBrowserWinClose" || // By pressing the close button.
			callerName == "RestMan_onCmdBrowserWinClose") { // By Ctrl+Shift+W, "DOMWindowClose" event.
			if(_RestMan.locals.detectOpenDlgs) {
				if(allBrWinCount == 1 && _RestMan.dialogDetector() && (!_RestMan.temps.lastDetectedDlg ||
				   (_RestMan.temps.openedDialog !== _RestMan.temps.lastDetectedDlg))) {
					_RestMan.temps.rejectRequest = true;
					_RestMan.temps.lastDetectedDlg = _RestMan.temps.openedDialog;
					_RestMan.temps.openedDialog.focus();
					_RestMan.temps.openedDialog.addEventListener("unload", function onUnload() {
						this.removeEventListener("unload", onUnload, false);
						_RestMan.temps.lastDetectedDlg = null;
					}, false);
					window.setTimeout(function() { // "hasFocus()" is not that reliable, especially on Linux..
						try { // Unlikely to happen due to this tiny timeframe, but if the target has been
						// closed, we'll be broken in appVer < 15 (TypeError). We'll be broken too if
						// "instantDlg()" is called, resulting in "rejectRequest" to remain "true".
							if(!_RestMan.temps.openedDialog.document.hasFocus()) {
								_RestMan.instantDlg(true); // Comment in "instantDlg()".
							} else {
								_RestMan.blinker(true, 0);
							};
						} catch(ex) {
							if(ex) {
								_RestMan.temps.rejectRequest = false;
								_RestMan.temps.openedDialog = null;

								// Repeat the cycle by emulating the close window request.
								var event = new MouseEvent("close");
								if(_RestMan.onBrowserWinClose(event)) {
									window.close();
								};
							};
						};
					}, 100);
					return;
				};
			};
			if(aParam == "checkNoPrivacy") { // Sent by "onBrowserWinClose()" or "onCmdBrowserWinClose()".
				if((allBrWinCount > 1 && (!isWinPrivate && !allTabsPrvCW)) ||
					(allBrWinCount == 1 && (!isWinPrivate && !allTabsPrvCW && nPrvNPndTabCnt >= 1))) {
					return true;
				};
				return false;
			};
			if(!isWinPrivate && !allTabsPrvCW && aParam == "brHandlesClosing") {
				if(allBrWinCount == 1 && (aEvent.type == "close" || aEvent.type == "command")) {
					_RestMan.onQuitRequest(undefined, aEvent);
				} else if(allBrWinCount > 1 && aEvent.type == "command") {
					window.addEventListener("beforeunload", function(event) { // In case of "onbeforeunload"
					// dialog, a long standing bug in all versions of pm 2-28, causes the dialog to appear
					// twice on "Leave Page", when the window is closed with Ctrl+Shift+W.
						event.stopPropagation();
					}, true);
					window.close(); // Being here, "WindowIsClosing()" (browser.js) has already returned "true".
				};
			} else if(allBrWinCount > 1 && (isWinPrivate || allTabsPrvCW) && aParam == "rmHandlesClosing") {
				if(allTabCountCW == 1 && window.WindowIsClosing()) { // browser.js
					window.addEventListener("beforeunload", function(event) { // Comment above.
						event.stopPropagation();
					}, true);
					window.close();
				} else if(allTabCountCW > 1) {
					_RestMan._RestManOnRestartRequest("close", "handleCloseWin");
				};
			} else if(allBrWinCount == 1 && aParam == "rmHandlesClosing") {
				_RestMan._RestManOnRestartRequest("close", "handleCloseLastWin");
			};
		};
	},

	dialogDetector: function(aParam) {
		// We check if our restart or warning dialog or any other dialog of this
		// type or alert type or our options dialog or pm's prefs dialog or some
		// other dialog type is currently opened, to prevent showing multiple dialogs.
		// Also, some dialogs may need user's attention or intervention.
		// (Note: although most of these dialogs have a "modal" attribute, they can
		// be opened in different windows. So, multiple dialogs is possible.)
		// (Unfortunately, some dialogs, e.g. "commonDialog.xul" & "about.xul", don't have
		// "windowtype" attribute..
		// Here we give them one temporarily, as long as they are loaded, if they are.
		// We do this also for some other child dialogs, opened over their parent dialog.)
		var allWinEnum = _RestMan.winMed.getEnumerator(null);
		while(allWinEnum.hasMoreElements()) {
			var eachWin = allWinEnum.getNext();
			var commonDlg = eachWin.document.getElementById("commonDialog");
			var genericAbout = eachWin.document.getElementById("genericAbout");
			var loaddevice = eachWin.document.getElementById("loaddevice");
			var setpass = eachWin.document.getElementById("set_password");
			var editCaCert = eachWin.document.getElementById("editCaCert");
			var delCert = eachWin.document.getElementById("deleteCertificate");
			var secWarn = eachWin.document.getElementById("SecurityWarnings");
			var sanDiag = eachWin.document.getElementById("SanitizeDialog");
			var selBkmDLg = eachWin.document.getElementById("selectBookmarkDialog");
			var fontsDlg = eachWin.document.getElementById("FontsDialog");
			var colorsDlg = eachWin.document.getElementById("ColorsDialog");
			var langDlg = eachWin.document.getElementById("LanguagesDialog");
			var connDlg = eachWin.document.getElementById("ConnectionsDialog");
			var advJsDlg = eachWin.document.getElementById("AdvancedJSDialog");
			var changeMpDlg = eachWin.document.getElementById("changemp");
			var crlViewDlg = eachWin.document.getElementById("crlviewer");
			var ocspDlg = eachWin.document.getElementById("OCSPDialog");
			var printPageSetup = eachWin.document.getElementById("printPageSetupDialog");
			var pwdedit = eachWin.document.getElementById("pwdedit"); // Password edit dialog of "Saved Password Editor" extension.
			var peOptions = eachWin.document.getElementById("PE_options"); // "Paste Email (original)"
			var pmcOptions = eachWin.document.getElementById("pmcommander-prefs"); // "Pale Moon Commander" extension.
			var status4evarPrefs = eachWin.document.getElementById("status4evar-prefs"); // "Status-4-Evar" extension.
			var location4evarPrefs = eachWin.document.getElementById("location4evar-prefs"); // "Location-4-Evar" extension, Pale Moon.
			var mapaPOptions = eachWin.document.getElementById("mapaPlusOptionsWindow"); // "Master Password+" extension.
			if(commonDlg) {
				commonDlg.setAttribute("windowtype", "dialog:commonDialog");
			};
			if(genericAbout) {
				genericAbout.setAttribute("windowtype", "dialog:genericAbout");
			};
			if(loaddevice) { // Child dialog.
				loaddevice.setAttribute("windowtype", "dialog:loaddevice");
			};
			if(setpass) { // Child dialog.
				setpass.setAttribute("windowtype", "dialog:setpass");
			};
			if(editCaCert) { // Child dialog.
				editCaCert.setAttribute("windowtype", "dialog:editCaCert");
			};
			if(delCert) { // Child dialog.
				delCert.setAttribute("windowtype", "dialog:delCert");
			};
			if(secWarn) { // Child dialog.
				secWarn.setAttribute("windowtype", "Security:Warnings");
			};
			if(sanDiag) { // Child dialog.
				sanDiag.setAttribute("windowtype", "Sanitize:Dialog");
			};
			if(selBkmDLg) { // Child dialog.
				selBkmDLg.setAttribute("windowtype", "selectBookmark:Dialog");
			};
			if(fontsDlg) { // Child dialog.
				fontsDlg.setAttribute("windowtype", "Fonts:Dialog");
			};
			if(colorsDlg) { // Child dialog.
				colorsDlg.setAttribute("windowtype", "Colors:Dialog");
			};
			if(langDlg) { // Child dialog.
				langDlg.setAttribute("windowtype", "Languages:Dialog");
			};
			if(connDlg) { // Child dialog.
				connDlg.setAttribute("windowtype", "Connections:Dialog");
			};
			if(advJsDlg) { // Child dialog.
				advJsDlg.setAttribute("windowtype", "AdvancedJS:Dialog");
			};
			if(changeMpDlg) { // Child dialog.
				changeMpDlg.setAttribute("windowtype", "changemp:Dialog");
			};
			if(crlViewDlg) { // Child dialog.
				crlViewDlg.setAttribute("windowtype", "crlviewer:manager");
			};
			if(ocspDlg) { // Child dialog.
				ocspDlg.setAttribute("windowtype", "OCSP:Dialog");
			};
			if(printPageSetup) {
				printPageSetup.setAttribute("windowtype", "printPage:SetupDialog");
			};
			if(pwdedit) { // "Saved Password Editor" extension, edit dialog.
				pwdedit.setAttribute("windowtype", "spe:pwdedit");
			};
			if(peOptions) { // "Paste Email (original)" extension, options dialog.
				peOptions.setAttribute("windowtype", "pasteemail:options");
			};
			if(pmcOptions) { // "Pale Moon Commander" extension, options dialog.
				pmcOptions.setAttribute("windowtype", "pmcommander:prefs");
			};
			if(status4evarPrefs) { // "Status-4-Evar" extension.
				status4evarPrefs.setAttribute("windowtype", "status4evar:prefs");
			};
			if(location4evarPrefs) { // "Location-4-Evar" extension, Pale Moon.
				location4evarPrefs.setAttribute("windowtype", "location4evar:prefs");
			};
			if(mapaPOptions) { // "Master Password+" extension.
				mapaPOptions.setAttribute("windowtype", "mapaPlus:OptionsWin");
			};
		};
		if(_RestMan.locals.detectOpenDlgs) {
			if(aParam == "onExit") {
					_RestMan.temps.openedDialog = _RestMan.winMed.getMostRecentWindow("dialog:commonDialog");
			} else {
				_RestMan.temps.openedDialog = _RestMan.winMed.getMostRecentWindow("restman:options") || /* Our options dialog. */
					_RestMan.winMed.getMostRecentWindow("dialog:commonDialog") ||
					_RestMan.winMed.getMostRecentWindow("dialog:genericAbout") ||
					_RestMan.winMed.getMostRecentWindow("alert:alert") || /* Desktop notification */
					_RestMan.winMed.getMostRecentWindow("Shell:SetDesktopBackground") ||
					_RestMan.winMed.getMostRecentWindow("Browser:Permissions") ||
					_RestMan.winMed.getMostRecentWindow("dialog:editCaCert") ||
					_RestMan.winMed.getMostRecentWindow("dialog:delCert") ||
					_RestMan.winMed.getMostRecentWindow("printPage:SetupDialog") ||
					_RestMan.winMed.getMostRecentWindow("mozilla:exceptiondialog") ||
					_RestMan.winMed.getMostRecentWindow("mozilla:certmanager") ||
					_RestMan.winMed.getMostRecentWindow("dialog:loaddevice") ||
					_RestMan.winMed.getMostRecentWindow("dialog:setpass") ||
					_RestMan.winMed.getMostRecentWindow("Toolkit:PasswordManagerExceptions") ||
					_RestMan.winMed.getMostRecentWindow("spe:pwdedit") || /* "Saved Password Editor" extension. */
					_RestMan.winMed.getMostRecentWindow("Toolkit:PasswordManager") ||
					_RestMan.winMed.getMostRecentWindow("Security:Warnings") ||
					_RestMan.winMed.getMostRecentWindow("Sanitize:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("selectBookmark:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("Fonts:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("Colors:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("Languages:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("Browser:Cookies") ||
					_RestMan.winMed.getMostRecentWindow("Weave:AccountSetup") ||
					_RestMan.winMed.getMostRecentWindow("Connections:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("mozilla:devicemanager") ||
					_RestMan.winMed.getMostRecentWindow("Preferences:DownloadActions") ||
					_RestMan.winMed.getMostRecentWindow("AdvancedJS:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("changemp:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("crlviewer:manager") ||
					_RestMan.winMed.getMostRecentWindow("OCSP:Dialog") ||
					_RestMan.winMed.getMostRecentWindow("Addons:Compatibility") ||
					_RestMan.winMed.getMostRecentWindow("Browser:Preferences") || /* Browser's pref window. */
					_RestMan.winMed.getMostRecentWindow("Addons:Install") || /* Extensions installation window */
					_RestMan.winMed.getMostRecentWindow("pasteemail:options") || /* "Paste Email (original)" extension. */
					_RestMan.winMed.getMostRecentWindow("pasteemailplus:alert") || /* "Paste Email Plus" extension. */
					_RestMan.winMed.getMostRecentWindow("pasteemailplus:help") || /* ” */
					_RestMan.winMed.getMostRecentWindow("pasteemailplus:options") || /* ” */
					_RestMan.winMed.getMostRecentWindow("pasteemailplus:add") || /* ” */
					_RestMan.winMed.getMostRecentWindow("tiletabs-options") || /* "Tile Tabs" extension. */
					_RestMan.winMed.getMostRecentWindow("Browser:Confmania") || /* "Configuration Mania" extension. */
					_RestMan.winMed.getMostRecentWindow("pmcommander:prefs") || /* "Pale Moon Commander" extension. */
					_RestMan.winMed.getMostRecentWindow("status4evar:prefs") || /* "Status-4-Evar" extension. */
					_RestMan.winMed.getMostRecentWindow("location4evar:prefs") || /* "Location-4-Evar" extension, Pale Moon. */
					_RestMan.winMed.getMostRecentWindow("mapaPlus:OptionsWin") || /* "Master Password+" extension. */
					_RestMan.winMed.getMostRecentWindow("SessionManager:Options"); /* "Session Manager" extension. */
			};
		} else {
			_RestMan.temps.openedDialog = _RestMan.winMed.getMostRecentWindow("restman:options"); // Always detection for presence of our options dialog.
		};
		if(_RestMan.temps.openedDialog !== null) {
			return true;
		};
		return false;
	},

	sessionSaveHelper: function RestMan_sessionSaveHelper(aParam) {
	/*** >>> All user actions to restart or quit the browser from private browsing mode, end up here. <<< ***/
	// In case of a mixed session (normal & private), if the last non-private window
	// is closed and thereafter or after a while pm is quitted or restarted, there
	// are some problems on next startup: some versions (e.g. ffx 23) show the "about:sessionrestore"
	// page almost always, while some other (e.g. pm 27 & 28) cause the session of last non-private
	// window to be lost. PM by design, saves the session of the last non-private window
	// and any other non-private data, only when the last private window is closed (not exit or restart pm).
	// We try here to simulate "closing of the last private window" on pm exit or restart, in order to
	// save the session of the last non-private window, on exit and restart too. It seems to work..  :-)
	// PM 3.5-19 manage very reliably the session, before and after the private browsing mode..
	// Note: This is questionable: "window not closed during a regular shut-down"
	// (SessionStore.jsm in pm 28, onClose(), line 899).
	// Why not to save any non-private data on shutdown (or even on restart) too? SessionStore.jsm filters and
	// excludes the private data, saving only the non-private data, so there are no any security or privacy concerns.
	// So, why not this procedure to not take place also on shutdown or restart? A good answer is wanted  :-)
		if(_RestMan.locals.codeMonitoring) {
			var suppressCodeMon = {value: false};
			var param = aParam ? aParam : "none";
			_RestMan.promptSrvc.alertCheck(null, "RestMan_sessionSaveHelper caller:",
				RestMan_sessionSaveHelper.caller.name + // arguments.callee.caller
				"\n\n" + "Passed parameter: " + param, "Suppress codeMon for this window", suppressCodeMon);
			if(suppressCodeMon.value) {
				_RestMan.locals.codeMonitoring = false;
			};
		};
		if(aParam == "handleExit") {
			var cancelQuit = cc["@mozilla.org/supports-PRBool;1"].createInstance(ci.nsISupportsPRBool);
			_RestMan.restmanSrvc.getSet("internalRequest", "setT"); // set true.
			_RestMan.obsSrvc.notifyObservers(cancelQuit, "quit-application-requested", null);
			if(cancelQuit.data) {
				return false;
			};
		};
		var currentBrWindow = _RestMan.winMed.getMostRecentWindow("navigator:browser");
		var shouldReturn = false;
		if((aParam == "restart") || (aParam == "handleExit") || (aParam == "restartOther") ||
		  (aParam == "restartOtherQuit") || (aParam == "warnOnRestartInSM") || (aParam == "forceRestart")) {
			var allWinEnum = _RestMan.winMed.getEnumerator(null);
			while(allWinEnum.hasMoreElements()) {
				var winToBeClosed = allWinEnum.getNext();
				if(winToBeClosed !== currentBrWindow) { // Close all windows except the current.
					function checkPermitUnload() {
						var tabsInWin = winToBeClosed.gBrowser.browsers;
						for(var br = 0; br < tabsInWin.length; ++br) {
							var ds = tabsInWin[br].docShell;
							if(ds.contentViewer && !ds.contentViewer.permitUnload()) {
								return false;
							};
						};
						return true;
					};
					if(winToBeClosed.document.documentElement.getAttribute("windowtype") == "navigator:browser") {
						if(aParam == "forceRestart") {
							winToBeClosed.close();
							continue;
						};
						if(checkPermitUnload()) {
							winToBeClosed.addEventListener("beforeunload", function(event) {
								event.stopPropagation(); // Otherwise, "window.close()" will trigger browser's
							}, true); // built-in check and will show again the dialog, if any.
							winToBeClosed.close();
						} else {
							shouldReturn = true;
							break;
						};
					} else if(winToBeClosed.document.documentElement.getAttribute("windowtype") !== "devtools:webconsole" &&
							  winToBeClosed.document.documentElement.getAttribute("windowtype") !== "devtools:scratchpad") {
						winToBeClosed.close();
					};
				};
			};
			if(shouldReturn) {
				return false;
			};
		};
		// currentBrWindow.close();
		// We prefer to let the browser manage the last window closing, to do first its default
		// checks, instead of a "cooldown" window.close(). 
		// First, we close all tabs one-by-one, to prevent pm of showing its warning dialog,
		// if applicable and if tabs > 1, on our restart or exit process. We show our own warning
		// dialog (optional) specifically in private browsing mode, because pm has only one option
		// to show/hide its warning dialog on closing multiple tabs, which applies in both private
		// and non-private browsing mode.
		// It is important to have a warning dialog in all cases (close window, exit, restart) in
		// private or mixed browsing mode, since private windows/sessions can not be restored.
		// In pm >= 26, there is a warning dialog on exiting or closing the last window in private
		// browsing mode, but as mentioned above, its option to hide it is applied also on the quit
		// dialog in non-private browsing mode.
		// So, we take the control, show our own warning dialog in all cases in private browsing mode
		// and save the session of the last non-private window.
		var tabBrowser = currentBrWindow.gBrowser;
		var tabs = tabBrowser.tabs;
		var sTab = tabBrowser.selectedTab;
		for(var t = tabs.length - 1; t >= 0; --t) { // Close all tabs one-by-one except the current.
			var ds = tabBrowser.getBrowserForTab(tabs[t]).docShell;
			if(tabs[t] !== sTab && ds.contentViewer) { // && !tabs[t].hasAttribute("pinned")
				if(aParam == "forceRestart") {
					tabBrowser.removeTab(tabs[t], {animate: false});
					continue;
				};
				if(ds.contentViewer.permitUnload()) {
					if(_RestMan.appInfos.appVer < 28) {
						tabBrowser.getBrowserForTab(tabs[t]).addEventListener("beforeunload", function(event) {
							event.stopPropagation();
						}, true);
					};
					var domWinUtils = tabBrowser.getBrowserForTab(tabs[t]).contentWindow.
						QueryInterface(ci.nsIInterfaceRequestor).getInterface(ci.nsIDOMWindowUtils);
					if(_RestMan.appInfos.appVer >= 27) {
						domWinUtils.disableDialogs();
					} else {
						domWinUtils.preventFurtherDialogs();
					};
					tabBrowser.removeTab(tabs[t], {animate: false});
				} else {
					shouldReturn = true;
					break;
				};
			};
		};
		if(shouldReturn) {
			return false;
		};
		function onlastBrWinClose() {
			currentBrWindow.addEventListener("unload", function ssHelperOnUnload() { // This is the last browser
				this.removeEventListener("unload", ssHelperOnUnload, false);		 // window that will be closed.
				if(_RestMan.locals.codeMonitoring && !_RestMan.restmanSrvc.getPrvBrowsingAutostart()) {
					_RestMan.promptSrvc.alert(null, "sessionSaveHelper:",
											"The last private browser window is unloaded." +
											"\nThe session of the last non-private window has been saved.");
				};
				function checkMozNoRemote() { // If the profile has been started with "-no-remote" parameter, we
				// clear the "MOZ_NO_REMOTE" environment variable on restart if this is the only running app instance.
					if(_RestMan.restmanSrvc.getSet("startedWithNoRemote", "getV")) { // get value.
						var getInfo = undefined;
						try {
							getInfo = new CustomEvent("getInfo"); // "CustomEvent()" constructor, appVer ~11+.
						} catch(ex) {
							getInfo = window.document.createEvent("Event");
							getInfo.initEvent("getInfo", true, true);
						};
						if(_RestMan.restmanProfMan(getInfo)[1] < 2) { // Number of profiles in use.
							if((_RestMan.locals.clrMnrOnProfile == 0) && (aParam !== "handleExit") && (aParam !== "handleCloseLastWin")) {
								_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
							} else if((_RestMan.locals.clrMnrOnProfile == 1) && ((aParam == "restart") || (aParam == "forceRestart") ||
								(aParam == "warnOnRestartInSM"))) { // Restart in "Safe Mode".
								_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
							} else if((_RestMan.locals.clrMnrOnProfile == 2) && ((aParam == "restartOther") || (aParam == "restartOtherQuit"))) {
								_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
							};
						};
					};
				};
				if((aParam == "restart") || (aParam == "forceRestart") || (aParam == "restartOther")) {
					if(aParam == "restart") {
						// If "InvalidateCachesOnRestart" is enabled, this keeps it so after restart.
						_RestMan.restmanSrvc.getSet("restartRequested", "setT"); // set true.
					};
					if(_RestMan.locals.clrMnrOnRestart) { // Clear "MOZ_NO_REMOTE" on restart.
						checkMozNoRemote();
					};
					_RestMan.appStartup.quit(ci.nsIAppStartup.eAttemptQuit | ci.nsIAppStartup.eRestart);
				} else if((aParam == "handleExit") || (aParam == "restartOtherQuit")) {
					_RestMan.appStartup.quit(ci.nsIAppStartup.eAttemptQuit);
				} else if(aParam == "warnOnRestartInSM") {
					if(_RestMan.locals.clrMnrOnRestart) { // Clear "MOZ_NO_REMOTE" on restart.
						checkMozNoRemote();
					};
					_RestMan.appStartup.restartInSafeMode(ci.nsIAppStartup.eAttemptQuit);
				};
			}, false);
		};
		if(aParam == "forceRestart") {
			onlastBrWinClose();
			currentBrWindow.close();
		} else {
			var lastTab = tabBrowser.selectedTab;
			var ds = tabBrowser.getBrowserForTab(lastTab).docShell;
			if((_RestMan.winMed.getMostRecentWindow("devtools:scratchpad") && ds.contentViewer &&
				ds.contentViewer.permitUnload()) || currentBrWindow.WindowIsClosing()) { // browser.js
				onlastBrWinClose();
				tabBrowser.getBrowserForTab(lastTab).addEventListener("beforeunload", function(event) {
					event.stopPropagation();
				}, true);
				if(_RestMan.temps.restoreUserSet !== 0) {
					if((aParam == "restart") || (aParam == "handleCloseLastWin") ||
					  (aParam == "warnOnRestartInSM") || (aParam == "restartOther") || (aParam == "restartOtherQuit")) {
						var ss = cc["@mozilla.org/browser/sessionstore;1"].getService(ci.nsISessionStore);
						if(!ss.canRestoreLastSession || (aParam == "warnOnRestartInSM")) {
							_RestMan.onQuitRequest("restUserSet");
						};
					};
				};
				currentBrWindow.close();
				return true;
			};
			return false;
		};
	},

	_RestManOnRestartRequest: function RestMan_RestManOnRestartRequest(aCallReason, aParam, aCaller, aParam2) {
	/* This is our restart logic according to user actions, preferences, conditions and pm versions:
	**	case1("askOnRestart", default):
	**		If all tabs > 1, will be shown our restart dialog, with options "Save and restart" (the session
	**		will be restored automatically after restart), "Restart" (the session will not be restored
	**		automatically after restart, "Cancel". If there is 1 tab, no dialog, this tab will be restored
	**		after restart.
	**	case2("!askOnRestart" && "sessionRestore"):
	**		No dialog, the session will be restored automatically after restart.
	**	case3("!askOnRestart" && "!sessionRestore" && "warnOnCloseMT"):
	**		If all tabs > 1, will be shown our "Warn me when closing multiple tabs" dialog, with options
	**		"Close tabs", "Cancel". Else no dialog, the session will not be restored automatically after restart.
	**	case4("!askOnRestart" && "!sessionRestore" && "!warnOnCloseMT"):
	**		No dialogs, the session will not be restored automatically after restart.
	**	case5("warnOnCloseMPT", default, In private browsing mode):
	**		If all private tabs > 1, will be shown our "Warn me when closing multiple tabs" dialog, with
	**		options "Close tabs", "Cancel". Else no dialog, the private session will be discarded during restart
	**		due to the nature of private browsing mode.
	**	case6("!warnOnCloseMPT", In private browsing mode):
	**		No dialog, the private session will be discarded during restart due to the nature of private browsing
	**		mode.
	**	case7, "Force restart":
	**		Can be initiated only by our toolbar button, by holding it pressed for the specified delay (default 2.5 sec).
	**		This method tries to suppress any window or content modal dialogs, pop-ups, warnings and other things, which
	**		may prevent closing a tab or exiting the browser.
	**		In normal browsing mode, after restart shows the "Restore Session" page with all session items deselected,
	**		to help the user select only what he really wants to restore.
	**		In private browsing mode, after restart will be available the last non-private session, if any.
	*/

		if(_RestMan.temps.rejectRequest && (aParam !== "forceRestart")) {
			return;
		};
		var callerName = null; // This function is called also by our component, which works in strict mode.
		// We get the caller of this function as follows, because the "function.caller" will not be
		// allowed in strict mode soon.
		if(aCaller) {
			if(aCaller == "rro") {
				callerName = "RestMan_RestartRequestObserver";
			} else {
				callerName = aCaller;
			};
		} else if(RestMan_RestManOnRestartRequest.caller) {
			callerName = RestMan_RestManOnRestartRequest.caller.name;
		} else {
			callerName = "top level..!";
		};
		if(_RestMan.locals.codeMonitoring) {
			var suppressCodeMon = {value: false};
			var param = aCallReason ? aCallReason : "none";
			var param2 = aParam ? aParam : "none";
			var param3 = aParam2 ? aParam2 : "none";
			_RestMan.promptSrvc.alertCheck(null, "RestMan_RestManOnRestartRequest caller:",
				callerName + "\n\nPassed parameters:" + "\nCall reason: " + param +
				"\nParameter 2: " + param2 + "\nParameter 3: " + param3, "Suppress codeMon for this window", suppressCodeMon);
			if(suppressCodeMon.value) {
				_RestMan.locals.codeMonitoring = false;
			};
		};
		if(_RestMan.locals.detectOpenDlgs && (aParam !== "forceRestart")) {
			if(_RestMan.dialogDetector() && (aCallReason == "restart") && (!_RestMan.temps.lastDetectedDlg ||
			   (_RestMan.temps.openedDialog !== _RestMan.temps.lastDetectedDlg))) {
				_RestMan.temps.rejectRequest = true;
				_RestMan.temps.lastDetectedDlg = _RestMan.temps.openedDialog;
				try { // Necessary in appVer < 3. It throws exception if the dialog
				// is non-focusable, because of a child modal dialog over it.
					_RestMan.temps.openedDialog.focus();
					_RestMan.temps.openedDialog.addEventListener("unload", function onUnload() {
						this.removeEventListener("unload", onUnload, false);
						_RestMan.temps.lastDetectedDlg = null;
					}, false);
				} catch(ex) {
					if(ex) {
						_RestMan.instantDlg(true); // Comment in "instantDlg()".
						return;
					};
				};
				if(_RestMan.appInfos.appVer >= 3) {
					window.setTimeout(function() { // "hasFocus()" is not that reliable, especially on Linux..
						try { // Unlikely to happen due to this tiny timeframe, but if the target has been
						// closed, we'll be broken in appVer < 15 (TypeError). We'll be broken too if
						// "instantDlg()" is called, resulting in "rejectRequest" to remain "true".
							if(!_RestMan.temps.openedDialog.document.hasFocus()) {
								_RestMan.instantDlg(true); // Comment in "instantDlg()".
							} else {
								_RestMan.blinker(true, 0);
							};
						} catch(ex) {
							if(ex) {
								_RestMan.temps.rejectRequest = false;
								_RestMan.temps.openedDialog = null;
								RestMan_RestManOnRestartRequest(aCallReason, aParam, callerName); 
							}; // Repeat invocation with same parameters.
						};
					}, 100);
				} else {
					_RestMan.blinker(true, 0);
				};
				return;
			};
		};

		var countsRst = _RestMan.winTabCounter("getCountsRst");
			// [allWinPrivate, allTabsPrivate, allTabCount, nonPrvTabCount, nPrvNPndTabCnt, allTabCountCW,
			// allPinnedTabs, pinnedTabsCW, prvTabCountCW, nPrWnNprTbCnt, isWinPrivate, privWinCount];

		if(aParam == "showWarnDlg") {
			return doRestartNoRestore(aParam); // Used as a callback here.
		};
		var cancelQuit = cc["@mozilla.org/supports-PRBool;1"].createInstance(ci.nsISupportsPRBool);

		if(aParam == "doRestart") {
			if(!aParam2) {
				doRestartCmd();
				return;
			} else if((aParam2 == "restartOther") || (aParam2 == "restartOtherQuit")) {
				return doRestartCmd(undefined, aParam2);
			};
		} else if(aParam == "doRestartSM") {
			doRestartCmd(true);
			return;
		} else if((aParam == "doRestartNoRestore") || ((aParam == "forceRestart") && !countsRst[0] && !countsRst[1]) ||
			(aParam == "doRestartNoRestoreSM")) {									// allWinPrivate[0]/allTabsPrivate[1].
			if(!aParam2) {
				doRestartNoRestore(aParam);
				return;
			} else if((aParam2 == "restartOther") || (aParam2 == "restartOtherQuit")) {
				return doRestartNoRestore(aParam, aParam2);
			};
		};

		var selectedAppLocale = _RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getCharPref("general.useragent.locale");
		if(_RestMan.appInfos.appLang !== selectedAppLocale) {
			_RestMan.sharedResources(true);
		} else if(!_RestMan.temps.sharedResLoaded) {
			_RestMan.sharedResources();
		};

		var privateBrModeFlag = false; // Used by warning dialog, to manage the correct preference.
		var warningDlgFlag = false; // Used to determine if the warning dialog will be shown.
		// No restart dialog if there are no at least 2 non-private tabs in non-private window(s)
		// (excluding pinned tabs).
		// In private & non-private or mixed browsing mode, we show also the warning dialog,
		// if this has been selected in options ("Warn me when closing multiple tabs").
		// The "privateBrModeFlag" is used to determine the browsing mode, to change the
		// corresponding preference if the check box of the warning dialog is unchecked.
		// It is used also as an indicator, to perform our special actions on pm restarting/exiting
		// from within private browsing mode.

		if(_RestMan.appInfos.appVer >= 20) {
			function codeMon() {
				if(_RestMan.locals.codeMonitoring) {
					var suppressCodeMon = {value: false};
					var param = aParam ? aParam : "none";
					_RestMan.promptSrvc.alertCheck(null, "Private browsing mode flags:",
						"privateBrModeFlag: " + privateBrModeFlag + 
						"\nwarningDlgFlag: " + warningDlgFlag +
						"\n(Call reason: " + aCallReason + ")" +
						"\n(Parameter: " + param + ")",
						"Suppress codeMon for this window", suppressCodeMon);
					if(suppressCodeMon.value) {
						_RestMan.locals.codeMonitoring = false;
					};
				};
			};

			if(aCallReason !== "close" && ((countsRst[0] || countsRst[1]) || // allWinPrivate[0]/allTabsPrivate[1].
				(countsRst[4] == 0 && (countsRst[2] - countsRst[6]) > 1))) { // At least 2 non-pinned private tabs.
				// nPrvNPndTabCnt[4]/allTabCount[2]/allPinnedTabs[6].
				privateBrModeFlag = true;
				if((aCallReason == "restart") && (aParam == "forceRestart")) { // Passed only by our ui restart button.
					codeMon();
					doRestartNoRestoreCmd(aParam);
					return;
				};
				if(!_RestMan.locals.warnOnCloseMPT) { // Multiple private tabs.
					codeMon();
					if((aCallReason == "restart") && ((aParam == "restart") || // Passed by our ui restart items or our observer.
					  (aParam == "restartOther") || (aParam == "restartOtherQuit"))) { // Passed by our ui restart items.
						if((aParam == "restartOther") || (aParam == "restartOtherQuit")) {
							_RestMan.onQuitRequest(aParam);
							return doRestartNoRestore("restart", aParam);
						} else {
							doRestartNoRestore(aParam);
							return;
						};
					} else if((aCallReason == "exit") && (aParam == "handleExit")) { // On "File --> Exit", private browsing mode,
						_RestMan.onQuitRequest(aParam);
						return;
					};
				} else {
					warningDlgFlag = true;
					codeMon();
					if(((aCallReason == "restart") && ((aParam == "restart") || // Passed by our ui restart items or our observer.
					  (aParam == "restartOther") || (aParam == "restartOtherQuit"))) || // Passed by our ui restart items.
					  ((aCallReason == "exit") && (aParam == "handleExit"))) { // On "File --> Exit", private browsing mode.
						if((aParam == "restartOther") || (aParam == "restartOtherQuit")) {
							return doRestartNoRestore("restart", aParam);
						} else {
							doRestartNoRestore(aParam);
							return;
						};
					};
				};
			};
			if(callerName == "RestMan_winTabCounter" && aCallReason == "close") {
				// We are here if the window to be closed is private or all of its tabs are private
				// or it's the last browser window which is private or all of its tabs are private.
				privateBrModeFlag = true;
				if(_RestMan.locals.warnOnCloseMPT) { // Multiple private tabs.
					warningDlgFlag = true;
				};
				codeMon();
				if(aParam == "handleCloseLastWin") {
					var sqw = _RestMan.brPrefBranch.getBoolPref("showQuitWarning");
					var twoc =  _RestMan.brPrefBranch.getBoolPref("tabs.warnOnClose");
					_RestMan.brPrefBranch.setBoolPref("showQuitWarning", false);
					_RestMan.brPrefBranch.setBoolPref("tabs.warnOnClose", false);
					_RestMan.obsSrvc.notifyObservers(cancelQuit, "browser-lastwindow-close-requested", null);
					if(sqw) {
						_RestMan.brPrefBranch.setBoolPref("showQuitWarning", true);
					};
					if(twoc) {
						_RestMan.brPrefBranch.setBoolPref("tabs.warnOnClose", true);
					};
					if(!cancelQuit.data) {
						doRestartNoRestore(aParam);
						return;
					};
				} else { // "aParam" will be "handleCloseWin".
					doRestartNoRestore(aParam);
					return;
				};
			};
		};

		if(aCallReason == "restart") { // Passed by our ui restart items or our observer.
		// Non-private browsing mode, all supported app versions.
			if((aParam == "restart") || // Passed by our ui restart items or our observer.
			  (aParam == "restartOther") || (aParam == "restartOtherQuit")) { // Passed by our ui restart items.
				if(_RestMan.locals.askOnRestart) {
					return showRestartDlg(aParam);
				} else if(_RestMan.appInfos.appVer >= 20) {
					if(_RestMan.locals.sessionRestore) {
						return doRestartCmd(undefined, aParam);
					} else {
						if(_RestMan.locals.warnOnCloseMT && (countsRst[2] - countsRst[6]) > 1) { // Multiple tabs, allTabCount[2]/allPinnedTabs[6].
							warningDlgFlag = true;
							if(_RestMan.locals.codeMonitoring) {
								var suppressCodeMon = {value: false};
								_RestMan.promptSrvc.alertCheck(null, "Non-private browsing mode flag:",
									"warningDlgFlag: " + warningDlgFlag, "Suppress codeMon for this window", suppressCodeMon);
								if(suppressCodeMon.value) {
									_RestMan.locals.codeMonitoring = false;
								};
							};
						};
						return doRestartNoRestore("doRestartNoRestore", aParam);
					};
				} else if(_RestMan.appInfos.appVer < 20) {
					if(_RestMan.appInfos.appVer >= 3.5) {
						return doRestartCmd(undefined, aParam);
					} else {
						doRestartCmd();
					};
				};
			 } else if(aParam == "forceRestart" && (_RestMan.appInfos.appVer < 20 && _RestMan.appInfos.appVer >= 4)) { // Passed only by our ui restart button.
				doRestartNoRestoreCmd(aParam);
			 } else if(aParam == "safeMode") { // Passed by "Restart in Safe Mode…" menu items in "Help" menus.
				if(!privateBrModeFlag) {
					if((countsRst[2] - countsRst[6]) < 2) { // allTabCount[2]/allPinnedTabs[6].
						if(_RestMan.appInfos.appVer >= 27) {
							_RestMan.restmanSrvc.observerUnregister("smDlg");
							// If "InvalidateCachesOnRestart" is enabled, this disables it on restart in safe mode.
							_RestMan.restmanSrvc.getSet("restartRequested", "setF"); // set false.
							restart(true); // browser.js
							_RestMan.restmanSrvc.observerRegister(); // If the dialog of "restart()" is accepted, the notification "quit-application-granted"
							// will set "true" the flag "weAreQuitting", preventing our observer to be registered again (in our js component).
						} else {
							// If "InvalidateCachesOnRestart" is enabled, this disables it on restart in safe mode.
							_RestMan.restmanSrvc.getSet("restartRequested", "setF"); // set false.
							safeModeRestart(); // browser.js - In app versions < 27, there is no "quit-application-requested" notification.
						};
						if(_RestMan.restmanSrvc.getSet("weAreQuitting", "getV") && _RestMan.temps.restoreUserSet !== 0) {
							_RestMan.onQuitRequest("restUserSet");
						};
					} else {
						restartDlg20plus(true);
					};
				} else {
					doRestartNoRestore("warnOnRestartInSM");
				};
			};
		};

		function permitWinUnload() {
			if(privateBrModeFlag) { // In private browser mode, this check is carried out by "sessionSaveHelper()".
				return true;
			};
			var allBrWinEnum = _RestMan.winMed.getEnumerator("navigator:browser");
			while(allBrWinEnum.hasMoreElements()) {
				var winToCheck = allBrWinEnum.getNext();
				for(var nTab = 0; nTab < winToCheck.gBrowser.browsers.length; ++nTab) {
					var ds = winToCheck.gBrowser.browsers[nTab].docShell;
					if(ds.contentViewer && !ds.contentViewer.permitUnload()) {
						return false;
					}; 
				};
			};
			// If the above check is passed, we proceed to restart and we don't want to retrigger any
			// "onbeforeunload" dialogs at the moment of restart command.
			var allBrWinEnum2 = _RestMan.winMed.getEnumerator("navigator:browser");
			while(allBrWinEnum2.hasMoreElements()) {
				allBrWinEnum2.getNext().addEventListener("beforeunload", function(event) {
					event.stopPropagation();
				}, true);
			};
			return true;
		};

		function doRestartCmd(safeMode, aParam) {
			// Session will be restored after restart (except the part of the session which is private).
			// (Pm/ffx 1.5 can't restore the session by any means..)
			if(_RestMan.locals.codeMonitoring) {
				var suppressCodeMon = {value: false};
				var param1 = safeMode ? "Safe Mode: true" : "Safe Mode: false";
				var param2 = aParam ? "Parameter 2: " + aParam : "Parameter 2: none";
				_RestMan.promptSrvc.alertCheck(null, "doRestartCmd caller:",
					doRestartCmd.caller.name + // arguments.callee.caller
					"\n\n" + "Passed parameters:" + "\n" + param1 + "\n" + param2,
					"Suppress codeMon for this window", suppressCodeMon);
				if(suppressCodeMon.value) {
					_RestMan.locals.codeMonitoring = false;
				};
			};
			if(_RestMan.appInfos.appVer >= 20) {
				_RestMan.restmanSrvc.getSet("internalRequest", "setT"); // set true.
			};
			_RestMan.obsSrvc.notifyObservers(cancelQuit, "quit-application-requested", "restart");
			if(cancelQuit.data || !permitWinUnload()) { // The quit request has been canceled.
				return false;
			};
			if((_RestMan.appInfos.appVer >= 20) && (_RestMan.temps.restoreUserSet !== 0)) {
				var ss = cc["@mozilla.org/browser/sessionstore;1"].getService(ci.nsISessionStore);
				if(!ss.canRestoreLastSession || safeMode || ((aParam == "restartOther") || (aParam == "restartOtherQuit"))) {
					_RestMan.onQuitRequest("restUserSet");
				};
			};
			if((_RestMan.appInfos.appVer >= 4) && ((aParam == "restartOther") || (aParam == "restartOtherQuit"))) { // If "InvalidateCachesOnRestart"
			// is enabled, this disables it. We do it on restart in other profile since the current profile exits.
				_RestMan.restmanSrvc.getSet("restartRequested", "setF"); // set false.
			};
			if(_RestMan.appInfos.appVer >= 2) {
				_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_session_once", true);
			};
			if(_RestMan.locals.invalCaches) {
				if(_RestMan.appInfos.appVer >= 20) {
					_RestMan.appInfo.QueryInterface(ci.nsIXULRuntime).invalidateCachesOnRestart();
				} else if(_RestMan.appInfos.appVer < 3) { // For pm v3-19, this is performed in "restartDlg3to19".
					var cacheFiles = ["compreg.dat", "xpti.dat", "extensions.cache", "extensions.ini"]; // "extensions.rdf", "pluginreg.dat"
					cacheFiles.forEach(function(name) {
						var profDirFiles = _RestMan.dirSrvc.get("ProfD", ci.nsIFile);
						if(profDirFiles) {
							profDirFiles.append(name);
						};
						if(profDirFiles.exists()) {
							profDirFiles.remove(false);
						};
					});
				};
			};
			if((_RestMan.appInfos.appVer >= 3.5) && _RestMan.locals.clrMnrOnRestart) { // Clear "MOZ_NO_REMOTE" on restart.
				if(_RestMan.restmanSrvc.getSet("startedWithNoRemote", "getV")) { // get value.
					var getInfo = undefined;
					try {
						getInfo = new CustomEvent("getInfo"); // "CustomEvent()" constructor, appVer ~11+.
					} catch(ex) {
						getInfo = window.document.createEvent("Event");
						getInfo.initEvent("getInfo", true, true);
					}; // If the profile has been started with "-no-remote" parameter, we clear the "MOZ_NO_REMOTE"
					// environment variable on restart if this is the only running app instance.
					if(_RestMan.restmanProfMan(getInfo)[1] < 2) { // Number of profiles in use.
						if(_RestMan.locals.clrMnrOnProfile == 0) {
							_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
						} else if((_RestMan.locals.clrMnrOnProfile == 1) && ((safeMode) || ((aParam !== "restartOther") && (aParam !== "restartOtherQuit")))) {
							_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
						} else if((_RestMan.locals.clrMnrOnProfile == 2) && ((aParam == "restartOther") || (aParam == "restartOtherQuit"))) {
							_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
						};
					};
				};
			};
			var quitType = (aParam !== "restartOtherQuit") ? ci.nsIAppStartup.eAttemptQuit | ci.nsIAppStartup.eRestart : ci.nsIAppStartup.eAttemptQuit;
			!safeMode ? _RestMan.appStartup.quit(quitType) :
						// If "InvalidateCachesOnRestart" is enabled, this disables it on restart in safe mode.
						(_RestMan.restmanSrvc.getSet("restartRequested", "setF"), // set false.
						_RestMan.appStartup.restartInSafeMode(ci.nsIAppStartup.eAttemptQuit));
			window.setTimeout(function() {
				if(window._RestMan && _RestMan.appInfos.appVer >= 2) { // If we are still alive, someone has aborted the quit command..
					_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_session_once", false);
				};
			}, 1500);
			if((aParam == "restartOther") || (aParam == "restartOtherQuit")) {
				return true;
			};
		};

		function doRestartNoRestoreCmd(aParam, aParam2) {
			// Non-private session will be retained for possible desirable "Restore Previous Session" action,
			// but will not be restored automatically after restart.
			if(_RestMan.locals.codeMonitoring) {
				var suppressCodeMon = {value: false};
				var param1 = aParam ? "Parameter 1: " + aParam : "Parameter 1: none";
				var param2 = aParam2 ? "Parameter 2: " + aParam2 : "Parameter 2: none";
				_RestMan.promptSrvc.alertCheck(null, "doRestartNoRestoreCmd caller:",
					doRestartNoRestoreCmd.caller.name + // arguments.callee.caller
					"\n\n" + "Passed parameters:" + "\n" + param1 + "\n" + param2,
					"Suppress codeMon for this window", suppressCodeMon);
				if(suppressCodeMon.value) {
					_RestMan.locals.codeMonitoring = false;
				};
			};

			if(aParam !== "forceRestart" && aParam !== "salvageRestart") {
				_RestMan.restmanSrvc.getSet("internalRequest", "setT"); // set true.
				_RestMan.obsSrvc.notifyObservers(cancelQuit, "quit-application-requested", "restart");
				if(cancelQuit.data || !permitWinUnload()) { // The quit request has been canceled.
					if(!privateBrModeFlag) {
						_RestMan.temps.dnsPrefObserver.observerUnregister();
					};
					return false;
				};
			};
			if((_RestMan.appInfos.appVer >= 4) && ((aParam2 == "restartOther") || (aParam2 == "restartOtherQuit"))) { // If "InvalidateCachesOnRestart"
			// is enabled, this disables it. We do it on restart in other profile since the current profile exits.
				_RestMan.restmanSrvc.getSet("restartRequested", "setF"); // set false.
			};

			// If "browser.startup.page" = 3, the session will be restored, even if
			// "sessionstore.resume_session_once = false". This does not serve us here for our feature
			// "restart no session restore" to work..
			// We check this pref, change it if necessary and restore it after "restart no session restore", 
			// on app quit or session restoration.
			// Also, maybe it is needed to set "warnOnQuit = false".
			// We have to check also for presence of "Session Manager" extension and change
			// temporarily its startup type if necessary, for our "restart no restore" to work.
			// All the details for our settings restoration process, can be found below, in "onQuitRequest()".
			if(aParam == "checkStartupType" || aParam == "checkStartupTypeSM") { // This has a meaning only in
			//  non-private sessions. In case of a mixed session, the private part of it will be lost as usual.
				var brStartupType = _RestMan.brPrefBranch.getIntPref("startup.page");
				var smStartupType = 0;
				var brSettings2rest = 0;
				if(_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).prefHasUserValue("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup")) {
					// "Session Manager" extension, installed/active or not. Cheap yet reliable way to check only
					// what we are interested for..
					smStartupType = _RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup");
				};
				if(brStartupType == 3) {
					_RestMan.brPrefBranch.setIntPref("startup.page", 1); // pm's default.
					if(_RestMan.brPrefBranch.getBoolPref("warnOnQuit")) { // pm's default.
						_RestMan.brPrefBranch.setBoolPref("warnOnQuit", false);
						brSettings2rest = 1;
					} else {
						brSettings2rest = 2;
					};
				};
				if(smStartupType !== 0) {
					_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup", 0);
				};
				if(brSettings2rest == 1 && smStartupType == 0) {
					_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 1);
				} else if(brSettings2rest == 2 && smStartupType == 0) {
					_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 2);
				} else if(brSettings2rest == 1 && smStartupType == 1) {
					_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 3);
				} else if(brSettings2rest == 2 && smStartupType == 1) {
					_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 4);
				} else if(brSettings2rest == 1 && smStartupType == 2) {
					_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 5);
				} else if(brSettings2rest == 2 && smStartupType == 2) {
					_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 6);
				} else if(brSettings2rest == 0 && smStartupType == 1) {
					_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 7);
				} else if(brSettings2rest == 0 && smStartupType == 2) {
					_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 8);
				};
			};
			if(_RestMan.locals.invalCaches) {
				_RestMan.appInfo.QueryInterface(ci.nsIXULRuntime).invalidateCachesOnRestart();
			};
			if(aParam == "forceRestart") { // With these settings, we prepare the land to make the browser think it has crashed,
			// to show the "about:sessionrestore" page. The second step will be carried out by our observer after restart. First
			// of all, we backup the current settings, to restore them after a "forced restart".
				var prvBrMode = false; // Private browsing mode.
				var showRsPage = true; // Show "Restore Session" page.
				if((_RestMan.appInfos.appVer >= 20 && privateBrModeFlag) || (_RestMan.appInfos.appVer < 20 && _RestMan.appInfos.appVer >= 4 &&
					cc["@mozilla.org/privatebrowsing;1"].getService(ci.nsIPrivateBrowsingService).privateBrowsingEnabled)) {
					prvBrMode = true;
					showRsPage = false;
				};
				var currentVal1 = _RestMan.brPrefBranch.getBoolPref("sessionstore.resume_from_crash");
				var currentVal2 = _RestMan.brPrefBranch.getIntPref("sessionstore.max_resumed_crashes");
				var currentVal3 = false;
				if(!prvBrMode && _RestMan.appInfos.appVer >= 19) { // If dev toolbar is open, don't show it after a "forced restart",
				// because it interferes with our process of "about:sessionrestore" page setup, in some old/weak systems.
					_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setBoolPref("devtools.toolbar.visible", false);
				};
				var allBrWinEnum = _RestMan.winMed.getEnumerator("navigator:browser");
				if(_RestMan.appInfos.appVer >= 26 && _RestMan.appInfos.appId == "{8de7fcbb-c55c-4fbe-bfc5-fc555c87dbc4}") { // Pale Moon's v25+ id.
					currentVal3 = _RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getBoolPref("dom.disable_beforeunload");
					_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setBoolPref("dom.disable_beforeunload", true);
					if(prvBrMode) {
						while(allBrWinEnum.hasMoreElements()) {
							var inEachWin = allBrWinEnum.getNext();
							inEachWin._RestMan.obsSrvc.notifyObservers(inEachWin, "domwindowclosed", null);
						};
					};
				} else {
					while(allBrWinEnum.hasMoreElements()) {
						var inEachWin = allBrWinEnum.getNext();
						inEachWin.addEventListener("beforeunload", function(event) {
							event.stopPropagation();
						}, true);
						if(prvBrMode) {
							inEachWin._RestMan.obsSrvc.notifyObservers(inEachWin, "domwindowclosed", null);
						};
					};
				};
				var currentVal4 = _RestMan.brPrefBranch.getBoolPref("warnOnQuit");
				var currentVal5 = true;
				var sessMan = false;
				if(_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getPrefType("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.use_browser_crash_prompt") == 128) { // PREF_BOOL.
				// For simplicity, we ckeck only for the presence of the pref of interest, regardless if "Session Manager" is installed/active or not.
					currentVal5 = _RestMan.prefSrvc.getBranch("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.").getBoolPref("use_browser_crash_prompt");
					sessMan = true;
				};
				_RestMan.rmPrefBranch.setCharPref("forced_to_restart", "["+showRsPage+", "+currentVal1+", "+currentVal2+", "+currentVal3+", "+currentVal4+", "+currentVal5+"]");
				_RestMan.brPrefBranch.setBoolPref("warnOnQuit", false);
				if(!prvBrMode) {
					_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_from_crash", true);
					_RestMan.brPrefBranch.setIntPref("sessionstore.max_resumed_crashes", 0);
					if(sessMan) {
						_RestMan.prefSrvc.getBranch("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.").setBoolPref("use_browser_crash_prompt", true);
					};
				};
				if(_RestMan.locals.clrMnrOnRestart && ((_RestMan.locals.clrMnrOnProfile == 0) || (_RestMan.locals.clrMnrOnProfile == 1))) {
					if(_RestMan.restmanSrvc.getSet("startedWithNoRemote", "getV")) { // get value.
						var getInfo = undefined;
						try {
							getInfo = new CustomEvent("getInfo"); // "CustomEvent()" constructor, appVer ~11+.
						} catch(ex) {
							getInfo = window.document.createEvent("Event");
							getInfo.initEvent("getInfo", true, true);
						}; // If the profile has been started with "-no-remote" parameter, we clear the "MOZ_NO_REMOTE"
						// environment variable on restart if this is the only running app instance.
						if(_RestMan.restmanProfMan(getInfo)[1] < 2) { // Number of profiles in use.
							_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
						};
					};
				};
				// If "InvalidateCachesOnRestart" is enabled, this keeps it so after restart.
				_RestMan.restmanSrvc.getSet("restartRequested", "setT"); // set true.
				_RestMan.obsSrvc.notifyObservers(null, "quit-application-granted", null); // Required in appVer >= 24.
				if(!prvBrMode || _RestMan.appInfos.appVer < 20) {
					_RestMan.appStartup.quit(ci.nsIAppStartup.eForceQuit | ci.nsIAppStartup.eRestart);
				} else if(_RestMan.appInfos.appVer >= 20 && prvBrMode) {
					_RestMan.sessionSaveHelper(aParam);
				};
				window.setTimeout(function() { // If we are still alive, the attempt to "force restart" failed miserably for
				// unknown/uncontrollable reason.. If this happens, a "force restart" must not be attempted second time, because
				// the setting changes we made will become permanent. To avoid this, we already have removed the "force restart"
				// command from our button (in "doForceRestart()") and now we restore the normal restart command.
					if(window._RestMan) {
						var restManBtn = window.document.getElementById("restman-button");
						restManBtn.setAttribute("command", "cmd_restmanRestartApp");

						// Prompt the user to perform a normal restart right now, to complete the failed/pending
						// "forced restart" and restore changed settings.
						var restmanBundle = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
						// "restmanBundle_fbk": Fallback for missing strings of incomplete locales, to get them from "en-US".
						var restmanBundle_fbk = _RestMan.strings.createBundle("chrome://restartmanager_fbk/locale/restartmanager.properties");
						var promptTitle = "";
						try {
							promptTitle = restmanBundle.GetStringFromName("failedForceRestart.title");
						} catch(ex) {
							promptTitle = restmanBundle_fbk.GetStringFromName("failedForceRestart.title");
						};
						var promptMessage = "";
						try {
							promptMessage = restmanBundle.GetStringFromName("failedForceRestart.msg");
						} catch(ex) {
							promptMessage = restmanBundle_fbk.GetStringFromName("failedForceRestart.msg");
						};
						var browserBundle = _RestMan.strings.createBundle("chrome://browser/locale/browser.properties");
						var restartText = browserBundle.GetStringFromName("addonInstallRestartButton"); // Restart Now.
						var sounds = cc["@mozilla.org/sound;1"].createInstance(ci.nsISound);
						var buttonFlags = _RestMan.promptSrvc.BUTTON_TITLE_IS_STRING * _RestMan.promptSrvc.BUTTON_POS_0 +
										  _RestMan.promptSrvc.BUTTON_TITLE_CANCEL    * _RestMan.promptSrvc.BUTTON_POS_1 +
										  _RestMan.promptSrvc.BUTTON_POS_0_DEFAULT;
						sounds.playEventSound(1); // "confirmEx" is silent. We want an alert type of prompt.
						var userButtonChoice = _RestMan.promptSrvc.confirmEx(window, promptTitle, promptMessage,
																			 buttonFlags, restartText, null, null,
																			 null, {});
						if(userButtonChoice !== 0) {
							return;
						} else {
							if(!cancelQuit.data) {
								doRestartNoRestoreCmd("salvageRestart");
							};
						};
					};
				}, 1500);
			} else if(privateBrModeFlag && (aParam == "restart" || aParam == "warnOnRestartInSM")) {
				var aParameter = ((aParam2 == "restartOther") || (aParam2 == "restartOtherQuit")) ? aParam2 : aParam;
				if(!_RestMan.sessionSaveHelper(aParameter)) {
					return false;
				};
			} else {
				if(_RestMan.locals.clrMnrOnRestart) { // Clear "MOZ_NO_REMOTE" on restart.
					if(_RestMan.restmanSrvc.getSet("startedWithNoRemote", "getV")) { // get value.
						var getInfo = undefined;
						try {
							getInfo = new CustomEvent("getInfo"); // "CustomEvent()" constructor, appVer ~11+.
						} catch(ex) {
							getInfo = window.document.createEvent("Event");
							getInfo.initEvent("getInfo", true, true);
						}; // If the profile has been started with "-no-remote" parameter, we clear the "MOZ_NO_REMOTE"
						// environment variable on restart if this is the only running app instance.
						if(_RestMan.restmanProfMan(getInfo)[1] < 2) { // Number of profiles in use.
							if(_RestMan.locals.clrMnrOnProfile == 0) {
								_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
							} else if((_RestMan.locals.clrMnrOnProfile == 1) && !aParam2) {
								_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
							} else if((_RestMan.locals.clrMnrOnProfile == 2) && ((aParam2 == "restartOther") || (aParam2 == "restartOtherQuit"))) {
								_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
							};
						};
					};
				};
				var quitType = (aParam2 !== "restartOtherQuit") ? ci.nsIAppStartup.eAttemptQuit | ci.nsIAppStartup.eRestart : ci.nsIAppStartup.eAttemptQuit;
				(aParam !== "checkStartupTypeSM") ?
					_RestMan.appStartup.quit(quitType) :
					// If "InvalidateCachesOnRestart" is enabled, this disables it on restart in safe mode.
					(_RestMan.restmanSrvc.getSet("restartRequested", "setF"), // set false.
					_RestMan.appStartup.restartInSafeMode(ci.nsIAppStartup.eAttemptQuit));
			};
			if(!privateBrModeFlag) {
				window.setTimeout(function() {
					if(window._RestMan) { // If we are still alive, someone has aborted the quit command..
						_RestMan.temps.dnsPrefObserver.observerUnregister();
					};
				}, 1500);
			};
			if((aParam2 == "restartOther") || (aParam2 == "restartOtherQuit")) {
				if(privateBrModeFlag && _RestMan.locals.warnOnCloseMPT) { // Multiple private tabs.
					_RestMan.onQuitRequest(aParam2);
				};
				return true;
			};
		};

		function doRestartNoRestore(aParam, aParam2) {
			// The session will not be restored after restart, but will be saved, in case
			// the user wants to "Restore Previous Session" (excluding a private session or 
			// the part of the session which is private).
			// Session's state will be saved just before restart, for the case of a change
			// and thereafter restart, before the expiration of current session save interval
			//(default interval = 15secs).

			if(_RestMan.locals.codeMonitoring) {
				var suppressCodeMon = {value: false};
				var param1 = aParam ? "Parameter 1: " + aParam : "Parameter 1 : none";
				var param2 = aParam2 ? "Parameter 2: " + aParam2 : "Parameter 2: none";
				_RestMan.promptSrvc.alertCheck(null, "doRestartNoRestore caller:",
					doRestartNoRestore.caller.name // arguments.callee.caller
					+ "\n\n" + "Passed parameters:" + "\n" + param1 + "\n" + param2,
					"Suppress codeMon for this window", suppressCodeMon);
				if(suppressCodeMon.value) {
					_RestMan.locals.codeMonitoring = false;
				};
			};
			if(aParam == "showWarnDlg") {
				return warningDialog(aParam); // Used as a callback here.
			};
			if(aParam == "warnOnRestartInSM") {
				warningDialog(aParam);
				return;
			};

			var doRestartNoRestorePrefObserver = {
			// We consider the use of a pref observer as more reliable and app friendly,
			// instead of locking the pref of interest.
				pref2Observe: "sessionstore.resume_session_once",
				observe: function(aSubject, aTopic, aData) {
					if(_RestMan.locals.codeMonitoring) {
						_RestMan.promptSrvc.alert(null, "doRestartNoRestore observe:",
							"Subject: " + aSubject + "\nTopic: " + aTopic + "\nData: " + aData);
					};
					if((aTopic == "nsPref:changed") && (aData == this.pref2Observe)) {
						if(_RestMan.brPrefBranch.getBoolPref(this.pref2Observe)) {
							_RestMan.brPrefBranch.setBoolPref(this.pref2Observe, false);
						};
					};
				},
				observerRegister: function() {
					if(_RestMan.brPrefBranch.getBoolPref(this.pref2Observe)) {
						_RestMan.brPrefBranch.setBoolPref(this.pref2Observe, false);
					};
					if(!_RestMan.temps.dnsPrefObserver) {
						_RestMan.brPrefBranch.addObserver(this.pref2Observe, this, false);
						_RestMan.temps.dnsPrefObserver = doRestartNoRestorePrefObserver;
						if(_RestMan.locals.codeMonitoring) {
							var suppressCodeMon = {value: false};
							_RestMan.promptSrvc.alertCheck(null, "doRestartNoRestore pref observer:", "Registered",
							"Suppress codeMon for this window", suppressCodeMon);
							if(suppressCodeMon.value) {
								_RestMan.locals.codeMonitoring = false;
							};
						};
					};
				},
				observerUnregister: function() {
					if(_RestMan.temps.dnsPrefObserver) {
						_RestMan.brPrefBranch.removeObserver(this.pref2Observe, this);
						_RestMan.temps.dnsPrefObserver = null;
						if(_RestMan.locals.codeMonitoring) {
							_RestMan.promptSrvc.alert(null, "doRestartNoRestore pref observer:", "Unregistered");
						};
					};
				}
			};

			// allWinPrivate[0]/allTabsPrivate[1].
			if(!countsRst[0] && !countsRst[1] && ((aParam == "doRestartNoRestore") || // Compatibility with
			// "Private Tab" extension. We may have non-private windows but all of their tabs may be private.
				(aParam == "forceRestart") || (aParam == "doRestartNoRestoreSM"))) {
				function checkURIs() { // Check URI of the non-pinned tab.
					var tabs = window.gBrowser.tabs;
					for(var t = 0; t < tabs.length; t++) {
						if(!tabs[t].hasAttribute("pinned") &&
							tabs[t].linkedBrowser.currentURI.spec.toLowerCase() !== "about:home" &&
							tabs[t].linkedBrowser.currentURI.spec.toLowerCase() !== "about:blank" &&
							tabs[t].linkedBrowser.currentURI.spec.toLowerCase() !== "about:logopage") {
							return true;
						}; // "gBrowser.getBrowserAtIndex(tabs[t])" works the same.
					};
					return false;
				};
				if(((countsRst[4] == 1) && checkURIs()) || (countsRst[4] > 1)) { // nPrvNPndTabCnt[4].
					doRestartNoRestorePrefObserver.observerRegister();
				};
				if(aParam == "forceRestart") {
					doRestartNoRestoreCmd(aParam);
					return;
				};
			};
			if(privateBrModeFlag) {
				if(countsRst[2] < 2 || !warningDlgFlag) { // allTabCount[2].
					if(aParam == "handleCloseWin") {
						var twoc =  _RestMan.brPrefBranch.getBoolPref("tabs.warnOnClose");
						_RestMan.brPrefBranch.setBoolPref("tabs.warnOnClose", false);
						if(window.WindowIsClosing()) {
							window.addEventListener("beforeunload", function(event) { // In case of
							// "onbeforeunload" dialog, a long standing bug in all versions of pm 2-28,
							// causes the dialog to appear twice on "Leave Page", when the window is
							// closed with Ctrl+Shift+W.
								event.stopPropagation();
							}, true);
							window.close();
						};
						if(twoc) {
							_RestMan.brPrefBranch.setBoolPref("tabs.warnOnClose", true);
						};
						return;
					} else if((aParam == "handleCloseLastWin") || (aParam == "handleExit")) {
						_RestMan.onQuitRequest(aParam);
						return;
					} else if((aParam == "restart") || (aParam == "restartOther") || (aParam == "restartOtherQuit")) {
						return doRestartNoRestoreCmd(aParam, aParam2);
					};
				} else if(warningDlgFlag && ((aParam == "restart") || (aParam == "handleCloseWin") ||
					(aParam == "handleCloseLastWin") || (aParam == "handleExit"))) {
						return warningDialog(aParam, aParam2);
				};
			} else {
				if(((countsRst[2] - countsRst[6]) < 2) || !warningDlgFlag) { // allTabCount[2]/allPinnedTabs[6].
					if((aParam2 == "restartOther") || (aParam2 == "restartOtherQuit")) {
						return doRestartNoRestoreCmd("checkStartupType", aParam2);
					} else {
						(aParam !== "doRestartNoRestoreSM") ? 
							doRestartNoRestoreCmd("checkStartupType") :
							doRestartNoRestoreCmd("checkStartupTypeSM");
						return;
					};
				} else {
					return warningDialog(aParam, aParam2);
				};
			};

			function warningDialog(aParam, aParam2) { // This is our warning dialog, used in six different cases,
			// with the corresponding title and message (appVer 20+):
			// 1.- On "restart no restore" in non-private browsing mode, if the restart dialog has been suppressed.
			// 2.- On closing a private window with > 1 tabs.
			// 3.- On closing the last browser window if it is private or all of its tabs are private (by using
			//   "Private tab" extension) with > 1 tabs.
			// 4.- On exiting pm from within private browsing mode, if there are more than one tabs.
			//   This is important in our opinion, because private windows get lost for good after closing them.
			//   Pm > 25 does show a warning dialog also in private browsing mode (on window close only or also
			//   on exit if "browser.showQuitWarning" is true), but its option to suppress it is common in both
			//   private and non-private browsing mode. We apply separate options in our warning dialogs in
			//   private and non-private browsing mode.
			// 5.- On restarting from within private browsing mode with > 1 tabs.
			// 6.- In all cases of non-private browsing mode, where pm does not show a dialog on exit or close
			//   the last browser window, by design. This is for those users who want to prevent unintentional
			//   app exits (optional).
			// We treat a non-private window as private, if there are no non-private tabs (excluded pinned) and at
			// least two private non-pinned tabs (when "Private tab" extension is used).
				if(_RestMan.locals.codeMonitoring) {
					var suppressCodeMon = {value: false};
					var param1 = aParam ? aParam : "none";
					var param2 = aParam2 ? aParam2 : "none";
					_RestMan.promptSrvc.alertCheck(null, "warningDialog caller:",
						warningDialog.caller.name + // arguments.callee.caller
						"\n\n" + "Passed parameters:" + "\nParameter 1: " + param1 + "\nParameter 2: " + param2,
						"Suppress codeMon for this window", suppressCodeMon);
					if(suppressCodeMon.value) {
						_RestMan.locals.codeMonitoring = false;
					};
				};

				/**<Localized strings, used in warning dialog.>*************************/
				var tabbrBundle = _RestMan.strings.createBundle("chrome://browser/locale/tabbrowser.properties");
				var quitBundle = _RestMan.strings.createBundle("chrome://browser/locale/quitDialog.properties");
				var warnDlgTitle = null;
				if(aParam == "restart" || aParam == "doRestartNoRestore") {
					warnDlgTitle = _RestMan.locals.restartTitle + " - " + tabbrBundle.GetStringFromName("tabs.closeWarningTitle"); // Restart brandShortName - Confirm close
				} else if(aParam == "handleExit" || // Passed by our js component on "File --> Exit".
					aParam == "handleCloseLastWin") {
					warnDlgTitle = quitBundle.formatStringFromName("quitDialogTitle", [_RestMan.appInfos.appName], 1) + " - " +
								   _RestMan.locals.confirmLabel; // Quit brandShortName - Confirm
				} else if(aParam == "handleCloseWin") {
					warnDlgTitle = _RestMan.winMed.getMostRecentWindow("navigator:browser").document.getElementById("closeWinLabel").value + // Close Window
								   " - " + _RestMan.locals.confirmLabel; // Confirm (--> Close Window - Confirm)
				} else if(aParam == "showWarnDlg") {
					warnDlgTitle = tabbrBundle.GetStringFromName("tabs.closeWarningTitle"); // Confirm close
				} else if(aParam == "warnOnRestartInSM") {
					warnDlgTitle = gNavigatorBundle.getString("safeModeRestartPromptTitle") + " - " +
								   tabbrBundle.GetStringFromName("tabs.closeWarningTitle"); // Restart in Safe Mode - Confirm close
				};
				var text1 = _RestMan.winMed.getMostRecentWindow("navigator:browser").document.getElementById("privateBrowsingTitle").value.replace(/[()]/g,""); // Private Browsing
				var tabsCount = 0;
				if(aParam == "restart" || aParam == "doRestartNoRestore" || aParam == "handleExit" ||
				   aParam == "showWarnDlg" || aParam == "warnOnRestartInSM") {
					if(privateBrModeFlag) {
						if((countsRst[2] - countsRst[3]) == 0 || countsRst[0]) { // allTabCount[2]/nonPrvTabCount[3]/allWinPrivate[0].
							tabsCount = countsRst[2];
						} else if((countsRst[2] - countsRst[3]) > 1) { // At least 2 non-pinned private tabs.
							tabsCount = (countsRst[2] - countsRst[3]);
						};
					} else {
						tabsCount = (countsRst[2] - countsRst[6]); // allTabCount[2]/allPinnedTabs[6].
					};
				} else if(aParam == "handleCloseWin" || aParam == "handleCloseLastWin") {
					if(privateBrModeFlag) {
						if((countsRst[2] - countsRst[3]) == 0 || countsRst[10]) { // isWinPrivate[10], current window.
							tabsCount = countsRst[5]; // allTabCountCW[5], current window.
						} else if((countsRst[2] - countsRst[3]) >= 1) { // allTabCount[2]/nonPrvTabCount[3].
							tabsCount = countsRst[8]; // prvTabCountCW[8], current window.
						};
					} else {
						tabsCount = (countsRst[5] - countsRst[7]); // allTabCountCW[5]/pinnedTabsCW[7], current window.
					};
				};
				var text2 = tabbrBundle.formatStringFromName("tabs.closeWarningMultipleTabs", [tabsCount], 1); // "You are about to close xx tabs .,,"
				var restmanBundle = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
				// "restmanBundle_fbk": Fallback for missing strings of incomplete locales, to get them from "en-US".
				var restmanBundle_fbk = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
				if(((countsRst[2] - countsRst[3]) >= 1 || countsRst[11] >= 1) && (aCallReason == "restart" || aParam == "showWarnDlg") && !privateBrModeFlag) {
					// allTabCount[2]/nonPrvTabCount[3]/privWinCount[11].
					if((countsRst[2] - countsRst[3]) == 0) {
						try {text2 = text2 + "\n" + // ("All private windows (n) will be discarded anyway" message).
							restmanBundle.formatStringFromName("prvWinWarning.label", [countsRst[11]], 1);
						} catch(ex) {text2 = text2 + "\n" + // ("All private windows (n) will be discarded anyway" message).
							restmanBundle_fbk.formatStringFromName("prvWinWarning.label", [countsRst[11]], 1);
						};
					} else {
						try {ttext2 = text2 + "\n" + // ("All private tabs (n) will be discarded anyway" message).
							restmanBundle.formatStringFromName("prvTabsWarning.label", [countsRst[2] - countsRst[3]], 1);
						} catch(ex) {ttext2 = text2 + "\n" + // ("All private tabs (n) will be discarded anyway" message).
							restmanBundle_fbk.formatStringFromName("prvTabsWarning.label", [countsRst[2] - countsRst[3]], 1);
						};	
					};
				};
				if(_RestMan.appInfos.appVer >= 26 && _RestMan.appInfos.appId == "{8de7fcbb-c55c-4fbe-bfc5-fc555c87dbc4}") { // Pale Moon's v25+ id.
				// We welcome and make use of the new string for the message in quit pm or close last window dialog, in private
				// browsing mode, in pm >= 26, inherited from ffx 34 ("browser.showQuitWarning" = true). However, moz-devs made
				// for second time the same mistake, as in the restart dialog in ffx 3-19: This message is a statement, not a
				// question, while the label in the checkbox to hide the dialog says "Do not ask next time"!
				// When we use this message in our warning dialog, we combine it with the "Warn me..." label in the checkbox..
					var text3 = quitBundle.formatStringFromName("messagePrivate", [_RestMan.appInfos.appName], 1); // "You're in private browsing mode. Quitting ..."
					var openedTabsLabel = "";
					if((countsRst[2] - countsRst[3]) >= 1) { // allTabCount[2]/nonPrvTabCount[3].
						try {
							openedTabsLabel = restmanBundle.GetStringFromName("privateTabs.label") + ": "; // Private tabs:
						} catch(ex) {
							openedTabsLabel = restmanBundle_fbk.GetStringFromName("privateTabs.label") + ": "; // Private tabs:
						};
					} else {
						openedTabsLabel = _RestMan.strings.createBundle("chrome://browser/locale/syncQuota.properties").
											GetStringFromName("collection.tabs.label") + ": "; // Tabs:
					};
				};
				var warnMe = (aParam == "warnOnRestartInSM") ? null : "&" + tabbrBundle.GetStringFromName("tabs.closeWarningPromptMe"); // "Warn me when..."
				var button0Title = tabbrBundle.GetStringFromName("tabs.closeButtonMultiple"); // Close tabs
				var _dlgMessage = "";
				var _defaultButton;
					if(privateBrModeFlag) {
						_defaultButton = _RestMan.promptSrvc.BUTTON_POS_1_DEFAULT;
						if(_RestMan.appInfos.appVer >= 26 && _RestMan.appInfos.appId == "{8de7fcbb-c55c-4fbe-bfc5-fc555c87dbc4}") { // Pale Moon's v25+ id.
							if(aParam == "restart" || aParam == "handleExit" ||
							   aParam == "handleCloseLastWin" || aParam == "warnOnRestartInSM") {
								_dlgMessage = text1 + "\n" + "(" + openedTabsLabel + tabsCount + ")" + "\n" + text3;
							} else if(aParam == "handleCloseWin") {
								_dlgMessage = text1 + "\n\n" + text2;
							};
						} else {
							_dlgMessage = text1 + "\n\n" + text2;
						};
					} else {
						_dlgMessage = text2;
						_defaultButton = _RestMan.promptSrvc.BUTTON_POS_0_DEFAULT;
					};
				/*************************</Localized strings, used in warning dialog.>**/

				var chkBox = {value: true};
				var flags = _RestMan.promptSrvc.BUTTON_TITLE_IS_STRING * _RestMan.promptSrvc.BUTTON_POS_0 +
							_RestMan.promptSrvc.BUTTON_TITLE_CANCEL    * _RestMan.promptSrvc.BUTTON_POS_1 +
							_defaultButton;
				var mostRecentBrowserWindow = _RestMan.winMed.getMostRecentWindow("navigator:browser");
				var userButtonChoice = _RestMan.promptSrvc.confirmEx(
						mostRecentBrowserWindow, warnDlgTitle, _dlgMessage,
						flags, button0Title, null, null, warnMe, chkBox
						);
				switch(userButtonChoice) {
					case 1: // Cancel close or restart, close dialog.
						if(aParam !== "showWarnDlg") {
							if(_RestMan.temps.dnsPrefObserver) {
								doRestartNoRestorePrefObserver.observerUnregister();
							};	
							if(privateBrModeFlag) {
								privateBrModeFlag = false;
							};
							warningDlgFlag = false;
						};
						break;
					case 0: // Close tabs/windows & restart or exit, if it's the last window.
						if(!chkBox.value) {
							if(aParam !== "showWarnDlg") {
								if(!privateBrModeFlag) {
									_RestMan.rmPrefBranch.setBoolPref("warnOnClose_multipleTabs", false);
								} else {
									_RestMan.rmPrefBranch.setBoolPref("warnOnClose_multiplePrvTabs", false);
								};
							} else {
								_RestMan.brPrefBranch.setBoolPref("tabs.warnOnClose", false);
								_RestMan.brPrefBranch.setBoolPref("tabs.warnOnCloseOtherTabs", false);
							};
						};
						if(aParam !== "showWarnDlg") {
							if(privateBrModeFlag) {
								if((aParam == "restart") || (aParam == "warnOnRestartInSM")) {
									return doRestartNoRestoreCmd(aParam, aParam2);
								} else if((aParam == "handleExit") || (aParam == "handleCloseLastWin")) {
									_RestMan.onQuitRequest(aParam);
								} else if(aParam == "handleCloseWin") {
									var twoc =  _RestMan.brPrefBranch.getBoolPref("tabs.warnOnClose");
									_RestMan.brPrefBranch.setBoolPref("tabs.warnOnClose", false);
									if(window.WindowIsClosing()) { // browser.js
										window.addEventListener("beforeunload", function(event) { // In case of
										// "onbeforeunload" dialog, a long standing bug in all versions of pm 2-28,
										// causes the dialog to appear twice on "Leave Page", when the window is
										// closed with Ctrl+Shift+W.
											event.stopPropagation();
										}, true);
										window.close();
									};
									if(twoc) {
										_RestMan.brPrefBranch.setBoolPref("tabs.warnOnClose", true);
									};
								};
							} else if(aParam == "doRestartNoRestore") {
								return doRestartNoRestoreCmd("checkStartupType", aParam2);
							};
						};
						break;
				};
				if(aParam == "showWarnDlg") {
					return userButtonChoice;
				} else if(((aParam2 == "restartOther") || (aParam2 == "restartOtherQuit")) && (userButtonChoice == 1)) {
					return false;
				};
			};
		};

		function restartDlg20plus(safeMode, aParam) {
			/*  Pm >= 20 don't have a restart dialog anymore..
				Our restart dialog here is based on the quit dialog.
			********************************************************/
			if(countsRst[4] < 2 && (countsRst[2] - countsRst[6]) < 2) {
				// nPrvNPndTabCnt[4]/allTabCount[2]/allPinnedTabs[6].
				if((aParam == "restartOther")  || (aParam == "restartOtherQuit")) {
					return RestMan_RestManOnRestartRequest("restart", "doRestart", undefined, aParam);
				} else {
					RestMan_RestManOnRestartRequest("restart", "doRestart");
					return;
				};
			};
			/*<Localized strings, used in restart dialog.>*********/
			var cdBundle = _RestMan.strings.createBundle("chrome://global/locale/commonDialogs.properties");
			var prefsBundle = _RestMan.strings.createBundle("chrome://browser/locale/preferences/preferences.properties");
			var quitBundle = _RestMan.strings.createBundle("chrome://browser/locale/quitDialog.properties");
			var restartDlgTitle = !safeMode ? _RestMan.locals.restartTitle + (" - ") + _RestMan.locals.confirmLabel : // Restart brandShortName - Confirm
				gNavigatorBundle.getString("safeModeRestartPromptTitle") + (" - ") + _RestMan.locals.confirmLabel; // Restart in Safe Mode - Confirm
			var browserBundle = _RestMan.strings.createBundle("chrome://browser/locale/browser.properties");
			var dlgMessage = "";
			var currentAppLang = _RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getCharPref("general.useragent.locale");
			// Pm shows the "message" message even when, while there are at least 2 windows,
			// only 1 of them is non-private. However, private windows can not survive after quiting
			// or restarting..
			// We show the "message" message only when there are at least 2 non-private windows, with
			// at least 1 non-private tab in each one (Compatibility with "Private Tab" extension).
			if(countsRst[9] == 1) { // nPrWnNprTbCnt[9] - non-private windows with non-private tabs.
				dlgMessage = quitBundle.formatStringFromName("messageNoWindows", [_RestMan.appInfos.appName], 1);
			} else {
				dlgMessage = quitBundle.formatStringFromName("message", [_RestMan.appInfos.appName], 1);
			};
			if((countsRst[2] - countsRst[3]) >= 1 || countsRst[11] >= 1) { // allTabCount[2], nonPrvTabCount[3]/privWinCount[11].
				var restmanBundle = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
				// "restmanBundle_fbk": Fallback for missing strings of incomplete locales, to get them from "en-US".
				var restmanBundle_fbk = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
				if((countsRst[2] - countsRst[3]) == 0) {
					try {dlgMessage = dlgMessage + "\n" + // ("All private windows (n) will be discarded anyway" message).
						restmanBundle.formatStringFromName("prvWinWarning.label", [countsRst[11]], 1);
					} catch(ex) {dlgMessage = dlgMessage + "\n" + // ("All private windows (n) will be discarded anyway" message).
						restmanBundle_fbk.formatStringFromName("prvWinWarning.label", [countsRst[11]], 1);
					};
				} else {
					try {dlgMessage = dlgMessage + "\n" + // ("All private tabs (n) will be discarded anyway" message).
						restmanBundle.formatStringFromName("prvTabsWarning.label", [countsRst[2] - countsRst[3]], 1);
					} catch(ex) {dlgMessage = dlgMessage + "\n" + // ("All private tabs (n) will be discarded anyway" message).
						restmanBundle_fbk.formatStringFromName("prvTabsWarning.label", [countsRst[2] - countsRst[3]], 1);
					};
				};
			};
			var saveAndquit = quitBundle.GetStringFromName("saveTitle"); // &Save and Quit
			var rep = /(\S+)(\s)(\S+)(\s)(\S+)/;
			var saveAnd = saveAndquit.replace(rep, "$1$2$3"); // &Save and Quit --> &Save and
			var restartBtn = browserBundle.GetStringFromName("safeModeRestartButton");
			/*********************************</Localized strings>*/

			var button0Title = saveAnd + (" ") + restartBtn.toLocaleLowerCase(); // &Save and restart
			var button1Title = quitBundle.GetStringFromName("cancelTitle"); // &Cancel
			var button2Title = "&Restart";
			if(!(/\(&.\)/).test(saveAndquit)) { // Probably it's not an eastern locale.
				button2Title = "&" + restartBtn; // .replace(restartAkey, "&"+restartAkey); // Restart --> &Restart
			} else {
				button2Title = restartBtn + "(&R)";
				button0Title = button0Title + "(&S)";
			};
			if(currentAppLang == "el" && button0Title.startsWith("&")) { // Fix same access key with
			// "Cancel" button, until update of "el" locale.
				var title = button0Title.slice(1);
				button0Title = title.replace(title.charAt(2), title.charAt(2) + "&");
			};
			if(!(/&/).test(button0Title)) { // In some locales the "&" is in the removed part of "Save and Quit".
				button0Title = "&" + button0Title;
			};
			var neverAskLabel = !safeMode ? _RestMan.locals.neverAskLabel : null;
			var neverAskChkbx = {value: false};
			var flags = _RestMan.promptSrvc.BUTTON_TITLE_IS_STRING * _RestMan.promptSrvc.BUTTON_POS_0 +
						_RestMan.promptSrvc.BUTTON_TITLE_IS_STRING * _RestMan.promptSrvc.BUTTON_POS_1 +
						_RestMan.promptSrvc.BUTTON_TITLE_IS_STRING * _RestMan.promptSrvc.BUTTON_POS_2 +
						_RestMan.promptSrvc.BUTTON_POS_0_DEFAULT;

			// We show the restart dialog always in non-private window, to avoid any possible confusions,
			// making some users think that private windows/tabs can be restored after restart.
			// Don't like that much the method below, but anyway it works..
			// A better way to get the most recently used non-private browser window, is wanted :-) We pay good..
			// Unfortunately, "getZOrderDOMWindowEnumerator()" does not work on Linux/Mac.. - bug 462222.
			// With our method below, in Windows we get always the most recently used non-private browser
			// window (ideal), or the current browser window if there are no private windows or the restart
			// is attempted in a non-private window (ideal).
			// On Linux/Mac, if there are private windows, we get always the most recently opened non-private
			// browser window, if the restart is attempted in a private window (not ideal..). Otherwise, the
			// current browser window (ideal).
			var brWinLinMac = null;
			var mostRecentNonPrvBrowserWindow = {
				checkWinPrivacy: function(aOS) {
					var allBrowserWinEnum = (aOS == "WINNT") ?
						_RestMan.winMed.getZOrderDOMWindowEnumerator("navigator:browser", true) :
						_RestMan.winMed.getEnumerator("navigator:browser"); // Works in "oldest > newest" fashion.
					while(allBrowserWinEnum.hasMoreElements()) {
						var browserWin = allBrowserWinEnum.getNext();
						var nonPrvVisTabCount = 0;
						Array.forEach(browserWin.gBrowser.visibleTabs, function(tabElement) {
							if(!tabElement.hasAttribute("privateTab-isPrivate")) {
							// Compatibility with "Private Tab" extension.
								nonPrvVisTabCount++;
							};
						});
						if(!browserWin.PrivateBrowsingUtils.isWindowPrivate(browserWin) && nonPrvVisTabCount >= 1) {
							if(aOS == "WINNT") {
								return browserWin.focus(); // The most recently used non-private browser window.
							} else {
								brWinLinMac = null;
								brWinLinMac = browserWin; // At the last iteration, "brWinLinMac" will be the
								// most recently opened non-private browser window. We consider this as the best deal,
								// since we can't get the most recently used non-private window in Linux/Mac. The other
								// option is to get the first opened non-private browser window, by "break" here.
							};
						};
					};
				}
			};
			var targetBrWin = {
				getBrWin: function() {
					function nonPrvTab() {
						// for(var tabEl of window.gBrowser.visibleTabs) { // "for...of" unsupported by appVer < 15 :-(
						// 	if(!tabEl.hasAttribute("privateTab-isPrivate")) { // At least 1 non-private tab.
						// 		return true;
						// 	};
						// };
						var tabs = window.gBrowser.visibleTabs;
						for(var nTab = 0; nTab < tabs.length; ++nTab) {
							if(!tabs[nTab].hasAttribute("privateTab-isPrivate")) { // At least 1 non-private tab.
								return true;
							};
						};
						return false;
					};
					if(!window.PrivateBrowsingUtils.isWindowPrivate(window) && nonPrvTab()) { // Non-private browser
					// window with at least 1 non-private tab. Compatibility with "Private Tab" extension.
						return window.focus();
					} else {
						if(_RestMan.appInfos.targetOS == "WINNT") {
							return mostRecentNonPrvBrowserWindow.checkWinPrivacy("WINNT");
						} else {
							mostRecentNonPrvBrowserWindow.checkWinPrivacy();
							brWinLinMac.focus();
							return brWinLinMac;
						};
					};
				}
			};

			var userButtonChoice =
				_RestMan.promptSrvc.confirmEx(targetBrWin.getBrWin(), restartDlgTitle, dlgMessage,
											   flags, button0Title, button1Title, button2Title,
											  neverAskLabel, neverAskChkbx);
			switch(userButtonChoice) {
				case 2: // Save session & restart, no session restore.
					if(neverAskChkbx.value) {
						_RestMan.rmPrefBranch.setBoolPref("askOnRestart", false);
						_RestMan.rmPrefBranch.setBoolPref("restoreSession_onRestart", false);
						_RestMan.rmPrefBranch.setBoolPref("warnOnClose_multipleTabs", true);
					};
					if((aParam == "restartOther") || (aParam == "restartOtherQuit")) {
						return RestMan_RestManOnRestartRequest("restart", "doRestartNoRestore", undefined, aParam);
					} else {
						!safeMode ? RestMan_RestManOnRestartRequest("restart", "doRestartNoRestore") :
									RestMan_RestManOnRestartRequest("restart", "doRestartNoRestoreSM");
					};
					break;
				case 1: // Do nothing, close dialog.
					break;
				case 0: // Save session, restart & restore session.
					if(neverAskChkbx.value) {
						_RestMan.rmPrefBranch.setBoolPref("askOnRestart", false);
						_RestMan.rmPrefBranch.setBoolPref("restoreSession_onRestart", true);
					};
					if((aParam == "restartOther") || (aParam == "restartOtherQuit")) {
						return RestMan_RestManOnRestartRequest("restart", "doRestart", undefined, aParam);
					} else {
						!safeMode ? RestMan_RestManOnRestartRequest("restart", "doRestart") :
									RestMan_RestManOnRestartRequest("restart", "doRestartSM");
					};
					break;
			};
			if(((aParam == "restartOther") || (aParam == "restartOtherQuit")) && (userButtonChoice == 1)) {
				return false;
			};
		};

		function restartDlg3to19(aParam) {
			// In  pm/ffx 3-19, we call the built-in restart dialog of these versions.
			// Nothing special here, other than that we patch the message of the dialog,
			// adding a string in the end, to make it be a question to match with
			// "Do not ask next time" label in its check box (_RestManOnLoad function).
			// Moz-devs should do something like this, instead of removing this dialog..
			_RestMan.obsSrvc.notifyObservers(cancelQuit, "quit-application-requested", "restart");
			if(cancelQuit.data || !permitWinUnload()) {
				return false; // The quit request has been canceled.
			};
			_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_session_once", true);
			if(_RestMan.locals.invalCaches) {
				if(_RestMan.appInfos.appVer >= 4) {
					_RestMan.appInfo.QueryInterface(ci.nsIXULRuntime).invalidateCachesOnRestart();
				} else {
					var cacheFiles = ["compreg.dat", "xpti.dat", "extensions.cache", "extensions.ini"]; // "extensions.rdf", "pluginreg.dat"
					cacheFiles.forEach(function(name) {
						var profDirFiles = _RestMan.dirSrvc.get("ProfD", ci.nsIFile);
						if(profDirFiles) {
							profDirFiles.append(name);
						};
						if(profDirFiles.exists()) {
							profDirFiles.remove(false);
						};
					});
				};
			};
			if((_RestMan.appInfos.appVer >= 4) && ((aParam == "restartOther") || (aParam == "restartOtherQuit"))) { // If "InvalidateCachesOnRestart"
			// is enabled, this disables it. We do it on restart in other profile since the current profile exits.
				_RestMan.restmanSrvc.getSet("restartRequested", "setF"); // set false.
			};
			if((_RestMan.appInfos.appVer >= 3.5) && _RestMan.locals.clrMnrOnRestart) { // Clear "MOZ_NO_REMOTE" on restart.
				if(_RestMan.restmanSrvc.getSet("startedWithNoRemote", "getV")) { // get value.
					var getInfo = undefined;
					try {
						getInfo = new CustomEvent("getInfo"); // "CustomEvent()" constructor, appVer ~11+.
					} catch(ex) {
						getInfo = window.document.createEvent("Event");
						getInfo.initEvent("getInfo", true, true);
					}; // If the profile has been started with "-no-remote" parameter, we clear the "MOZ_NO_REMOTE"
					// environment variable on restart if this is the only running app app instance.
					if(_RestMan.restmanProfMan(getInfo)[1] < 2) { // Number of profiles in use.
						if(_RestMan.locals.clrMnrOnProfile == 0) {
							_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
						} else if((_RestMan.locals.clrMnrOnProfile == 1) && (aParam == "restart")) {
							_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
						} else if((_RestMan.locals.clrMnrOnProfile == 2) && ((aParam == "restartOther") || (aParam == "restartOtherQuit"))) {
							_RestMan.restmanSrvc.getSet("clearMozNoRemote", "setT"); // set true.
						};
					};
				};
			};
			var quitType = (aParam !== "restartOtherQuit") ? ci.nsIAppStartup.eAttemptQuit | ci.nsIAppStartup.eRestart : ci.nsIAppStartup.eAttemptQuit;
			_RestMan.appStartup.quit(quitType);
			window.setTimeout(function onQuit3to19() {
				if(window._RestMan && _RestMan.winTabCounter("getBrWinCount") > 0) {
					// If we are still alive, someone has aborted the quit command..
					_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_session_once", false);
				};
			}, 1500);
			if((aParam == "restartOther") || (aParam == "restartOtherQuit")) {
				return true;
			};
		};

		function restartDlg1v5to2() {
			// A simple restart dialog for pm/ffx 1.5 & 2, similar to this one in pm/ffx 3-19.
			if(countsRst[2] < 2) { // allTabCount[2].
				RestMan_RestManOnRestartRequest("restart", "doRestart");
			} else {
				var dlgMessage = _RestMan.locals.restartMessage;
				if(_RestMan.appInfos.appVer < 2) {
				// Pm/ffx 1.5 can't restore the session after restart. For this, we show also the
				// message "You are about to close xx open tabs."
					var tabsBundle = _RestMan.strings.createBundle("chrome://global/locale/tabbrowser.properties");
					var closeTabsMsg =  tabsBundle.formatStringFromName("tabs.closeWarningMultiple",
						[countsRst[2]], 1).match(/^[^\.\,\?\!]+[\.\,\?\!]/); // Does not work in some eastern languages..
					if(closeTabsMsg != null) {
						dlgMessage = closeTabsMsg + "\n\n" + _RestMan.locals.restartMessage + "\n";
					};
				};
				var neverAskChkbx = {value: false};
				var flags = _RestMan.promptSrvc.BUTTON_TITLE_YES    * _RestMan.promptSrvc.BUTTON_POS_0 +
							_RestMan.promptSrvc.BUTTON_TITLE_CANCEL * _RestMan.promptSrvc.BUTTON_POS_1 +
							_RestMan.promptSrvc.BUTTON_POS_0_DEFAULT;
				var mostRecentBrowserWindow = _RestMan.winMed.getMostRecentWindow("navigator:browser");
				var userButtonChoice = _RestMan.promptSrvc.confirmEx(
					mostRecentBrowserWindow, null, dlgMessage, flags,
					null, null, null, _RestMan.locals.neverAskLabel, neverAskChkbx
					);
				switch(userButtonChoice) {
					case 1: // Do nothing, close dialog.
						break;
					case 0: // Restart.
						if(neverAskChkbx.value) {
							_RestMan.rmPrefBranch.setBoolPref("askOnRestart", false);
						};
						RestMan_RestManOnRestartRequest("restart", "doRestart");
						break;
				};
			};
		};

		function showRestartDlg(aParam) {
			if(_RestMan.appInfos.appVer >= 20) {
				return restartDlg20plus(undefined, aParam);
			} else if((_RestMan.appInfos.appVer < 20) && (_RestMan.appInfos.appVer >= 3)) {
				return restartDlg3to19(aParam);
			} else if(_RestMan.appInfos.appVer < 3) {
				restartDlg1v5to2();
			};
		};
	},

	instantDlg: function(b, t) { // Workaround to bring in foreground a non-focusable dialog, because of a child modal dialog
	// over it. We cause the detected dialog to open an other self-closeable and invisible modal dialog, which steals the
	// modality and brings the detected dialog in foreground. A bit hackish method, but it's harmless and most importantly,
	// it works at least on all major platforms of Windows/Linux/Mac OS.
		var dlgUri = "data:application/vnd.mozilla.xul+xml," +
			window.encodeURIComponent("<?xml version='1.0'?>\n\
				<?xml-stylesheet href='chrome://global/skin/' type='text/css'?>\n\
				<dialog xmlns='http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul'\n\
				buttons=','\n\
				hidden='true'\n\
				style='display: none; background: transparent;'\n\
				onload='this.close();'></dialog>");
		var targetDialog = _RestMan.temps.openedDialog;
		if(t) {
			targetDialog = t;
		};
		targetDialog.openDialog(dlgUri, "_blank", "chrome,dialog,titlebar=no,centerscreen,modal");
		targetDialog.focus(); // It's not really necessary, just in case..
		if(b) {
			_RestMan.blinker(true, 0);
		};
	},

	blinker: function RestMan_blinker(on, blinks, stl, trg) { // A simple flip-flop highlighting functionality to better attract
	// user's attention, mainly when the detected dialog is already in foreground/fully visible. It is called when our feature
	// "Detection of opened dialogs" is enabled and a dialog has been detected on restart/exit/closing of last browser window.
		if(!(_RestMan.temps.openedDialog.document instanceof XULDocument) || // appVer < 15.
		   !(_RestMan.temps.openedDialog.document.defaultView instanceof ChromeWindow)) { // appVer 15+.
			_RestMan.temps.rejectRequest = false;
			_RestMan.temps.openedDialog = null;
			return; // In case the target is closed while blinker is still running..
		};
		if(blinks < 5 && on) {
			if(blinks == 0) {
				if(_RestMan.temps.openedDialog.document.getElementById("alertNotification")) {
					trg = _RestMan.temps.openedDialog.document.getElementById("alertBox");
				} else {
					trg = _RestMan.temps.openedDialog.document.documentElement;
				};
				if(trg.hasAttribute("style")) { // At the end of iterations, remove the attribute 
				// "style" or only our css properties.
					stl = true;
				};
				// Set this property only once, at first invocation of "blinker".
				trg.style.setProperty("outline-offset", "-4px", "important");
				if(_RestMan.appInfos.appVer < 3) {
					_RestMan.temps.openedDialog.unStyle = function() { // If the target is closed while blinker
					// is still running, it retains the css properties we add, in dialogs with predefined
					// "style" attribute ("alertNotification > alertBox" doesn't have). This oldie..
						this.removeEventListener("unload", _RestMan.temps.openedDialog.unStyle, false);
						if(stl) {
							this.document.documentElement.style.removeProperty("outline-offset");
							this.document.documentElement.style.removeProperty("outline");
						};
					};
					_RestMan.temps.openedDialog.addEventListener("unload", _RestMan.temps.openedDialog.unStyle, false);
				};
			};
			trg.style.setProperty("outline", "4px solid red", "important");
			blinks++;
			window.setTimeout(RestMan_blinker, 150, false, blinks, stl, trg);
		} else {
			trg.style.removeProperty("outline");
			if(blinks == 5) {
				if(!stl) {
					trg.removeAttribute("style");
				} else {
					trg.style.removeProperty("outline-offset");
				};
				if(_RestMan.appInfos.appVer < 3) {
					_RestMan.temps.openedDialog.removeEventListener("unload", _RestMan.temps.openedDialog.unStyle, false);
					delete _RestMan.temps.openedDialog.unStyle;
				};
				_RestMan.temps.openedDialog = null;
				_RestMan.temps.rejectRequest = false;
				return;
			};
			window.setTimeout(RestMan_blinker, 150, true, blinks, stl, trg);
		};
	},

	doForceRestart: function(aEvent) { // It is called by our toolbar button only. Details for this feature
	// can be found at the end of "_RestManOnLoad()" and in "doRestartNoRestoreCmd()".
		if(!aEvent || (aEvent && (aEvent.button !== 0 || aEvent.type !== "mousedown")) || _RestMan.appInfos.appVer < 4 ||
		   !_RestMan.restmanSrvc.getSet("allowSpecials", "getV")) { // get value.
			return;
		};
		var timeout = null;
		var delayPref = _RestMan.rmPrefBranch.getIntPref("forceRestart_delay"); // Default: 2500 ms.
		var delay = 500; // Limitation of minimum delay to "delay" ms, for safety/usability reasons.
		if(delayPref > delay) {
			delay = delayPref;
		};
		var restManBtn = window.document.getElementById("restman-button");
		timeout = window.setTimeout(function forceRestartCmd() {
			restManBtn.removeEventListener("mouseout", timeoutCancel, false);
			restManBtn.removeEventListener("mouseup", timeoutCancel, false);
			restManBtn.removeAttribute("command"); // Will be restored if the attempt to "force restart" fail.
			restManBtn.removeAttribute("oncommand");
			restManBtn.setAttribute("onmousedown", ""); // Must be overwritten first, before its removal.
			restManBtn.removeAttribute("onmousedown"); // Don't attempt a "force restart" second time, if the first failed.
			timeout = null;
			_RestMan._RestManOnRestartRequest("restart", "forceRestart");
		}, delay);
		function timeoutCancel() {
			window.clearTimeout(timeout);
			timeout = null;
			restManBtn.removeEventListener("mouseout", timeoutCancel, false);
			restManBtn.removeEventListener("mouseup", timeoutCancel, false);
		};
		restManBtn.addEventListener("mouseout", timeoutCancel, false);
		restManBtn.addEventListener("mouseup", timeoutCancel, false);
	},

	restmanProfMan: function(aEvent) {
		if(!aEvent || (((aEvent.type == "popupshowing") || (aEvent.type == "popuphidden")) && (aEvent.target.id !== "restman-contextMenu")) ||
		  ((aEvent.type == "command") && (aEvent.target.parentNode.id !== "profMenuList1") && (aEvent.target.parentNode.id !== "profMenuList2") &&
		  (aEvent.target.id !== "rm-rProfman") && (aEvent.target.id !== "rm-sProfman")) || (_RestMan.appInfos.appVer < 3.5)) {
			return;
		};
		var scriptLoader = cc["@mozilla.org/moz/jssubscript-loader;1"].getService(ci.mozIJSSubScriptLoader);
		var sub = {profMenuList1: window.document.getElementById("profMenuList1"),
				   profMenuList2: window.document.getElementById("profMenuList2")};
		scriptLoader.loadSubScript("chrome://restartmanager/content/restmanProfMan.js", sub, "UTF-8");
		switch(aEvent.type) {
			case "popupshowing":
				sub.profileListCreate(true);
				break;
			case "popuphidden":
				sub.profileListDestroy(0, sub.profMenuList1);
				break;
			case "command":
				sub.profRestartToOrStart(aEvent);
				break;
			case "getInfo": // Custom event.
				return sub.profileListCreate(false);
		};
	},

	profileNameContext: function(aParam, aValue) {
		if(aParam == true) {
			var rcpnContext = window.document.getElementById("rcpnContext");
			var currentPrefs = {
				icon: _RestMan.rmPrefBranch.getBoolPref("profileName_icon"),
				bold: _RestMan.rmPrefBranch.getBoolPref("profileName_bold"),
				ital: _RestMan.rmPrefBranch.getBoolPref("profileName_italic"),
				color: _RestMan.rmPrefBranch.getCharPref("profileName_color")
			};
			rcpnContext.childNodes[3].checked = currentPrefs.icon;
			rcpnContext.childNodes[2].checked = currentPrefs.bold;
			rcpnContext.childNodes[1].checked = currentPrefs.ital;
			if(currentPrefs.color) {
				rcpnContext.childNodes[0].color = currentPrefs.color;
			} else {
				rcpnContext.childNodes[0].color = window.getComputedStyle(rcpnContext.parentNode.childNodes[1], null).color;
			};
			rcpnContext.setAttribute("setup", true);
			return;
		};
		if(["icon", "bold", "ital", "color"].indexOf(aParam) > -1) {
			_RestMan.applyUISettings(); // Set our pref observer in stand-by mode.
			switch(aParam) {
				case "icon":
					window.document.getElementById("rcpNameIcon").valueFromPreferences = aValue;
					break;
				case "bold":
					window.document.getElementById("rcpNameBold").valueFromPreferences = aValue;
					break;
				case "ital":
					window.document.getElementById("rcpNameItal").valueFromPreferences = aValue;
					break;
				case "color":
					window.document.getElementById("rcpNameColor").valueFromPreferences = aValue;
					break;
			};
		};
	},

	onBrowserWinClose: function RestMan_onBrowserWinClose(aEvent) { // On closing a browser window by the (X) button.
		if(_RestMan.temps.rejectRequest) {
			return false;
		};
		if(_RestMan.locals.codeMonitoring) {
			var suppressCodeMon = {value: false};
			_RestMan.promptSrvc.alertCheck(null, "RestMan_onBrowserWinClose caller:",
				RestMan_onBrowserWinClose.caller.name + "\n\n" + "Passed parameter: " + aEvent.type + " event",
				"Suppress codeMon for this window", suppressCodeMon);
			if(suppressCodeMon.value) {
				_RestMan.locals.codeMonitoring = false;
			};
		};
		var noPrivacy = _RestMan.winTabCounter("checkNoPrivacy");
		if(noPrivacy == true) { // Non-private window.
			_RestMan.onQuitRequest("patchOnly");
			if(!window.WindowIsClosing()) { // In case of warning dialog, "WindowIsClosing()" (browser.js)
				// function returns true only if the dialog is accepted and the rest of its checks were passed.
				return false;
			} else {
				if((_RestMan.brPrefBranch.getIntPref("startup.page") == 3) || // No default dialog on last window close.
				   (_RestMan.temps.restoreUserSet == 1) || (_RestMan.temps.restoreUserSet == 2)) { // "startup.page" should be 3 but currently is 1.
					_RestMan.winTabCounter("brHandlesClosing", aEvent); // Ability to show our dialog (optional).
				} else { // Closing a non-private browser window, if there is a private tab (using "Private Tab" extension) with "onbeforeunload"
					// dialog in its content, the dialog appears twice on "Leave Page". The listener is added on first "Leave Page" to fix this issue.
					window.addEventListener("beforeunload", function(event) {
						event.stopPropagation();
					}, true);
				};
				return true;
			};
		} else if(noPrivacy == false) { // The window is private or all of its tabs are private.
			var showQuitWarning = _RestMan.brPrefBranch.getBoolPref("showQuitWarning");
			var currentSetting = false;
			if(_RestMan.appInfos.appVer < 21 && showQuitWarning) { // If the quit dialog is enabled, appVer 20 shows it on
			// attempt to close private windows, even if they have 1 tab. If the dialog is accepted, the private
			// window gets closed but leaves the browser in a cripple state: When the time to exit the browser comes,
			// the (expected) quit dialog does not appear.. Obviously this is a bug.
				currentSetting = true;
				_RestMan.brPrefBranch.setBoolPref("showQuitWarning", false);
			};
			var winTabCnt = _RestMan.winTabCounter("getWinTabCount"); // [allBrWinCount, allTabCountCW].
			if(winTabCnt[0] == 1 && winTabCnt[1] == 1 && window.WindowIsClosing()) { // browser.js
				if(showQuitWarning || currentSetting) { // Somewhat arbitrary setting, assuming the users who
				// have enabled the quit dialog, most likely in most cases they want the last non-private session
				// (if any) to being restored automatically in next startup, when exiting from private browsing
				// mode where this dialog is not shown.
					_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_session_once", true);
				};
				if(_RestMan.locals.codeMonitoring && !_RestMan.restmanSrvc.getPrvBrowsingAutostart()) {
					window.addEventListener("unload", function onlastBrWinClose() {
						window.removeEventListener("unload", onlastBrWinClose, false);
						_RestMan.promptSrvc.alert(null, "Last private browser window:",
												"The last private browser window is unloaded." +
												"\nThe session of the last non-private window has been saved.");
					}, false);
				};
				if(_RestMan.temps.restoreUserSet !== 0) {
					var ss = cc["@mozilla.org/browser/sessionstore;1"].getService(ci.nsISessionStore);
					if(!ss.canRestoreLastSession) {
						_RestMan.onQuitRequest("restUserSet");
					};
				};
				return true;
			} else if(winTabCnt[0] > 1 && winTabCnt[1] == 1 && window.WindowIsClosing()) { // allBrWinCount[0]/allTabCountCW[1].
 				if(currentSetting) { // Restore user's setting.
					_RestMan.brPrefBranch.setBoolPref("showQuitWarning", true);
				};
				return true;
			} else if(winTabCnt[1] > 1) { // allTabCount[1].
				aEvent.preventDefault();
				window.focus();
				_RestMan.winTabCounter("rmHandlesClosing");
				if(currentSetting) { // Restore user's setting.
					_RestMan.brPrefBranch.setBoolPref("showQuitWarning", true);
				};
				return false;
			};
		};
		return false; // "noPrivacy" = undefined, when "detectOpenDlgs" = true and a dialog has been detected.
	},	

	onCmdBrowserWinClose: function RestMan_onCmdBrowserWinClose(aEvent) { // On closing a browser window with Ctrl+Shift+W.
		if(_RestMan.locals.codeMonitoring) {
			var suppressCodeMon = {value: false};
			_RestMan.promptSrvc.alertCheck(null, "RestMan_onCmdBrowserWinClose caller:",
				RestMan_onCmdBrowserWinClose.caller.name + "\n\n" + "Passed parameter: " + aEvent.type + " event",
				"Suppress codeMon for this window", suppressCodeMon);
			if(suppressCodeMon.value) {
				_RestMan.locals.codeMonitoring = false;
			};
		};
		var noPrivacy = _RestMan.winTabCounter("checkNoPrivacy");
		if(noPrivacy == true) { // Non-private window.
			_RestMan.onQuitRequest("patchOnly");
			if(window.WindowIsClosing()) { // In case of warning or exit dialog, "WindowIsClosing()"
			// (browser.js) function returns true only if the dialog is accepted.
				_RestMan.winTabCounter("brHandlesClosing", aEvent);
			};
		} else if(noPrivacy == false) { // The window is private or all of its tabs are private.
			var currentSetting = false; // Comment above, in "onBrowserWinClose".
			if(_RestMan.appInfos.appVer < 21 && _RestMan.brPrefBranch.getBoolPref("showQuitWarning")) {
				currentSetting = true;
				_RestMan.brPrefBranch.setBoolPref("showQuitWarning", false);
			};
			_RestMan.winTabCounter("rmHandlesClosing");
			if(currentSetting) { // Restore user's setting.
				_RestMan.brPrefBranch.setBoolPref("showQuitWarning", true);
			};
		} else { // "noPrivacy" = undefined, when "detectOpenDlgs" = true and a dialog has been detected.
			return; // Not actually required, just for the comment above.
		};
	},

	onQuitRequest: function RestMan_onQuitRequest(aParam, aEvent, aCaller) {
		var param1 = aParam ? aParam : "none";
		var param2 = aEvent ? aEvent.type : "none";
		var callerName = null; // This function is called also by our component, which works in strict mode.
		// We get the caller of this function as follows, because the "function.caller" will not be
		// allowed in strict mode soon.
		if(aCaller && aCaller == "rro") {
			callerName = "RestMan_RestartRequestObserver";
		} else {
			callerName = RestMan_onQuitRequest.caller.name;
		};
		if(_RestMan.locals.codeMonitoring) {
			var suppressCodeMon = {value: false};
			_RestMan.promptSrvc.alertCheck(null, "RestMan_onQuitRequest caller:",
				callerName + "\n\n" + "Passed parameters: " + "\naParam: " + param1 + "\naEvent: " + param2,
				"Suppress codeMon for this window", suppressCodeMon);
			if(suppressCodeMon.value) {
				_RestMan.locals.codeMonitoring = false;
			};
		};
		if((aParam == "restUserSet") || (aParam == "handleExit") || (aParam == "handleCloseLastWin") ||
		  (aParam == "restartOther") || (aParam == "restartOtherQuit")) {
			onQuitCmd(aParam);
			return;
		};
		function onQuitCmd(aParam) {
			var param = aParam ? aParam : "none";
			if(_RestMan.locals.codeMonitoring) {
				var suppressCodeMon = {value: false};
				_RestMan.promptSrvc.alertCheck(null, "onQuitCmd caller:",
					onQuitCmd.caller.name + // arguments.callee.caller
					"\n\n" + "Passed parameter: " + param,
					"Suppress codeMon for this window", suppressCodeMon);
				if(suppressCodeMon.value) {
					_RestMan.locals.codeMonitoring = false;
				};
			};
			if(aParam == "restUserSet") {
				restoreUserSettingsCmd();
				return;
			};
			function restoreUserSettingsCmd() {
				// We restore here user settings if necessary (if we have changed them before "restart no restore").
				// We could do it easily at next startup, but there is a small issue: (Note: value = value of "browser.startup.page" pref).
				// After our "restart no restore", there is a difference between "value = 1" and "value = 3": If the user
				// decide to "Restore Previous Session", in first case (value = 1) the session is restored as usual,
				// while in second case (value = 3) the initial tab is not overwritten but is merged with the
				// restored session.
				// This is because the condition where, "value = 3" and the option "Restore Previous Session" be available,
				// is not included in pm's design.
				// With value = 0 there is no difference, but however, we use "value = 1" because this is pm's default. With
				// this value we have also the convenience of "Restore Previous Session" button in "about:home" page, if this has
				// been set as the default home page.
				// Also, when "value = 3", there are no dialogs on quit). If we set "value = 1", causes the quit or warning dialog
				// to be shown on quit in some app versions. So, for an exact simulation of "value = 3", we also suppress temporarily
				// all pm's dialogs, by setting "warnOnQuit = false", if this is not already set by the user as a personal
				// preference, until next quit or "Restore Previous Session".
				// In case of "Restore Previous Session", there is no need any more restoration of user settings to be pending.
				// So, this function is called also immediately after "Restore Previous Session".
				var smStartupType = 0;
				if(_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).prefHasUserValue("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup")) {
					// "Session Manager" extension, installed/active or not. Cheap yet reliable way to check only
					// what we are interested for..
					smStartupType = _RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup");
				};
				if(_RestMan.temps.restoreUserSet == 1) {
					_RestMan.brPrefBranch.setIntPref("startup.page", 3);
					_RestMan.brPrefBranch.setBoolPref("warnOnQuit", true); // pm's default.
				} else if(_RestMan.temps.restoreUserSet == 2) {
					_RestMan.brPrefBranch.setIntPref("startup.page", 3);
				};
				if(_RestMan.locals.codeMonitoring) {
					var setByUser = "";
					if(_RestMan.temps.restoreUserSet == 2) {
						setByUser = " - (set by the user)";
					};
					_RestMan.promptSrvc.alert(null, "Restore startup/warnOnQuit settings:",
						"browser.startup.page: " + _RestMan.brPrefBranch.getIntPref("startup.page") +
						"\nbrowser.warnOnQuit: " + _RestMan.brPrefBranch.getBoolPref("warnOnQuit") +
						setByUser);
				};
				_RestMan.rmPrefBranch.setIntPref("restore_UserSettings", 0);
				_RestMan.temps.restoreUserSet = 0;
				if(smStartupType !== 0) { // "Session Manager" observes "browser.startup.page" and if is changed,
				// it will reset its pref. So we have to restore it, after we restore "browser.startup.page".
					_RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).setIntPref("extensions.{1280606b-2510-4fe0-97ef-9b5a22eafe30}.startup", smStartupType);
				};
			};

			if((aParam == "handleExit") || // Passed by our js component on "File --> Exit", in private browsing mode.
			   (aParam == "handleCloseLastWin") || // On closing the last browser window which is private or all of its tabs are private.
			   (aParam == "restartOther") || (aParam == "restartOtherQuit")) { // Restarting to other profile (from private browsing mode here) means exiting the current.
				if(_RestMan.brPrefBranch.getBoolPref("showQuitWarning")) { // Somewhat arbitrary setting, assuming the users who
				// have enabled the quit dialog, most likely in most cases they want the last non-private session (if any) to being
				// restored automatically in next startup, when exiting from private browsing mode where this dialog is not shown.
					_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_session_once", true);
				};
				if((aParam !== "restartOther") && (aParam !== "restartOtherQuit")) {
					_RestMan.sessionSaveHelper(aParam);
				};
			} else if(aParam == "closeWin") { // On closing the last browser window with Ctrl+Shift+W which is not private.
				window.addEventListener("beforeunload", function(event) { // Being here, "WindowIsClosing()" (browser.js) has
				// already returned "true". However, in case of "onbeforeunload" dialog, a long standing bug in all versions
				// of pm 2-28, causes the dialog to appear twice on "Leave Page".
					event.stopPropagation();
				}, true);
				window.close();
			};
		};

		var loopCount = 1; // Normally, the dialog is detected instantly, but let's stay in the safe
						   // side, taking this precautionary measure for possible infinite loops..
		var detectOpenDlgsPref = true;
		function onQuitDlg(aParam) {
			if(!_RestMan.locals.detectOpenDlgs) {
				detectOpenDlgsPref = false;
				_RestMan.locals.detectOpenDlgs = true;
			};
			_RestMan.timer.initWithCallback(function() {
				var targetDlg = null;
				if(this._RestMan.dialogDetector("onExit") && this._RestMan.temps.openedDialog.
					document.documentElement.getAttribute("windowtype") == "dialog:commonDialog") {
					targetDlg = this._RestMan.temps.openedDialog;
					if(aParam) {
						targetDlg.addEventListener("dialogaccept", function onDialogaccept(){
								onQuitCmd();
								return;
						}, false);
						targetDlg.addEventListener("dialogextra1", function onDialogextra1(){
								onQuitCmd();
								return;
						}, false);
					};
					if((countsQt[0] - countsQt[3]) >= 1 || countsQt[5] >= 1) { // allTabCount[0], nonPrvTabCount[3]/privWinCount[4].
						var restmanBundle = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
						// "restmanBundle_fbk": Fallback for missing strings of incomplete locales, to get them from "en-US".
						var restmanBundle_fbk = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
						var dlgExtraMessage = "";
						if((countsQt[0] - countsQt[3]) == 0) {
							try {
								dlgExtraMessage = restmanBundle.formatStringFromName("prvWinWarning.label", [countsQt[5]], 1);
							} catch(ex) {
								dlgExtraMessage = restmanBundle_fbk.formatStringFromName("prvWinWarning.label", [countsQt[5]], 1);
							};
						} else {
							try {
								dlgExtraMessage = restmanBundle.formatStringFromName("prvTabsWarning.label", [countsQt[0] - countsQt[3]], 1);
							} catch(ex) {
								dlgExtraMessage = restmanBundle_fbk.formatStringFromName("prvTabsWarning.label", [countsQt[0] - countsQt[3]], 1);
							};
						};
						var patchTxt = targetDlg.document.createTextNode("\n" + dlgExtraMessage);
						function patcher() {
							var dlgMessage = targetDlg.document.getElementById("info.body");
							if(dlgMessage) {
								var pn = dlgMessage.parentNode;
								var cln = dlgMessage.cloneNode(true);
								cln.insertBefore(patchTxt, cln.firstChild.nextSibling);
								dlgMessage.parentNode.removeChild(dlgMessage);
								pn.appendChild(cln);
							}; // By doing directly "dlgMessage.appendChild(patchTxt)", does not work always correctly
							   // in some app versions, appending "patchTxt" as first child instead of last. Strange..
							targetDlg.sizeToContent();
						};
						if(targetDlg.document.readyState == "complete") {
							patcher();
						} else {
							targetDlg.addEventListener("readystatechange", function(event) {
								if(event.target.readyState == "complete") {
									patcher();
								};
							}, true);
						};
					};
					if(_RestMan.locals.codeMonitoring) {
						_RestMan.promptSrvc.alert(null, "Dialog detection:",
						"Dialog: " + targetDlg.document.documentElement.getAttribute("windowtype") +
									  "\nDetected in " + loopCount + " attempts");
					};
					if(!detectOpenDlgsPref) {
						_RestMan.locals.detectOpenDlgs = false;
					};
					_RestMan.temps.openedDialog = null;
					return;
				} else if(!_RestMan.temps.openedDialog && loopCount < 20) {
					loopCount++;
					onQuitDlg(); // arguments.callee
				} else if(!_RestMan.temps.openedDialog && loopCount == 20) {
					// The dialog is not detected for some reason..
					if(!detectOpenDlgsPref) {
						_RestMan.locals.detectOpenDlgs = false;
					};
					if(aParam) {
						onQuitCmd();
					};
					return;
				};
			}.bind(this), 25, ci.nsITimer.TYPE_ONE_SHOT);
		};

		var countsQt = _RestMan.winTabCounter("getCountsQt");
		// [allTabCount, visTabCount, allPinnedTabs, nonPrvTabCount, allBrWinCount, privWinCount].
		
		if(aParam == "patchOnly" && (callerName == "RestMan_onBrowserWinClose" || callerName == "RestMan_onCmdBrowserWinClose")) {
			if(countsQt[4] == 1 && (countsQt[0] - countsQt[2]) > 1 && _RestMan.brPrefBranch.getBoolPref("warnOnQuit") &&
				_RestMan.brPrefBranch.getBoolPref("tabs.warnOnClose") &&
				_RestMan.brPrefBranch.getIntPref("startup.page") !== 3) {
				onQuitDlg(false);
			};
			return;
		};
		
		// We are here by our observer, on exiting pm via menu "File --> Exit",
		// on non-private browser mode, so maybe we have pm's quit dialog.
		// Trying to determine if we'll have the dialog:
		if((countsQt[0] - countsQt[2]) < 2 || // allTabCount[0]/allPinnedTabs[2].
			_RestMan.brPrefBranch.getIntPref("startup.page") == 3 ||
			_RestMan.brPrefBranch.getBoolPref("sessionstore.resume_session_once") ||
			!_RestMan.brPrefBranch.getBoolPref("warnOnQuit") ||
			!_RestMan.brPrefBranch.getBoolPref("showQuitWarning") ||
				// No pm's quit dialog. On exiting, if there are only private windows and
				// >= 2 tabs, we show our warning dialog if this is selected (default)
				// as we do also in our restart process from private browsing mode.
				// Pm should also show a similar (optional) warning dialog, because
				// private windows can not be restored after closing them..
			callerName == "RestMan_winTabCounter") {
				// If above, we are here on closing the last non-private browser window via close
				// button or Ctrl+Shift+W. If there was a dialog, it has been already accepted.
			if(_RestMan.locals.codeMonitoring) {
				_RestMan.promptSrvc.alert(null, "No pm's quit or close dialog is expected", "All tabs: "
										  + countsQt[0] + "\nVisible tabs: " + countsQt[1]);
			};

			// If this is enabled, we show an exit warning in all cases where pm does not do it by design.
			// This is for those users who want to prevent unintentional app exits.
			if((countsQt[0] - countsQt[2]) > 1 && _RestMan.locals.alwaysWrnOnExit && // allTabCount[0]/allPinnedTabs[2].
				_RestMan.brPrefBranch.getBoolPref("tabs.warnOnClose") &&
				(_RestMan.brPrefBranch.getBoolPref("warnOnQuit") || (_RestMan.temps.restoreUserSet == 1)) &&
				((aParam == "checkShowWarn") || (aEvent && ((_RestMan.brPrefBranch.getIntPref("startup.page") == 3) ||
				(_RestMan.temps.restoreUserSet == 1) || (_RestMan.temps.restoreUserSet == 2))))) {
				if(_RestMan._RestManOnRestartRequest(undefined, "showWarnDlg") == 1) { // Used as a callback here.
					if(aEvent) {
						if(aEvent.type == "close") {
							aEvent.preventDefault();
							return;
						} else if(aEvent.type == "command") {
							return;
						};
					} else if(aParam == "checkShowWarn") {
						return "cancelReq";
					};
				} else if(aEvent) {
					if((_RestMan.temps.restoreUserSet == 1) || (_RestMan.temps.restoreUserSet == 2)) { // "startup.page"
					// should be 3 but currently is 1 (1 or 2). "warnOnQuit" should be "true" but currently is "false" (1).
						_RestMan.brPrefBranch.setBoolPref("sessionstore.resume_session_once", true);
					};
					if(aEvent.type == "close") { // On closing the last non-private browser window, if there is a private
					// tab (using "Private Tab" extension) with "onbeforeunload" dialog in its content, the dialog appears
					// twice on "Leave Page". The listener is added on first "Leave Page" to fix this issue.
						window.addEventListener("beforeunload", function(event) {
							event.stopPropagation();
						}, true);
					};
				};
			};
			if(aEvent && aEvent.type == "command") {
			// On closing the last non-private browser window with Ctrl+Shift+W.
				onQuitCmd("closeWin");
			} else {
				onQuitCmd();
			};
		} else { // We expect pm's quit dialog.
			if(_RestMan.locals.codeMonitoring) {
				var suppressCodeMon = {value: false};
				_RestMan.promptSrvc.alertCheck(null, "Pm's quit dialog is expected", "All tabs: "
										  + countsQt[0] + "\nVisible tabs: " + countsQt[1],
										  "Suppress codeMon for this window", suppressCodeMon);
					if(suppressCodeMon.value) {
						_RestMan.locals.codeMonitoring = false;
					};
				};
			onQuitDlg(true);
		};
	},

	optionsDlgOnLoad: function() {
		_RestMan.setLogger();
		var browserWin = _RestMan.winMed.getMostRecentWindow("navigator:browser");
		var winOpenerType = null;
		if(window.opener && window.opener.document.documentElement.hasAttribute("windowtype")) {
			winOpenerType = window.opener.document.documentElement.getAttribute("windowtype");
			if(winOpenerType == "navigator:browser") {
				browserWin = window.opener;
			};
		};
		var isChromeWin = (window instanceof ci.nsIDOMChromeWindow) ? true : false;
		function winHider() {
			window.document.documentElement.removeAttribute("persist");
			window.document.documentElement.removeAttribute("minwidth");
			window.document.documentElement.style.visibility = "collapse";
			window.onfocus = function() {window.moveTo(0, -60);};
		};
		if(!browserWin || !isChromeWin) {
			window.document.title = "\u0020‎";
			window.document.documentElement.setAttribute("onbeforeunload", "");
			window.document.documentElement.removeAttribute("onbeforeunload"); // Must be overwritten first, before its removal (previous line), appVer 27+.
			var nTabs = 0;
			if(browserWin) {
				nTabs = browserWin.gBrowser.browsers.length;
				window.document.documentElement.style.visibility = "hidden";
				if(nTabs > 1) {
					window.onunload = function() {
						browserWin.setTimeout(browserWin._RestMan.launchOptionsDlg, 350);
					};
				} else {
					window.onpagehide = function() {
						browserWin.setTimeout(browserWin._RestMan.launchOptionsDlg, 350);
					};
				};
			} else {
				winHider();
				window.onunload = function() {_RestMan.promptSrvc.alert(null, "Restart manager",
					"A browser window is required to be open for the preferences\nwindow of Restart manager to work properly");};
			};
			if(nTabs > 1) {
				window.setTimeout(window.close, 0); // Actually, closes the current tab where our prefs was tried to load.
			} else {
				browserWin.gBrowser.loadURI("about:blank");
			};
			return; // We don't support opening of our prefs in a tab. Additionally, we need a browser
		}; // window to be present at the moment of opening of our prefs and as long as it stays open.

		if((!window.arguments || !window.arguments[0]) &&
		  (winOpenerType !== "Addons:Manager") && // appVer 4+.
		  (winOpenerType !== "Extension:Manager")) { // appVer < 4.
			var rmPrefWinEnum = _RestMan.winMed.getEnumerator("restman:options");
			while(rmPrefWinEnum.hasMoreElements()) {
				var rmPrefWin = rmPrefWinEnum.getNext();
				if((rmPrefWin !== window) && !rmPrefWin.closed) {
					winHider();
					rmPrefWin.focus();
					window.close();
					return;
				}; // Ensuring only one instance of our pref window, regardless of how it is opened.
			};
		};
		_RestMan.sharedResources(true); // In the scope of our pref window. Important!
		browserWin2 = browserWin; // restmanOptions.js
		winOpenerType2 = winOpenerType; // restmanOptions.js
		function windowClose() {
			window.close();
		};
		browserWin.addEventListener("unload", windowClose, false);
		window.addEventListener("unload", function onPrefsUnload() {
			browserWin.removeEventListener("unload", windowClose, false);
			window.removeEventListener("unload", onPrefsUnload, false);
		}, false);
		var selectedAppLocale = _RestMan.prefSrvc.QueryInterface(ci.nsIPrefBranch).getCharPref("general.useragent.locale");
		var appLang = browserWin._RestMan.appInfos.appLang;
		var rmVer = browserWin._RestMan.locals.restmanVer;
		if(appLang !== selectedAppLocale) {
			_RestMan.refreshPrefs(); // In the scope of browser window(s). See at the end of this document.
		};
		var prefWin = window.document.getElementById("restman_Options");
		var rmStrings = window.document.getElementById("rm_strset");
		// "rmStrings_fbk": Fallback for potentially missing file of incomplete locales, to use "en-US" file.
		var rmStrings_fbk = window.document.getElementById("rm_strset_fbk");
		var extensionsBundle = _RestMan.strings.createBundle("chrome://mozapps/locale/extensions/extensions.properties");
		var askCheckbox = window.document.getElementById("askCheckbox");
		var warnPrvChkbx = window.document.getElementById("warnOnClosePrvChkbox");
		var iconCheckbox = window.document.getElementById("iconCheckbox");
		var shortcutVisState = window.document.getElementById("shortcutVis-state");
		var shortcutState = window.document.getElementById("shortcut-state");
		var restmanVersion = window.document.getElementById("restmanVersion");
		var hChkbx = window.document.getElementById("highlight");
		var hChkbxLabel = window.document.getElementById("hiddenLabel1").value;

		// Language resources are cached in some app versions (4-6..?) and don't change if browser's locale is
		// changed, until restart. We need to know the language of our pref window and browser always, when our
		// pref window is opening.
		var chromeRegSrvc = cc["@mozilla.org/chrome/chrome-registry;1"].getService(ci.nsIXULChromeRegistry);
		var currentAppLang = chromeRegSrvc.getSelectedLocale("browser");

		// We use this method to avoid calling "nsIXULChromeRegistry" once more.
		var prefWinCurrentLocale = "en-US";
		try {
			prefWinCurrentLocale = rmStrings.getString("currentLocale-tag");
		} catch(ex) {
			prefWinCurrentLocale = rmStrings_fbk.getString("currentLocale-tag");
		};

		restmanVersion.setAttribute("value", extensionsBundle.formatStringFromName("aboutWindowVersionString", [rmVer], 1));
		if(!(/^\d/).test(restmanVersion.value)) {
			restmanVersion.value = restmanVersion.value.replace(/^\S/, restmanVersion.value.charAt(0).toLocaleUpperCase()); // version XXX --> Version XXX.
		};

		if(_RestMan.appInfos.appVer >= 3.5) {
			prefWin.setAttribute("title", _RestMan.locals.optionsTitle);
		} else if(_RestMan.appInfos.appVer < 3.5) {
			if(prefWin) { // Only this method works in pm 1.5, 2 & 3.
				window.document.title = _RestMan.locals.optionsTitle;
			};
		};
		window.document.getElementById("confirmCap").label = _RestMan.locals.confirmLabel;
		window.document.getElementById("restart-cap").label = _RestMan.locals.restartLabel;
		if((currentAppLang == "fr" && _RestMan.appInfos.appVer >= 4) || currentAppLang == "hu" || currentAppLang == "nl" || currentAppLang == "tr") {
			var secondStr = hChkbxLabel.replace(/^\S+\s/,""); // All highlight --> highlight.
			var cap = secondStr.charAt(0).toLocaleUpperCase(); // h --> H.
			hChkbx.label = secondStr.replace(/^\S/,cap); // highlight --> Highlight.
		} else {
			hChkbx.label = hChkbxLabel.replace(/ +\S+/,""); // Highlight All --> Highlight.
		};
		if(_RestMan.appInfos.appVer >= 23) { // The label "Do not ask next time" in pm >= 23 has access
			// key with ampersand ("&"), recognized only by prompt service. Lets try to remove it..
			if((/(^|[^&])&([^&])/).test(_RestMan.locals.neverAskLabel)) {
				askCheckbox.label = RegExp.leftContext + RegExp.$1 + RegExp.$2 + RegExp.rightContext;
			} else {
				askCheckbox.label = _RestMan.locals.neverAskLabel;
			};
		} else {
			askCheckbox.label = _RestMan.locals.neverAskLabel;
		};

		if((prefWinCurrentLocale !== "en-US") && (prefWinCurrentLocale !== "en-GB")) {
		// Set unique access keys for controls which don't have. In "en-US/en-GB" locales are used the default
		// access keys, in "restmanOptions.xul".
 			function setAccesskeys() {
				askCheckbox.setAttribute("accesskey", rmStrings.getString("askCheckbox_ak"));
				if(warnPrvChkbx) { // appVer >= 20.
					warnPrvChkbx.setAttribute("accesskey", rmStrings.getString("warnPrvChkbx_ak"));
					// Required action for access key to become visible.
					warnPrvChkbx.parentNode.insertBefore(warnPrvChkbx, warnPrvChkbx.nextSibling);
				};
				iconCheckbox.setAttribute("accesskey", rmStrings.getString("iconCheckbox_ak"));
				shortcutVisState.setAttribute("accesskey", rmStrings.getString("shortcutVisState_ak"));
				shortcutState.setAttribute("accesskey", rmStrings.getString("shortcutState_ak"));
				hChkbx.setAttribute("accesskey", rmStrings.getString("hChkbx_ak"));
				
				// Required actions for access keys to become visible.
				askCheckbox.parentNode.insertBefore(askCheckbox, askCheckbox.nextSibling);
				iconCheckbox.parentNode.insertBefore(iconCheckbox, iconCheckbox.nextSibling);
				shortcutVisState.parentNode.insertBefore(shortcutVisState, shortcutVisState.nextSibling);
				shortcutState.parentNode.insertBefore(shortcutState, shortcutState.nextSibling);
				hChkbx.parentNode.insertBefore(hChkbx, hChkbx.nextSibling);
			};

			// We'll be broken here if we don't do the following check and the applied locale does not
			// have these property keys for some reason.
			var pks = ["askCheckbox_ak", "warnPrvChkbx_ak", "iconCheckbox_ak",
					   "shortcutVisState_ak", "shortcutState_ak", "hChkbx_ak"];
			var matches = 0;
			var strEnum = rmStrings.strings;
			while(strEnum.hasMoreElements() && matches < 6) {
				var prop = strEnum.getNext().QueryInterface(ci.nsIPropertyElement);
				for(var p = 0; p < pks.length; p++) {
					if(prop.key == pks[p]) {
						matches++;
						if(matches === 6) {
							setAccesskeys();
							break;
						};
						pks.splice(p, 1);
						continue;
					};
				};
			};
		};

		if(_RestMan.appInfos.appVer >= 6) {
			window.document.getElementById("btnStyle-label").value = _RestMan.locals.appearanceLabel;
		};
		if(_RestMan.appInfos.appVer >= 4) {
			Array.forEach(window.document.getElementsByClassName("restman-radioButton"),
				function(inEachBtn) {
					inEachBtn.setAttribute("iconStyle", _RestMan.locals.iconStyle);
					if(_RestMan.locals.invCachHghlght) {
						inEachBtn.setAttribute("highlightBtn", true);
					};
			});
			window.document.getElementById("restman-radioBtn2").setAttribute("iconStyle", _RestMan.locals.iconStyle);
			if(_RestMan.locals.invCachHghlght) {
				window.document.getElementById("restman-radioBtn2").setAttribute("highlightBtn", true);
			};
			window.document.getElementById("invCaches").addEventListener("command", function() {
				window.addEventListener("unload", function() {
					if(_RestMan.rmPrefBranch.getBoolPref("invalidate_caches")) {
						_RestMan.prefSrvc.getBranch("nglayout.").setBoolPref("debug.disable_xul_fastload", true);
						_RestMan.prefSrvc.getBranch("nglayout.").setBoolPref("debug.disable_xul_cache", true);
					} else {
						_RestMan.prefSrvc.getBranch("nglayout.").setBoolPref("debug.disable_xul_fastload", false);
						_RestMan.prefSrvc.getBranch("nglayout.").setBoolPref("debug.disable_xul_cache", false);
					};
				}, false);
			}, false);
		};

		var txtField = window.document.getElementById("shortcut-char");
		if((txtField.value == "") || (txtField.value == " ")) {
			window.setTimeout(function() {
				if(_RestMan.appInfos.appVer >= 13) {
					if(txtField) {
						txtField.reset(); // Restore default.
						txtField.dispatchEvent(new Event("input", {
							view: window, bubbles: true, isTrusted: true}));
						return true;
					};
				} else if(_RestMan.appInfos.appVer < 13) {
					var event = window.document.createEvent("UIEvents");
					event.initUIEvent("input", true, true, window, 1);
					if(txtField) {
						txtField.value = "r"; // Restore default.
						txtField.dispatchEvent(event);
						return true;
					};
				};
			}, 650);
		};

		var incompleteLocalesBundle = _RestMan.strings.createBundle("chrome://restartmanager_infolocales/locale/incompleteLocales.properties");
		try {
			var incompleteLocales = incompleteLocalesBundle.GetStringFromName("incompleteLocales").split(",");
			function chekLoc(pwcLoc) {
				if("includes" in window.Array.prototype) { // Pale Moon 27+, Firefox 43+.
					if(incompleteLocales.includes(pwcLoc)) {
						return true;
					};
				} else for(var loc = 0; loc < incompleteLocales.length; loc++) { // Cheaper than "Array.indexOf()" for small arrays..
					if(pwcLoc == incompleteLocales[loc]) {
						return true;
					};
				};
				return false;
			};
			if(chekLoc(prefWinCurrentLocale) || (prefWinCurrentLocale !== currentAppLang)) {
				window.document.getElementById("infoImg").style.visibility = "visible";
			};
		} catch(ex) {
			_RestMan.logger("Restart manager: The file \"incompleteLocales.properties\" could not be found or is corrupted");
		};

		var localeDir = window.document.getElementById("restmanVersionBox").getAttribute("chromedir");
		if(localeDir == "rtl") {
			var prevBtn = window.document.getElementById("prevCtrl");
			var nextBtn = window.document.getElementById("nextCtrl");
			prevBtn.label = ">"; prevBtn.setAttribute("accesskey", ">");
			nextBtn.label = "<"; nextBtn.setAttribute("accesskey", "<");
			if(_RestMan.appInfos.appVer >= 3) {
				var ideaBtn = window.document.getElementById("iconSel");
				var ideaTxt = ideaBtn.childNodes[0].childNodes[4].getAttribute("tooltiptext").replace(/ ;-\)/,"");
				ideaBtn.childNodes[0].childNodes[4].setAttribute("tooltiptext", "(-; " + ideaTxt);
				var infoImg = window.document.getElementById("infoImg");
				var infoTipTxt = infoImg.getAttribute("tooltiptext").replace(/;-\) /,"");
				infoImg.setAttribute("tooltiptext", "(-;" + infoTipTxt);
			};
		};
	},

	applyUISettings: function(aParam, nkMod) { // It is called when one of our "instant apply" settings
	// is changed by the user, to update all existing browser windows and our pref window.
		var rmPrefObserver = {
			checkObsereverResponseDelay: null,
			checkObsereverResponse: function() {
			// Clicking already selected radio buttons, the pref observer
			// gets registered and stays there in standby.. Don't like this :-)
				this.checkObsereverResponseDelay = window.setTimeout(function() {
					if(_RestMan.appInfos.appVer < 3 && _RestMan.dialogDetector() && _RestMan.locals.codeMonitoring &&
						_RestMan.temps.openedDialog.document.documentElement.getAttribute("windowtype") == "dialog:commonDialog") {
						_RestMan.temps.openedDialog = null;
						checkObsereverResponseDelay = null;
						return this.checkObsereverResponse(); // If code monitoring is enabled, this reset/repeat
						// function is required in pm 1.5 & 2. Otherwise, on timeout expiration, the pref observer will be
						// removed if the first alert ("Registered") is not accepted within the specified timeout.
					};
					if(_RestMan.temps.setPrefObserver) {
						rmPrefObserver.observerUnregister();
						_RestMan.temps.setPrefObserver = false;
					};
				}, 500);
			},
			prefBranch2Observe: _RestMan.rmPrefBranch,
			observerRegister: function() {
				if(!_RestMan.temps.setPrefObserver) {
					if(_RestMan.appInfos.appVer < 13) {
						this.prefBranch2Observe.QueryInterface(ci.nsIPrefBranch2); // Pm < 13.
					};
					this.prefBranch2Observe.addObserver("", this, false);
					_RestMan.temps.setPrefObserver = true;
					if(_RestMan.locals.codeMonitoring) {
						var suppressCodeMon = {value: false};
						var param1 = aParam ? aParam : "none";
						var param2 = nkMod ? nkMod : "none";
						_RestMan.promptSrvc.alertCheck(null, "applyUISettings pref observer:", "Registered"
						+ "\n--------------\nPassed parameters:" + "\nParameter1: " + param1 + "\nParameter2: " + param2,
						"Suppress codeMon for this window", suppressCodeMon);
						if(suppressCodeMon.value) {
							_RestMan.locals.codeMonitoring = false;
						};
					};
					this.checkObsereverResponse();
				};
			},
			observerUnregister: function() {
				if(_RestMan.temps.setPrefObserver) {
					this.prefBranch2Observe.removeObserver("", this);
					_RestMan.temps.setPrefObserver = false;
					rmPrefObserver = null;
					if(_RestMan.locals.codeMonitoring) {
						_RestMan.promptSrvc.alert(null, "applyUISettings pref observer:", "Unregistered");
					};
					var rmPrefWin = _RestMan.winMed.getMostRecentWindow("restman:options");
					if(aParam == "check") {
						if(rmPrefWin && nkMod) {
							rmPrefWin.checkAvailability(nkMod);
						} else if(rmPrefWin) {
							rmPrefWin.checkAvailability();
						};
					};
					if(aParam == "add+check") {
						if(rmPrefWin) {
							rmPrefWin.checkAvailability(aParam);
						};
					};
				};
			},
			observe: function(aSubject, aTopic, aData) {
				if(_RestMan.locals.codeMonitoring) {
					_RestMan.promptSrvc.alert(null, "applyUISettings pref observer, Observe:",
					"Subject: " + aSubject + "\nTopic: " + aTopic + "\nData: " + aData);
				};
				if(aTopic !== "nsPref:changed") {
					rmPrefObserver.observerUnregister();
					return;
				};
				var winEnumerator = _RestMan.winMed.getEnumerator(null); // "null", because we want to be included
																		 // also our options dialog.
				while(winEnumerator.hasMoreElements()) {
					var winDoc = winEnumerator.getNext().document;
					var appMenu = winDoc.getElementById("appmenuPrimaryPane");
					var fileMenu = winDoc.getElementById("menu_FilePopup");
					var appMenuRestartItem = winDoc.getElementById("restman-appMenuItem");
					var fileMenuRestartItem = winDoc.getElementById("restman-fileMenuItem");
					var neighbor = null;
					var appMenuExitItem = winDoc.getElementById("appmenu-quit");
					var fileMenuExitItem = winDoc.getElementById("menu_FileQuitItem");
					var appmenuHistoryRPSItem = winDoc.getElementById("appmenu_restoreLastSession");
					var historyBtnRPSItem = winDoc.getElementById("HMB_historyRestoreLastSession");
					var menuHistoryRPSItem = winDoc.getElementById("historyRestoreLastSession");
					var restartButton = winDoc.getElementById("restman-button");
					var restmanKeyset = winDoc.getElementById("restmanKeyset_keys"); // Our keyset.
					var rmRestartKey = winDoc.getElementById("keyRestmanRestart_KeyCombo"); // Restart shortcut.
					var rmExitKey = winDoc.getElementById("keyRestmanExit_KeyCombo"); // Exit shortcut.
					var rmSessRestKey = winDoc.getElementById("keyRestmanRPS_KeyCombo"); // "Restore Previous Session" shortcut - appVer >= 4.
					var profileName = winDoc.getElementById("restman-profileName"); // Profile name indicator toolbar item - appVer >= 3.5.
					var winDocType = winDoc.documentElement.getAttribute("windowtype");
					function rShortcutSetUp(aElem) { // Constructor
						// Recreates the menu item as it was before its removal, with or without shortcut.
						var newMenuItemId = null;
						if(aElem == "appMenuRestartItem") {
							newMenuItemId = "restman-appMenuItem";
						} else if(aElem == "fileMenuRestartItem") {
							newMenuItemId = "restman-fileMenuItem";
						};
						var newMenuItem = winDoc.createElement("menuitem");
						newMenuItem.id = newMenuItemId;
						newMenuItem.setAttribute("class", "menuitem-iconic");
						newMenuItem.setAttribute("label", _RestMan.locals.restartLabel);
						newMenuItem.setAttribute("accesskey", _RestMan.locals.restartAccKey);
						newMenuItem.setAttribute("command", "cmd_restmanRestartApp");
						if(_RestMan.rmPrefBranch.getBoolPref("menuItemIcon")) {
							newMenuItem.setAttribute("menuicon", true);
						};
						newMenuItem.setAttribute("iconStyle", _RestMan.rmPrefBranch.getIntPref("iconStyle"));
						if(_RestMan.rmPrefBranch.getBoolPref("invalCaches_highlight")) {
							newMenuItem.setAttribute("highlightMenuItem", true);
						};
						if(_RestMan.appInfos.appVer >= 22) {
							newMenuItem.setAttribute("appVer22plus", "");
						}; //
						var addRestartShortcut = _RestMan.rmPrefBranch.getBoolPref("restart_shortcut");
						var restartShortcutVis = _RestMan.rmPrefBranch.getBoolPref("restart_shortcutVis");
						if(addRestartShortcut) {
						// If true, recreates the shortcut as it was before its removal
						// or in accordance the new user settings.
							var rShortcutKeyMod = null;
							if(_RestMan.rmPrefBranch.getCharPref("rShortcutKey_modifier") == "alt-option") {
								rShortcutKeyMod = "alt";
							} else if(_RestMan.rmPrefBranch.getCharPref("rShortcutKey_modifier") == "shift") {
								rShortcutKeyMod = "shift";
							};
							var rShortcutChar = _RestMan.rmPrefBranch.getComplexValue("rShortcut_char", ci.nsISupportsString).data;
							var newRestartShortcut = winDoc.createElement("key");
							newRestartShortcut.id = "keyRestmanRestart_KeyCombo";
							newRestartShortcut.setAttribute("command", "cmd_restmanRestartApp");
							newRestartShortcut.setAttribute("modifiers", ("accel," + rShortcutKeyMod));
							newRestartShortcut.setAttribute("key", rShortcutChar);
							if(!winDoc.getElementById("keyRestmanRestart_KeyCombo")) { // If we don't do this
							// check, the new "key" will be appended twice. This function is called twice,
							// once for app menu item and once for file menu item.
								restmanKeyset.appendChild(newRestartShortcut);
								var restmanKeysetClone = restmanKeyset.cloneNode(true);
								restmanKeyset.parentNode.removeChild(restmanKeyset);
								winDoc.getElementById("main-window").appendChild(restmanKeysetClone); // If we
								// don't do this, the shortcut will not work in all currently opened windows..
							};
							if(restartShortcutVis) {
								newMenuItem.setAttribute("key", "keyRestmanRestart_KeyCombo");
							};
						}; //
						return newMenuItem;
					};
					function exitSesRestScSetUp(aType) { // Constructor
					// Recreates the "Exit" or "Restore Previous Session" shortcut with current parameters.
						var ss = cc["@mozilla.org/browser/sessionstore;1"].getService(ci.nsISessionStore);
						var scKeyMod = null;
						var scChar = null;
						var keyId = null;
						if(aType == "exit") {
							if(_RestMan.rmPrefBranch.getCharPref("exitScKey_modifier") == "shift") {
								scKeyMod = "shift";
							} else if(_RestMan.rmPrefBranch.getCharPref("exitScKey_modifier") == "alt-option") {
								scKeyMod = "alt";
							};
							scChar = _RestMan.rmPrefBranch.getComplexValue("exitShortcut_char", ci.nsISupportsString).data;
							keyId = "keyRestmanExit_KeyCombo";
						} else if(aType == "srest") {
							if(_RestMan.rmPrefBranch.getCharPref("sRestShortcutKey_mod") == "shift") {
								scKeyMod = "shift";
							} else if(_RestMan.rmPrefBranch.getCharPref("sRestShortcutKey_mod") == "alt-option") {
								scKeyMod = "alt";
							};
							scChar = _RestMan.rmPrefBranch.getComplexValue("sesRestShortcut_char", ci.nsISupportsString).data;
							keyId = "keyRestmanRPS_KeyCombo";
						} else if(aType == "exitLin") {
							scKeyMod = "accel";
							scChar = "Q";
							keyId = "keyRestmanExit_KeyCombo";
						};
						var newShortcutKey = winDoc.createElement("key");
						newShortcutKey.id = keyId;
						if(aType == "exit" || aType == "exitLin") {
							if(_RestMan.appInfos.appVer >= 3.5) {
								newShortcutKey.setAttribute("command", "cmd_quitApplication");
							} else {
								newShortcutKey.setAttribute("oncommand", "goQuitApplication();");
							};
						} else if(aType == "srest") {
							newShortcutKey.setAttribute("oncommand", "restoreLastSession();");
						};
						if(aType !== "exitLin") {
							newShortcutKey.setAttribute("modifiers", ("accel," + scKeyMod));
						} else {
							newShortcutKey.setAttribute("modifiers", scKeyMod);
						};
						newShortcutKey.setAttribute("key", scChar);
						if(aType == "srest") {
							if((_RestMan.appInfos.appVer >= 20 && (!ss.canRestoreLastSession
								|| winDoc.defaultView.PrivateBrowsingUtils.isWindowPrivate(winDoc.defaultView)))
								|| (_RestMan.appInfos.appVer < 20 && (cc["@mozilla.org/privatebrowsing;1"].getService
								(ci.nsIPrivateBrowsingService).privateBrowsingEnabled || !ss.canRestoreLastSession))) {
								newShortcutKey.setAttribute("disabled", true);
							} else if(winDoc.defaultView._RestMan.temps.rpsObs == null) {
								winDoc.defaultView._RestMan.prevSessionRestObs(true);
							};
						};
						restmanKeyset.appendChild(newShortcutKey);
						var restmanKeysetClone = restmanKeyset.cloneNode(true);
						restmanKeyset.parentNode.removeChild(restmanKeyset);
						winDoc.getElementById("main-window").appendChild(restmanKeysetClone); // If we don't do this,
						// the shortcut will not work in all currently opened windows..
					};

					switch(aData) {
						case "openDlg_detection": // Enable/disable of opened dialogs detection.
							if(winDoc && winDocType == "navigator:browser") {
								var openDlgDetPref = this.prefBranch2Observe.getBoolPref("openDlg_detection");
								if(openDlgDetPref) {
									winDoc.defaultView._RestMan.locals.detectOpenDlgs = true;
								} else {
									winDoc.defaultView._RestMan.locals.detectOpenDlgs = false;
								}; // Update of all existing ChromeWindows.
							};
							break;
						case "menuItemIcon": // Add/remove the icon of our menu items.
							if(winDoc && winDocType == "navigator:browser") {
								var iconOnMenuItems = this.prefBranch2Observe.getBoolPref("menuItemIcon");
								if(!iconOnMenuItems) {
									if(appMenuRestartItem) {
										appMenuRestartItem.removeAttribute("menuicon");
									};
									if(fileMenuRestartItem) {
										fileMenuRestartItem.removeAttribute("menuicon");
									};
								} else {
									if(appMenuRestartItem) {
										appMenuRestartItem.setAttribute("menuicon", true);
									};
									if(fileMenuRestartItem) {
										fileMenuRestartItem.setAttribute("menuicon", true);
									};
								};
							};
							break;
						case "restart_shortcut": // Enable/disable our restart shortcut.
						case "restart_shortcutVis": // Remove/restore the indication of our restart shortcut from restart menu items.
						case "rShortcutKey_modifier": // Set key modifier for our restart shortcut (Alt/Option or Shift).
						case "rShortcut_char": // Set key letter for our restart shortcut.
							if(winDoc && winDocType == "navigator:browser") {
								if(rmRestartKey) {
									restmanKeyset.removeChild(rmRestartKey);
								};
								if(appMenuRestartItem) {
									neighbor = appMenuRestartItem.nextSibling;
									appMenuRestartItem.parentNode.removeChild(appMenuRestartItem);
									appMenu.insertBefore(rShortcutSetUp("appMenuRestartItem"), neighbor);
								};
								if(fileMenuRestartItem) {
									neighbor = fileMenuRestartItem.nextSibling;
									fileMenuRestartItem.parentNode.removeChild(fileMenuRestartItem);
									fileMenuRestartItem = fileMenu.insertBefore(rShortcutSetUp("fileMenuRestartItem"), neighbor); // Refreshes also the variable.
								};
							};
							break;
						case "iconStyle": // Set icon style for our menu items.
							if(winDoc && (winDocType == "navigator:browser" || winDocType == "restman:options")) {
								var iconStylePref = this.prefBranch2Observe.getIntPref("iconStyle");
								if(restartButton) {
									restartButton.setAttribute("iconStyle", iconStylePref);
								};
								if(appMenuRestartItem) {
									appMenuRestartItem.setAttribute("iconStyle", iconStylePref);
								};
								if(fileMenuRestartItem) {
									fileMenuRestartItem.setAttribute("iconStyle", iconStylePref);
								};
								if(_RestMan.appInfos.appVer >= 4) {
									Array.forEach(winDoc.getElementsByClassName("restman-radioButton"),
										function(inEachBtn) {
											inEachBtn.setAttribute("iconStyle", iconStylePref);
									});
									var radioButton2 = winDoc.getElementById("restman-radioBtn2");
									if(radioButton2) {
										radioButton2.setAttribute("iconStyle", iconStylePref);
									};
								};
							};
							break;
						case "invalCaches_highlight": // Highlight/un-highlight all of our restart items.
							if(winDoc && (winDocType == "navigator:browser" || winDocType == "restman:options")) {
								var invalCachesHighlightPref = this.prefBranch2Observe.getBoolPref("invalCaches_highlight");
								if(invalCachesHighlightPref) {
									if(appMenuRestartItem) {
										appMenuRestartItem.setAttribute("highlightMenuItem", true);
									};
									if(fileMenuRestartItem) {
										fileMenuRestartItem.setAttribute("highlightMenuItem", true);
									};
									if(restartButton) {
										restartButton.setAttribute("highlightBtn", true);
									};
									if(_RestMan.appInfos.appVer >= 4) {
										Array.forEach(winDoc.getElementsByClassName("restman-radioButton"),
											function(inEachBtn) {
												inEachBtn.setAttribute("highlightBtn", true);
										});
										var radioButton2 = winDoc.getElementById("restman-radioBtn2");
										if(radioButton2) {
											radioButton2.setAttribute("highlightBtn", true);
										};
									};
								} else {
									if(appMenuRestartItem) {
										appMenuRestartItem.removeAttribute("highlightMenuItem");
									};
									if(fileMenuRestartItem) {
										fileMenuRestartItem.removeAttribute("highlightMenuItem");
									};
									if(restartButton) {
										restartButton.removeAttribute("highlightBtn");
									};
									if(_RestMan.appInfos.appVer >= 4) {
										Array.forEach(winDoc.getElementsByClassName("restman-radioButton"),
											function(inEachBtn) {
												inEachBtn.removeAttribute("highlightBtn");
										});
										var radioButton2 = winDoc.getElementById("restman-radioBtn2");
										if(radioButton2) {
											radioButton2.removeAttribute("highlightBtn");
										};
									};
								};
							};
							break;
						case "buttonStyle": // Set ours or default style for our toolbar button - appVer >= 4.
							if(winDoc && winDocType == "navigator:browser") {
								if(restartButton) {
									var btnStylePref = this.prefBranch2Observe.getIntPref("buttonStyle");
									if(btnStylePref == 1) {
										restartButton.setAttribute("class", "restman-toolbarbutton");
									} else if(btnStylePref == 0) {
										restartButton.setAttribute("class", "toolbarbutton-1");
									};
								};
							};
							break;
						case "allways_smallBtn": // Use small button allways - appVer >= 4.
							if(winDoc && winDocType == "navigator:browser") {
								if(restartButton) {
									if(this.prefBranch2Observe.getBoolPref("allways_smallBtn")) {
										restartButton.setAttribute("smallButton", true);
									} else {
										restartButton.removeAttribute("smallButton");
									};
								};
							};
							break;
						case "exit_shortcut": // Enable/disable our "Exit" shortcut.
						case "exitScKey_modifier": // Set key modifier (Alt/Option or Shift).
						case "exitShortcut_char": // Set key letter in shortcut.
							if(winDoc && winDocType == "navigator:browser") {
								var exitScState = _RestMan.rmPrefBranch.getIntPref("exit_shortcut");
								if(rmExitKey) {
									restmanKeyset.removeChild(rmExitKey);
								};
								if(exitScState == 0) {
									if(_RestMan.appInfos.targetOS == "Linux") { // Recreate our key with built-in key
									// parameters in Linux under our keyset, until next startup.
										exitSesRestScSetUp("exitLin");
									};
									if(appMenuExitItem && appMenuExitItem.hasAttribute("key")) {
										var nd = appMenuExitItem.cloneNode(false);
										nd.removeAttribute("key");
										nd.setAttribute("acceltext", ""); // Required in older app versions.
										if(_RestMan.appInfos.targetOS == "Linux") {
										// Use our key with Linux default shortcut parameters until next startup.
											nd.setAttribute("key", "keyRestmanExit_KeyCombo");
										};
										appMenuExitItem.parentNode.removeChild(appMenuExitItem);
										appMenu.appendChild(nd);
									};
									if(fileMenuExitItem && fileMenuExitItem.hasAttribute("key")) {
										var nd = fileMenuExitItem.cloneNode(false);
										nd.removeAttribute("key");
										nd.setAttribute("acceltext", ""); // Required in older app versions.
										if(_RestMan.appInfos.targetOS == "Linux") {
										// Use our key with Linux default shortcut parameters until next startup.
											nd.setAttribute("key", "keyRestmanExit_KeyCombo");
										};
										fileMenuExitItem.parentNode.removeChild(fileMenuExitItem);
										fileMenu.appendChild(nd);
									};
								} else if(exitScState !== 0) {
									if(exitScState == 2) {// Remove built-in shortcut in Linux.
										var builtInSc = winDoc.getElementById("key_quitApplication");
										if(builtInSc) {
											builtInSc.parentNode.removeChild(builtInSc);
										};
									};
									exitSesRestScSetUp("exit");
 									if(appMenuExitItem) {
										var nd = appMenuExitItem.cloneNode(false);
										nd.removeAttribute("acceltext"); // Required in older app versions.
										nd.setAttribute("key", "keyRestmanExit_KeyCombo");
										appMenuExitItem.parentNode.removeChild(appMenuExitItem);
										appMenu.appendChild(nd);
									};
									if(fileMenuExitItem) {
										var nd = fileMenuExitItem.cloneNode(false);
										nd.removeAttribute("acceltext"); // Required in older app versions.
										nd.setAttribute("key", "keyRestmanExit_KeyCombo");
										fileMenuExitItem.parentNode.removeChild(fileMenuExitItem);
										fileMenu.appendChild(nd);
									};
								};
							};
							break;
						case "sRestore_shortcut": // Enable/disable our "Restore Previous Session" shortcut - appVer >= 4.
						case "sRestShortcutKey_mod": // Set key modifier (Alt/Option or Shift).
						case "sesRestShortcut_char": // Set key letter in shortcut.
							if(winDoc && winDocType == "navigator:browser") {
								if(rmSessRestKey) {
									restmanKeyset.removeChild(rmSessRestKey);
								};
								if(!_RestMan.rmPrefBranch.getBoolPref("sRestore_shortcut")) {
									if(appmenuHistoryRPSItem && appmenuHistoryRPSItem.hasAttribute("key")) {
										var appMenuHist = appmenuHistoryRPSItem.parentNode.id;
										neighbor = appmenuHistoryRPSItem.nextSibling;
										var nd = appmenuHistoryRPSItem.cloneNode(false);
										nd.removeAttribute("key");
										nd.setAttribute("acceltext", ""); // Required in older app versions.
										appmenuHistoryRPSItem.parentNode.removeChild(appmenuHistoryRPSItem);
										winDoc.getElementById(appMenuHist).insertBefore(nd, neighbor);
									};
									if(historyBtnRPSItem && historyBtnRPSItem.hasAttribute("key")) { // appVer >- 26.
										var historyBtn = historyBtnRPSItem.parentNode.id;
										neighbor = historyBtnRPSItem.nextSibling;
										var nd = historyBtnRPSItem.cloneNode(false);
										nd.removeAttribute("key");
										nd.setAttribute("acceltext", ""); // Required till now (app version 28.6).
										historyBtnRPSItem.parentNode.removeChild(historyBtnRPSItem);
										winDoc.getElementById(historyBtn).insertBefore(nd, neighbor);
									};
									if(menuHistoryRPSItem && menuHistoryRPSItem.hasAttribute("key")) {
										var histMenu = menuHistoryRPSItem.parentNode.id;
										neighbor = menuHistoryRPSItem.nextSibling;
										var nd = menuHistoryRPSItem.cloneNode(false);
										nd.removeAttribute("key");
										nd.setAttribute("acceltext", ""); // Required in older app versions.
										menuHistoryRPSItem.parentNode.removeChild(menuHistoryRPSItem);
										winDoc.getElementById(histMenu).insertBefore(nd, neighbor);
									};
									if(winDoc.defaultView._RestMan.temps.restoreUserSet == 0 &&
										winDoc.defaultView._RestMan.temps.rpsObs !== null) {
										winDoc.defaultView._RestMan.temps.rpsObs.remObs();
									};
								} else {
									exitSesRestScSetUp("srest");
 									if(appmenuHistoryRPSItem) {
										var appMenuHist = appmenuHistoryRPSItem.parentNode.id;
										neighbor = appmenuHistoryRPSItem.nextSibling;
										var nd = appmenuHistoryRPSItem.cloneNode(false);
										nd.removeAttribute("acceltext"); // Required in older app versions.
										nd.setAttribute("key", "keyRestmanRPS_KeyCombo");
										appmenuHistoryRPSItem.parentNode.removeChild(appmenuHistoryRPSItem);
										winDoc.getElementById(appMenuHist).insertBefore(nd, neighbor);
									};
									if(historyBtnRPSItem) { // appVer >- 26.
										var historyBtn = historyBtnRPSItem.parentNode.id;
										neighbor = historyBtnRPSItem.nextSibling;
										var nd = historyBtnRPSItem.cloneNode(false);
										nd.removeAttribute("acceltext"); // Required till now (app version 28.6).
										nd.setAttribute("key", "keyRestmanRPS_KeyCombo");
										historyBtnRPSItem.parentNode.removeChild(historyBtnRPSItem);
										winDoc.getElementById(historyBtn).insertBefore(nd, neighbor);
									};
									if(menuHistoryRPSItem) {
										var histMenu = menuHistoryRPSItem.parentNode.id;
										neighbor = menuHistoryRPSItem.nextSibling;
										var nd = menuHistoryRPSItem.cloneNode(false);
										nd.removeAttribute("acceltext"); // Required in older app versions.
										nd.setAttribute("key", "keyRestmanRPS_KeyCombo");
										menuHistoryRPSItem.parentNode.removeChild(menuHistoryRPSItem);
										winDoc.getElementById(histMenu).insertBefore(nd, neighbor);
									};
								};
							};
							break;
						case "profileName_icon": // Profile name indicator item: show/hide the icon.
							if(winDoc && winDocType == "navigator:browser") {
								if(!this.prefBranch2Observe.getBoolPref("profileName_icon")) {
									profileName.setAttribute("icon", false);
								} else {
									profileName.removeAttribute("icon");
								};
							};
							break;
						case "profileName_bold": // Profile name indicator item: "font-weight".
							if(winDoc && winDocType == "navigator:browser") {
								if(this.prefBranch2Observe.getBoolPref("profileName_bold")) {
									profileName.childNodes[1].style.fontWeight = "bold";
								} else {
									profileName.childNodes[1].style.removeProperty("font-weight");
								};
							};
							break;
						case "profileName_italic": // Profile name indicator item: "font-style".
							if(winDoc && winDocType == "navigator:browser") {
								if(this.prefBranch2Observe.getBoolPref("profileName_italic")) {
									profileName.childNodes[1].style.fontStyle = "italic";
								} else {
									profileName.childNodes[1].style.removeProperty("font-style");
								};
							};
							break;
						case "profileName_color": // Profile name indicator item: color.
							if(winDoc && winDocType == "navigator:browser") {
								var rcpnColor = this.prefBranch2Observe.getCharPref("profileName_color");
								if(rcpnColor) {
									profileName.childNodes[1].style.color = rcpnColor;
								} else {
									profileName.childNodes[1].style.removeProperty("color");
								};
							};
							break;
					};
				};
				window.clearTimeout(this.checkObsereverResponseDelay);
				this.checkObsereverResponseDelay = null;
				this.observerUnregister();
			}
		};
		if(aParam == "checkObserverState") {
			if(_RestMan.temps.setPrefObserver) {
				if(_RestMan.locals.codeMonitoring) {
					_RestMan.locals.codeMonitoring = false;
					rmPrefObserver.observerUnregister();
				} else {
					rmPrefObserver.observerUnregister();
				};
			};
			_RestMan.applyUISettings = null;
			return;
		} else {
			rmPrefObserver.observerRegister();
		};
	},

	animBtnIcon: function() {
		if(_RestMan.appInfos.appVer >= 4) {
			var targetBtn = window.document.getElementById("restman-button");
			if(!targetBtn) {
				return;
			};
			if(targetBtn.hasAttribute("rotateIcon")) { // Prevent retriggering of animation while it is running.
				return;
			};
			var animDelay = window.setTimeout(function() {
								targetBtn.setAttribute("rotateIcon", true);
								animReset();
								targetBtn.removeEventListener("mouseout", animCancel, false);
							}, 550); // Time for "mouseover" on the button to trigger the animation.
			function animCancel() {
				window.clearTimeout(animDelay);
				targetBtn.removeEventListener("mouseout", animCancel, false);
			};
 			targetBtn.addEventListener("mouseout", animCancel, false);
			function animReset() { // We could listen for the "transitionend" event, but this is
			// dispatched also on button press, interrupting the animation. So, we use a timer instead..
				window.setTimeout(function() {
					targetBtn.removeAttribute("rotateIcon");
				}, 1550);
			};
   		};
	},

	launchOptionsDlg: function() {
		if(_RestMan.dialogDetector() &&
			_RestMan.temps.openedDialog.document.documentElement.getAttribute("windowtype") == "restman:options") {
			try { // Necessary in appVer < 3. It throws exception if the dialog
			// is non-focusable, because of a child modal dialog over it.
				_RestMan.temps.openedDialog.focus();
			} catch(ex) {
				if(ex) {
					_RestMan.instantDlg(false); // Comment in "instantDlg()".
				};
			};
			if(_RestMan.appInfos.appVer >= 3) {
				var dlg = _RestMan.temps.openedDialog;
				window.setTimeout(function() { // "hasFocus()" is not that reliable, especially on Linux..
					if(!dlg.document.hasFocus()) {
						_RestMan.instantDlg(false, dlg); // Comment in "instantDlg()".
					};
				}, 150);
			};
			_RestMan.temps.openedDialog = null;
			return;
		};
		var instantApply = _RestMan.brPrefBranch.getBoolPref("preferences.instantApply");
		var winLinuxMac = !instantApply ? ",dialog,modal" : ",dialog=no";
		window.openDialog("chrome://restartmanager/content/restmanOptions.xul","_blank",
			"chrome,titlebar,centerscreen,buttons='accept,extra1'" + winLinuxMac, true);
	},

	refreshPrefs: function() { // Refreshes our prefs and resources: 1.- Always after our pref window is closed.
	// 2.- When our pref window is opening, if browser's language has been changed but it hasn't been restarted yet.
		var allBrWinEnum = _RestMan.winMed.getEnumerator("navigator:browser");
		while(allBrWinEnum.hasMoreElements()) {
			var inEachWin = allBrWinEnum.getNext();
			if(inEachWin._RestMan) {
				inEachWin._RestMan.sharedResources(true);
			};
		};
	}
};

/*
*********This is the end...,*******my only friend...,**********the end...***************************************
** This small project was a high school for me, to learn things :-)
** I got the idea for it when mozilla decided to remove the (very useful to me) restart dialog in Firefox 20..
****************************************************************************************************************
*/

